Package: domir
Title: Tools to Support Relative Importance Analysis
Version: 1.2.0
Date: 2024-5-4
Authors@R: 
    person(given = "Joseph",
           family = "Luchman",
           role = c("aut", "cre"),
           email = "jluchman@gmail.com",
           comment = c(ORCID = "0000-0002-8886-9717"))
Description: Methods to apply decomposition-based relative importance 
  analysis for R functions. This package supports the application of 
  decomposition methods by providing 'lapply'- or 'Map'-like meta-functions that 
  compute dominance analysis (Azen, R., & Budescu, D. V. (2003) 
  <doi:10.1037/1082-989X.8.2.129>; Grömping, U. (2007) 
  <doi:10.1198/000313007X188252>) an extension of Shapley value regression 
  (Lipovetsky, S., & Conklin, M. (2001) <doi:10.1002/asmb.446>)
  based on the values returned from other functions.
Imports: parallel, stats, utils
Suggests: dplyr, dominanceanalysis, forcats, Formula, ggplot2, knitr,
        lme4, parameters, performance, pscl, purrr, relaimpo, rlang,
        rmarkdown, stringr, systemfit, testthat (>= 3.0.0), tidyr
License: GPL (>= 3)
URL: https://github.com/jluchman/domir,
        https://jluchman.github.io/domir/
BugReports: https://github.com/jluchman/domir/issues
Encoding: UTF-8
RoxygenNote: 7.3.1
Config/testthat/edition: 3
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2024-05-04 23:09:05 UTC; josephluchman
Author: Joseph Luchman [aut, cre] (<https://orcid.org/0000-0002-8886-9717>)
Maintainer: Joseph Luchman <jluchman@gmail.com>
Repository: CRAN
Date/Publication: 2024-05-04 23:20:02 UTC
Built: R 4.2.3; ; 2024-05-05 00:46:01 UTC; unix

Package: domir
Title: Dominance Analysis Methods
Version: 1.3.0
Date: 2026-4-19
Authors@R: 
    person(given = "Joseph",
           family = "Luchman",
           role = c("aut", "cre"),
           email = "jluchman@gmail.com",
           comment = c(ORCID = "0000-0002-8886-9717"))
Description: Dominance analysis relative importance methods that are intended 
    for predictive models.
Imports: parallel, stats, utils
Suggests: dplyr, dominanceanalysis, forcats, Formula, ggplot2, knitr,
        lme4, parameters, performance, pscl, purrr, relaimpo, rlang,
        rmarkdown, stringr, systemfit, testthat (>= 3.0.0), tidyr
License: GPL (>= 3)
URL: https://github.com/jluchman/domir,
        https://jluchman.github.io/domir/
BugReports: https://github.com/jluchman/domir/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/testthat/edition: 3
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2026-04-19 15:08:02 UTC; josephluchman
Author: Joseph Luchman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8886-9717>)
Maintainer: Joseph Luchman <jluchman@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-19 15:20:02 UTC
Built: R 4.6.0; ; 2026-04-25 18:21:34 UTC; unix

Package: domir
Title: Dominance Analysis Methods
Version: 1.3.0
Date: 2026-4-19
Authors@R: 
    person(given = "Joseph",
           family = "Luchman",
           role = c("aut", "cre"),
           email = "jluchman@gmail.com",
           comment = c(ORCID = "0000-0002-8886-9717"))
Description: Dominance analysis relative importance methods that are intended 
    for predictive models.
Imports: parallel, stats, utils
Suggests: dplyr, dominanceanalysis, forcats, Formula, ggplot2, knitr,
        lme4, parameters, performance, pscl, purrr, relaimpo, rlang,
        rmarkdown, stringr, systemfit, testthat (>= 3.0.0), tidyr
License: GPL (>= 3)
URL: https://github.com/jluchman/domir,
        https://jluchman.github.io/domir/
BugReports: https://github.com/jluchman/domir/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/testthat/edition: 3
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2026-04-19 15:08:02 UTC; josephluchman
Author: Joseph Luchman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8886-9717>)
Maintainer: Joseph Luchman <jluchman@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-19 15:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 16:36:12 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(domir,formula)
S3method(domir,formula_list)
S3method(print,domin)
S3method(print,domir)
S3method(summary,domin)
S3method(summary,domir)
export(domin)
export(domir)
export(fmllst2Fml)
export(formula_list)

Dominance Analysis Methods
================

# domir <img src="man/figures/logo.png" align="right" height="139"/>

[![Stable
version](http://www.r-pkg.org/badges/version-last-release/domir)](https://cran.r-project.org/package=domir)
[![downloads](http://cranlogs.r-pkg.org/badges/grand-total/domir)](https://cran.r-project.org/package=domir)

# Overview

**domir** implements several methods to compute dominance analysis[^1].
Dominance analysis is a relative importance analysis approach that
derives conceptually from Shapley values in that it ascribes ‘values’
from some function to inputs (known as ‘names’ in the package) to that
function.

When applied to predictive models, the method compares components of a
fit metric ascribed to each ‘name’ (i.e., independent variable,
predictor, feature, or parameter estimate) to each other ‘name’ in a
pairwise fashion to determine a hierarchy of dominance or relative
importance.

# Installation

To install the most recent version of **domir** from CRAN use:

`install.packages("domir")`

**domir** is also used as the computational engine underlying the
[`dominance_analysis()`](https://easystats.github.io/parameters/reference/dominance_analysis.html)
function for the
[**parameters**](https://easystats.github.io/parameters/) package in
[**easystats**](https://easystats.github.io/easystats/).

# What **`domir`** Does

`domir` computes dominance analysis results based on a set of
inputs/names and the values returned from a function like this linear
regression model.

`lm(mpg ~ am + vs + cyl, data = mtcars)`

Using the variance explained $R^2$ as fit statistic as implemented by
`lm`’s `summary` method as the returned value, `domir` produces:

``` r
lm_wrapper <-       
  function(formula, data) {
    lm(formula, data = data) |> 
      summary() |>
      _[["r.squared"]]
  }

domir(mpg ~ am + vs + cyl, lm_wrapper, data = mtcars)
```

    ## 
    ## Overall Value:      0.7619773 
    ## 
    ## General Dominance Values:
    ##     General Dominance Standardized Ranks
    ## am          0.1774892    0.2329324     3
    ## vs          0.2027032    0.2660226     2
    ## cyl         0.3817849    0.5010450     1
    ## 
    ## Conditional Dominance Values:
    ##     Include At: 1 Include At: 2 Include At: 3
    ## am      0.3597989     0.1389842   0.033684441
    ## vs      0.4409477     0.1641982   0.002963748
    ## cyl     0.7261800     0.3432799   0.075894823
    ## 
    ## Complete Dominance Proportions:
    ##       > am > vs > cyl
    ## am >    NA  0.5     0
    ## vs >   0.5   NA     0
    ## cyl >  1.0  1.0    NA

`domir` requires a set of inputs/names, submitted as a `formula` or a
specialized
[`formula_list`](https://jluchman.github.io/domir/reference/formula_list.html)
object, and a function that accepts the input/names and returns a
single, numeric value.

The function supplied to `domir` must then be a full ‘analysis pipeline’
function and is necessary for the effective use of `domir`. In fact,
`domir`’s value is in that it allows the use of such pipelines as the
user can define them to apply to almost any predictive model. This
example uses wrapper function, `lm_wrapper`, that accepts a `formula`
and returns the $R^2$. A user could use an anonymous function defined
within the `domir` call that has a similar format as an alternative.

# Comparison with Existing Relative Importance Packages

Several other relative importance packages can produce results identical
to `domir` under specific circumstances. I will focus on discussing two
of the most relevant comparison packages below.

The `calc.relimpo` function in the **relaimpo** package with
`type = "lmg"` produces the general dominance values for `lm` as in the
example below:

``` r
relaimpo::calc.relimp(mpg ~ am + vs + cyl, data = mtcars, type = "lmg")
```

    ## Response variable: mpg 
    ## Total response variance: 36.3241 
    ## Analysis based on 32 observations 
    ## 
    ## 3 Regressors: 
    ## am vs cyl 
    ## Proportion of variance explained by model: 76.2%
    ## Metrics are not normalized (rela=FALSE). 
    ## 
    ## Relative importance metrics: 
    ## 
    ##           lmg
    ## am  0.1774892
    ## vs  0.2027032
    ## cyl 0.3817849
    ## 
    ## Average coefficients for different model sizes: 
    ## 
    ##            1X       2Xs       3Xs
    ## am   7.244939  4.316851  3.026480
    ## vs   7.940476  2.995142  1.294614
    ## cyl -2.875790 -2.795816 -2.137632

**relaimpo** is for importance analysis with linear regression with
variance explained $R^2$ as a fit statistic and is optimized to analyze
that model-fit statistic pairing across multiple ways of submitting data
(i.e., correlation matrices, fitted `lm` object, a `data.frame`).

The `dominanceAnalysis` function in **dominanceAnalysis** produces many
of the same statistics as `domir` as in the example below:

``` r
dominanceanalysis::dominanceAnalysis(lm(mpg ~ am + vs + cyl, data = mtcars))
```

    ## 
    ## Dominance analysis
    ## Predictors: am, vs, cyl 
    ## Fit-indices: r2 
    ## 
    ## * Fit index:  r2 
    ##     complete conditional general
    ## am                              
    ## vs                            am
    ## cyl    am,vs       am,vs   am,vs
    ## 
    ## Average contribution:
    ##   cyl    vs    am 
    ## 0.382 0.203 0.177

**dominanceAnalysis** is for the relative importance of specific
model-fit statistic pairs as it is implemented using S3 methods focused
on model types to implement similar to how
`parameters::dominance_analysis` works but using a custom implementation
not dependent on the **insight** package to parse model components and
implement the methodology.

# Further Examples

Further examples of `domir`s functionality will be populated on the
[**domir** wiki](https://github.com/jluchman/domir/wiki).

[^1]: see this
    [vignette](https://CRAN.R-project.org/package=domir/vignettes/domir_basics.html)
    for a conceptual discussion of dominance analysis

## ----include = FALSE, results='hide'------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
#devtools::load_all()
library(ggplot2)
library(dplyr)
library(purrr)
library(forcats)
library(tidyr)
library(stringr)

## ----setup_lm-----------------------------------------------------------------
library(datasets)

lm_cars <- 
  lm(mpg ~ am + cyl + carb, data = mtcars)

summary(lm_cars)

## ----setup_domir--------------------------------------------------------------
library(domir)

domir(
  mpg ~ am + cyl + carb, 
  function(formula) {
    lm_model <- lm(formula, data = mtcars)
    summary(lm_model)[["r.squared"]]
  }
)

## ----capture_r2s--------------------------------------------------------------
lm_capture <- 
  function(formula, data, ...) { # wrapper program that accepts formula, data, and ellipsis arguments
    count <<- count + 1 # increment counter in enclosing environment
    lm_obj <- lm(formula, data = data, ...) # estimate 'lm' model and save object
    DA_results[count, "formula"] <<- 
      deparse(formula) # record string version of formula passed in 'DA_results' in enclosing environment
    DA_results[count, "R^2"] <<- 
      summary(lm_obj)[["r.squared"]] # record R^2 in 'DA_results' in enclosing environment
    summary(lm_obj)[["r.squared"]] # return R^2
  }

count <- 0 # initialize the count indicating the row in which the results will fill-in

DA_results <- # container data frame in which to record results
  data.frame(formula = rep("", times = 2^3-1), 
             `R^2` = rep(NA, times = 2^3-1), 
             check.names = FALSE)

lm_da <- domir(mpg ~ am + cyl + carb, # implement the DA with the wrapper
               lm_capture, 
               data = mtcars)

DA_results

## ----cpt_am_cyl, echo=FALSE---------------------------------------------------
knitr::kable(
  cbind(DA_results[grepl("am", DA_results$formula) & !grepl("cyl", DA_results$formula) ,],
        DA_results[!grepl("am", DA_results$formula) & grepl("cyl", DA_results$formula) ,]), 
  row.names = FALSE, caption = "Complete Dominance Comparisons: `am` versus `cyl` ", digits = 3)

## ----cpt_am_carb, echo=FALSE--------------------------------------------------
knitr::kable(cbind(DA_results[grepl("am", DA_results$formula) & !grepl("carb", DA_results$formula) ,], DA_results[!grepl("am", DA_results$formula) & grepl("carb", DA_results$formula) ,]), row.names = FALSE, caption = "Complete Dominance Comparisons: `am` versus `carb` ", digits = 3)

## ----cpt_cyl_carb, echo=FALSE-------------------------------------------------
knitr::kable(cbind(DA_results[grepl("cyl", DA_results$formula) & !grepl("carb", DA_results$formula) ,], DA_results[!grepl("cyl", DA_results$formula) & grepl("carb", DA_results$formula) ,]), row.names = FALSE, caption = "Complete Dominance Comparisons: `cyl` versus `carb` ", digits = 3)

## ----lm_complete--------------------------------------------------------------
lm_da$Complete_Dominance

## ----cdl_am, echo=FALSE-------------------------------------------------------
first_order <- 
  cbind(DA_results[DA_results$formula == "mpg ~ am",], data.frame(`formula subtrahend` = "mpg ~ 1", `R^2 subtrahend` = 0, difference = DA_results[DA_results$formula == "mpg ~ am", "R^2"], check.names = FALSE))

names(first_order)[1:2] <- c("formula minuend", "R^2 minuend")

second_order <- 
  cbind(DA_results[grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),], 
        DA_results[!grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),], 
        data.frame(difference = 
                     DA_results[grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"] - DA_results[!grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),"R^2"]))

names(second_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

third_order <- 
  cbind(DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),],
        DA_results[!grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),],
        data.frame(difference = 
                     DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),"R^2"] - DA_results[!grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"]))

names(third_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

knitr::kable(first_order, row.names = FALSE, caption = "Conditional Dominance Computations: `am` with One IV/Alone", digits = 3)

knitr::kable(second_order, row.names = FALSE, caption = "Conditional Dominance Computations: `am` with Two IVs", digits = 3)

knitr::kable(third_order, row.names = FALSE, caption = "Conditional Dominance Computations: `am` with Three IVs/Full Model", digits = 3)

## ----cdl_cyl, echo=FALSE------------------------------------------------------
first_order <- 
  cbind(DA_results[DA_results$formula == "mpg ~ cyl",], data.frame(`formula subtrahend` = "mpg ~ 1", `R^2 subtrahend` = 0, difference = DA_results[DA_results$formula == "mpg ~ cyl", "R^2"], check.names = FALSE))

names(first_order)[1:2] <- c("formula minuend", "R^2 minuend")

second_order <- 
  cbind(DA_results[grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),], 
        DA_results[!grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),], 
        data.frame(difference = 
                     DA_results[grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"] - DA_results[!grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),"R^2"]))

names(second_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

third_order <- 
  cbind(DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),],
        DA_results[!grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),],
        data.frame(difference = 
                     DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),"R^2"] - DA_results[!grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"]))

names(third_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

knitr::kable(first_order, row.names = FALSE, caption = "Conditional Dominance Computations: `cyl` with One IV/Alone", digits = 3)

knitr::kable(second_order, row.names = FALSE, caption = "Conditional Dominance Computations: `cyl` with Two IVs", digits = 3)

knitr::kable(third_order, row.names = FALSE, caption = "Conditional Dominance Computations: `cyl` with Three IVs/Full Model", digits = 3)

## ----cdl_carb, echo=FALSE-----------------------------------------------------
first_order <- 
  cbind(DA_results[DA_results$formula == "mpg ~ carb",], data.frame(`formula subtrahend` = "mpg ~ 1", `R^2 subtrahend` = 0, difference = DA_results[DA_results$formula == "mpg ~ carb", "R^2"], check.names = FALSE))

names(first_order)[1:2] <- c("formula minuend", "R^2 minuend")

second_order <- 
  cbind(DA_results[grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),], 
        DA_results[!grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),], 
        data.frame(difference = 
                     DA_results[grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"] - DA_results[!grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),"R^2"]))

names(second_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

third_order <- 
  cbind(DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),],
        DA_results[!grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),],
        data.frame(difference = 
                     DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),"R^2"] - DA_results[!grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"]))

names(third_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

knitr::kable(first_order, row.names = FALSE, caption = "Conditional Dominance Computations: `carb` with One IV/Alone", digits = 3)

knitr::kable(second_order, row.names = FALSE, caption = "Conditional Dominance Computations: `carb` with Two IVs", digits = 3)

knitr::kable(third_order, row.names = FALSE, caption = "Conditional Dominance Computations: `carb` with Three IVs/Full Model", digits = 3)

## ----lm_conditional-----------------------------------------------------------
lm_da$Conditional_Dominance

## ----cdl_am_cyk, echo=FALSE---------------------------------------------------
knitr::kable(data.frame(t(lm_da$Conditional_Dominance[c("am", "cyl"),]), comparison= lm_da$Conditional_Dominance["am",] > lm_da$Conditional_Dominance["cyl",]), caption = "Conditional Dominance Designation: `am` Compared to `cyl`", digits = 3)

## ----cdl_am_carb, echo=FALSE--------------------------------------------------
knitr::kable(data.frame(t(lm_da$Conditional_Dominance[c("am", "carb"),]), comparison= lm_da$Conditional_Dominance["am",] > lm_da$Conditional_Dominance["carb",]), caption = "Conditional Dominance Designation: `am` Compared to `carb`", digits = 3)

## ----cdl_cyl_carb, echo=FALSE-------------------------------------------------
knitr::kable(data.frame(t(lm_da$Conditional_Dominance[c("cyl", "carb"),]), comparison= lm_da$Conditional_Dominance["cyl",] > lm_da$Conditional_Dominance["carb",]), caption = "Conditional Dominance Designation: `cyl` Compared to `carb`", digits = 3)

## ----condit_gph, echo=FALSE---------------------------------------------------
lm_da |> pluck("Conditional_Dominance") |> as_tibble(rownames = "pred") |> pivot_longer(names_to = "ivs", values_to = "stat", cols = starts_with("Inclu")) |> mutate(ivs = fct_relabel(ivs, ~ str_replace(., "_", ": "))) |> ggplot(aes(x = ivs, y = stat, group = pred, color= pred)) + geom_line() + ylab("Conditional Dominance Statistic Value") + xlab("Number of Independent Variables") + labs(color = "Independent\nVariable") + theme_linedraw() + scale_color_viridis_d() 

## ----gen_am, echo=FALSE-------------------------------------------------------
knitr::kable(data.frame(t(as.data.frame(lm_da$Conditional_Dominance["am",])), `general dominance` = lm_da$General_Dominance[["am"]], check.names = FALSE), row.names = FALSE, caption = "General Dominance Computations: `am`", digits = 3)

## ----gen_cyl, echo=FALSE------------------------------------------------------
knitr::kable(data.frame(t(as.data.frame(lm_da$Conditional_Dominance["cyl",])), `general dominance` = lm_da$General_Dominance[["cyl"]], check.names = FALSE), row.names = FALSE, caption = "General Dominance Computations: `cyl`", digits = 3)

## ----gen_carb, echo=FALSE-----------------------------------------------------
knitr::kable(data.frame(t(as.data.frame(lm_da$Conditional_Dominance["carb",])), `general dominance` = lm_da$General_Dominance[["carb"]], check.names = FALSE), row.names = FALSE, caption = "General Dominance Computations: `carb`", digits = 3)

## ----lm_general---------------------------------------------------------------
lm_da$General_Dominance

## ----gen_rank, echo = FALSE---------------------------------------------------
knitr::kable(data.frame(IV = names(lm_da$General_Dominance), `general dominance` = lm_da$General_Dominance, ranks = lm_da$Ranks, check.names = FALSE), row.names = FALSE, caption = "General Dominance Designations", digits = 3)

## ----lm_strongest-------------------------------------------------------------
summary(lm_da)$Strongest_Dominance

#' @title Dominance analysis meta-function that returns scalar
#' @description Internal dominance analysis computation function assuming scalar
#' or vector of length 1 returned value.
#' @param .obj `formula` or `formula_list` processed with `formula_parse()`.
#' @param .fct A `function` or string function name.
#' @param .nms A named list. Groups the results in `.set` and `.wst` together 
#' and is used to determine valid name combinations.
#' @param .val Numeric vector of length 1/scalar. Value associated with all 
#' names included.
#' @param .adj Numeric vector of length 1/scalar. Value associated with no 
#' names included.
#' @param .all Numeric vector of length 1/scalar. Value associated with names 
#' in `.all` included.
#' @param .rev Logical. Reverses interpretation for values that are better 
#' when low.
#' @param .cst Object of class c("SOCKcluster", "cluster") from
#' [`parallel-package`].
#' @param .prg Logical. Shows progress bar.
#' @param .arg List. Additional arguments from `...` to submit to `.fct`.
#' @param .is_fmllst Logical. Changes from `formula` to `formula_list` methods.
#' @return A list of dominance analysis results. To be renamed and used as 
#' `domir`'s list of returned results.
#' @noRd
dominance_scalar <- 
  function(.obj, .fct, .nms, .val, .adj, .all, .rev, .cst, .prg, 
           .arg, .is_fmllst) {
    #print("dominance object") # ~~
    #print(.obj) # ~~
    selector_locations <- 
      as.data.frame(
        matrix(
          Reduce(
            rbind, 
            unlist(format_selector(.obj, .is_fmllst))
          ), ncol = 3, byrow = TRUE, 
          dimnames = list(NULL, c("eq", "elem", "name"))
        )
      )
    selector_locations$eq <- as.numeric(selector_locations$eq)
    selector_locations$elem <- as.numeric(selector_locations$elem)
    #print("selector locations") # ~~
    #print(selector_locations) # ~~
    #print("names that receive value; formatted as list") # ~~
    #print(.nms) # ~~
    wsts <- grepl("^wst", names(.nms))
    wsts <- .nms[wsts]
    sets <- grepl("^set", names(.nms))
    sets <- .nms[sets]
    subset_matrix <- 
      subset_matrix_constructor(.nms, sets, wsts, selector_locations)
    #print("overall subset matrix") # ~~
    #print(subset_matrix) # ~~
    if (.prg) {
      pg_bar <-
        utils::txtProgressBar(min = 0, max = nrow(subset_matrix) - 1, style = 3)
    } else {
      pg_bar <- NULL
    }
    if (.is_fmllst) {
      obtain_value <- obtain_value_fmllst
    } else {
      obtain_value <- obtain_value_fml
    }
    if (is.null(.cst)) {
      value_vector <- 
        sapply(seq_len(nrow(subset_matrix)), obtain_value,
               .obj, .fct, .arg, pg_bar, .val, .adj, .all, selector_locations, 
               subset_matrix)
    } else {
      value_vector <- 
        parallel::parSapply(
          cl = .cst,
          seq_len(nrow(subset_matrix)),
          obtain_value,
          .obj, .fct, .arg, pg_bar, .val, .adj, .all, selector_locations, 
          subset_matrix)
    }
    names(value_vector) <- rownames(subset_matrix)
    #print("R2s") # ~~
    #print(value_vector) # ~~
    conditional_dominance <-
      compute_conditional_dominance(value_vector, subset_matrix, .nms)
    # print("conditional dominance") # ~~
    # print(conditional_dominance) # ~~
    general_dominance <- rowMeans(conditional_dominance)
    # print("general dominance") # ~~
    # print(general_dominance) # ~~
    # print("complete dominance") # ~~
    if (length(wsts) == 0) {
      complete_dominance <-
        compute_complete_dominance(value_vector, subset_matrix, .nms)
    } else {
      complete_dominance <- NULL
    }
    #print(complete_dominance) # ~~
    if (.rev) {
      if (!is.null(complete_dominance)) {
        complete_dominance <- (1 - complete_dominance)
      }
      ranks <- rank(general_dominance, ties.method = "min")
      standard <- 1 - (general_dominance/sum(general_dominance))
    } else {
      ranks <- rank(-general_dominance, ties.method = "min")
      standard <- general_dominance/sum(general_dominance)
    }
    list(
      general = general_dominance,
      conditional = conditional_dominance,
      complete = complete_dominance,
      ranks = ranks,
      standard = standard
    )
  }
#' @title Construct matrix of all valid names
#' @description Internal function to construct a logical matrix where columns 
#' represent names and each row is a different valid combination.
#' @param .nms A named list. Groups the results in `.set` and `.wst` together 
#' and is used to determine valid name combinations.
#' @param .set Subset of `.nms` associated with sets.
#' @param .wst Subset of `.nms` associated with within-group sets.
#' @param .loc The result of `format_selector` applied to all names.
#' @return A logical matrix with column names associated with each name in 
#' `.nms`. 
#' @noRd
subset_matrix_constructor <- function(.nms, .set, .wst, .loc) {
  in_out_constructor <- function(name) {c(FALSE, TRUE)}
  in_out_namelist <- lapply(seq_len(length(.nms)), in_out_constructor)
  subset_matrix <- expand.grid(in_out_namelist, KEEP.OUT.ATTRS = FALSE)
  names(subset_matrix) <- names(.nms)
  # print("initial subset matrix") # ~~
  # print(subset_matrix) # ~~
  if (length(.set) > 0) {
    for (matrix in seq_len(length(.set))) {
      merge_name <- names(.set)[matrix]
      num_names <- sum(sapply(.set[[matrix]], function(elem) length(elem)))
      merge_matrix <- 
        data.frame(
          rbind(
            rep(TRUE, times = num_names + 1),
            rep(FALSE, times = num_names + 1)
          )
        )
      names(merge_matrix) <- 
        c(merge_name, paste(merge_name, ".Var", seq_len(num_names), sep = ""))
      #print(merge_matrix) # ~~
      subset_matrix <- 
        merge(subset_matrix, merge_matrix, by = merge_name)
    }
    subset_matrix <- 
      subset_matrix[-which(names(subset_matrix) %in% names(.set))]
  }
  if (length(.wst) > 0) {
    in_out_namelist_wst <- 
      lapply(
        seq_len(length(.wst)),
        function(elem) lapply(seq_len(length(.wst[[elem]])), in_out_constructor)
      )
    subset_matrices_wst <- 
      lapply(
        in_out_namelist_wst,
        function(elem) expand.grid(elem, KEEP.OUT.ATTRS = FALSE)
      )
    for (wst in seq_len(length(subset_matrices_wst))) {
      names(subset_matrices_wst[[wst]]) <- .wst[[wst]]
    }
    names(subset_matrices_wst) <- names(.wst)
    between_wst_subsets <- lapply(seq_len(length(.wst)), in_out_constructor)
    between_wst_subsets <- 
      expand.grid(between_wst_subsets, KEEP.OUT.ATTRS = FALSE)
    names(between_wst_subsets) <- names(.wst)
    add_subsets <- 
      lapply(
        seq_len(length(subset_matrices_wst)),
        function(matrix) {
          merge_name <- names(.wst)[[matrix]]
          subset_combs <- 
            subset_matrices_wst[[matrix]][
              -c(1,nrow(subset_matrices_wst[[matrix]])),
            ]
          merge_matrix <- 
            data.frame(name = TRUE, subset_combs, check.names = FALSE)
          names(merge_matrix)[1] <- merge_name
          merge_matrix <- 
            merge(between_wst_subsets, merge_matrix, by = merge_name)
          other_wsts <- subset_matrices_wst[-matrix]
          merge_others <- 
            lapply(
              names(other_wsts),
              function(other_wst) {
                other_names <- names(other_wsts[[other_wst]])
                other_df <- as.data.frame(matrix(ncol = length(other_names)))
                names(other_df) <- other_names
                merge_matrix <- 
                  data.frame(merge_matrix, other_df)
                merge_matrix[, other_names] <- merge_matrix[[other_wst]]
                merge_matrix[, other_names]
              }
            )
          if (length(merge_others) > 0) 
            merge_matrix <- cbind(merge_matrix, merge_others)
          merge_matrix
        })
    add_subsets <- do.call("rbind", add_subsets)
    subset_matrix_wst <- merge(subset_matrix, add_subsets)
    wst_subset_merge <- 
      lapply(
        names(subset_matrices_wst),
        function(matrix) {
          cols <- ncol(subset_matrices_wst[[matrix]]) + 1
          temp_mat <- 
            data.frame(matrix(rep(c(FALSE, TRUE), times = cols), ncol = cols))
          names(temp_mat) <- 
            c(matrix, names(subset_matrices_wst[[matrix]]))
          temp_mat
        }
      )
    for (matrix in wst_subset_merge) {
      subset_matrix <- merge(subset_matrix, matrix)
    }
    subset_matrix <- rbind(subset_matrix, subset_matrix_wst)
    wst_locator <- grep("^wst", names(subset_matrix))
    subset_matrix <- subset_matrix[,-wst_locator]
  }
  name_locator <- grep("^var", names(subset_matrix))
  set_locator <- grep("^set", names(subset_matrix))
  names(subset_matrix)[name_locator] <- .nms[grep("^var", names(.nms))]
  names(subset_matrix)[set_locator] <- unlist(.set)
  subset_matrix <- subset_matrix[, .loc$name]
  subset_matrix
}
#' @title Reconstruct formula and submit
#' @description Internal function to reconstruct a `formula` from a 
#' combination of names, submit it to the value generating function, and 
#' collect the returned value.
#' @param subset An integer vector of length 1/scalar.
#' @param .obj `formula` processed with `formula_parse()`.
#' @param .fct A `function` or string function name.
#' @param .arg List. Additional arguments from `...` to submit to `.fct`.
#' @param .prg Object of class "txtProgressBar" from [`utils-package`].
#' @param .val Numeric vector of length 1/scalar. Value associated with all 
#' names included.
#' @param .adj Numeric vector of length 1/scalar. Value associated with no 
#' names included.
#' @param .all Numeric vector of length 1/scalar. Value associated with names 
#' in `.all` included.
#' @param .loc A result of `format_selector` applied to all names.
#' @param .mat A result of `subset_matrix_contructor` applied to all names.
#' @return Numeric vector of length 1/scalar.
#' @noRd
obtain_value_fml <-
  function(subset, .obj, .fct, .arg, .prg, .val, .adj, .all, .loc, .mat) {
    if (!is.null(.prg)) utils::setTxtProgressBar(.prg, subset)
    lgl_vec <- unlist(.mat[subset, ])
    which_change <- which(lgl_vec) 
    select_lgl_change <- .loc$elem[which_change]
    .obj$select_lgl[select_lgl_change] <- TRUE
    if (all(lgl_vec)) {
      return(.val)
    } else if (!any(lgl_vec)) {
      return(.adj + .all)
    } else {
      fml <-
        stats::reformulate(
          c(.obj$rhs_names[.obj$select_lgl], .obj$offset),
          response = .obj$lhs_names,
          intercept = .obj$intercept_lgl
        )
      return(do.call(.fct, append(list(fml), .arg)))
    }
  }
#' @title Reconstruct formula_list and submit
#' @description Internal function to reconstruct a `formula_list` from a 
#' combination of names, submit it to the value generating function, and 
#' collect the returned value.
#' @param subset An integer vector of length 1/scalar.
#' @param .obj `formula_list` processed with `formula_parse()`.
#' @param .fct A `function` or string function name.
#' @param .arg List. Additional arguments from `...` to submit to `.fct`.
#' @param .prg Object of class "txtProgressBar" from [`utils-package`].
#' @param .val Numeric vector of length 1/scalar. Value associated with all 
#' names included.
#' @param .adj Numeric vector of length 1/scalar. Value associated with no 
#' names included.
#' @param .all Numeric vector of length 1/scalar. Value associated with names 
#' in `.all` included.
#' @param .loc A result of `format_selector` applied to all names.
#' @param .mat A result of `subset_matrix_contructor` applied to all names.
#' A logical matrix with column names.
#' @return Numeric vector of length 1/scalar.
#' @noRd
obtain_value_fmllst <-
  function(subset, .obj, .fct, .arg, .prg, .val, .adj, .all, .loc, .mat) {
    if (!is.null(.prg)) utils::setTxtProgressBar(.prg, subset)
    lgl_vec <- unlist(.mat[subset, , drop = TRUE])
    which_change <- which(lgl_vec)
    chosen_selectors <- .loc[which_change, ]
    for (chg in seq_len(nrow(chosen_selectors))) {
      .obj[[chosen_selectors$eq[[chg]]]]$select_lgl[[
        chosen_selectors$elem[[chg]]
      ]] <- TRUE
    }
    if (all(lgl_vec)) {
      return(.val)
    } else if (!any(lgl_vec)) {
      return(.adj + .all)
    } else {
      fmllst <-
        lapply(
          .obj,
          function(elem) {
            varlist <- elem$rhs_names[elem$select_lgl]
            if (length(varlist) == 0) varlist <- "1"
            stats::reformulate(
              c(varlist, elem$offset),
              response = elem$lhs_names,
              intercept = elem$intercept_lgl
            )
          }
        )
      fmllst <- do.call("formula_list", fmllst)
      return(do.call(.fct, append(list(fmllst), .arg)))
    }
  }
#' @title Conditional dominance computation
#' @description Internal function to compute conditional dominance values for 
#' each name at each valid  inclusion precedence position.
#' @param value_vector A numeric vector.
#' @param subset_matrix A result of `subset_matrix_contructor` applied to 
#' all names. A logical matrix with column names.
#' @param .nms A named list. Groups the results in `.set` and `.wst` together 
#' and is used to determine valid name combinations.
#' @return A numeric matrix. Names are associated with rows. Inclusion 
#' precedence positions are associated with columns.
#' @noRd
compute_conditional_dominance <- 
  function(value_vector, subset_matrix, .nms) {
    num_names <- 
      length(unlist(.nms[grep("^wst", names(.nms))])) + 
      length(.nms[grep("^set|^var", names(.nms))])
    conditional_dominance <- matrix(NA, nrow = num_names, ncol = length(.nms))
    m_vector_inclusive <- name_counter(subset_matrix, .nms, inclusive = TRUE)
    m_vector_exclusive <- name_counter(subset_matrix, .nms, inclusive = FALSE)
    name_loc <- 1
    for (name in .nms) {
      in_wst <- all(name %in% unlist(.nms[grep("^wst", names(.nms))]))
      namelist_locs <- which(sapply(.nms, function(elem) all(name %in% elem)))
      namelist <- .nms[namelist_locs]
      if (in_wst) namelist <- unlist(namelist)
      k_vectors <- 
        lapply(
          namelist,
          function(elem) {
            return_vec <- 
              name_counter(subset_matrix, namelist, inclusive = TRUE)
            condit_vec <- 
              (m_vector_inclusive == m_vector_exclusive) | 
              (m_vector_inclusive > 0 & return_vec > 0 & 
                 m_vector_inclusive != m_vector_exclusive) &
                 (return_vec != length(namelist))
            select_vec <- apply(subset_matrix[elem], 1, all)
            return_vec <-return_vec*condit_vec*select_vec
            return_vec
          }
        )
      processed <- 
        lapply(
          k_vectors,
          function(elem) {
            return_list <- 
              list(
                subset_matrix_for_name = subset_matrix[elem > 0, ],
                value_vector_for_name = value_vector[elem > 0],
                k_vector_for_name = elem[elem > 0]
              )
            if (in_wst) {
              add_list <- list(m_vector_for_name = m_vector_inclusive[elem > 0])
            } else {
              add_list <- list(m_vector_for_name = m_vector_exclusive[elem > 0])
            }
            append(return_list, add_list)
          }
        )
      for (var in seq_len(length(processed))) { # by row
        for (inc_seq in seq_len(length(.nms))) { # by column
          select_at_inc <- processed[[var]][["m_vector_for_name"]] == inc_seq
          k_vector_for_name_at_inc_seq <- 
            processed[[var]][["k_vector_for_name"]]
          k_vector_for_name_at_inc_seq <- 
            k_vector_for_name_at_inc_seq[select_at_inc]
          value_vector_for_name_at_inc_seq <- 
            processed[[var]][["value_vector_for_name"]]
          value_vector_for_name_at_inc_seq <- 
            value_vector_for_name_at_inc_seq[select_at_inc]
          weight_vector <- 
            (lfactorial(inc_seq - 1) + lfactorial(length(.nms) - inc_seq) + 
            lfactorial(k_vector_for_name_at_inc_seq - 1) + 
            lfactorial(length(namelist) - k_vector_for_name_at_inc_seq)) - 
            (lfactorial(length(.nms) - 1) + lfactorial(length(namelist)))
          value_with_name <- value_vector_for_name_at_inc_seq*exp(weight_vector)
          subset_matrix_for_name_at_inc_seq <- 
            processed[[var]][["subset_matrix_for_name"]]
          subset_matrix_for_name_at_inc_seq <- 
            subset_matrix_for_name_at_inc_seq[select_at_inc, ]
          increment_rows <- 
            find_increments(
              subset_matrix, 
              subset_matrix_for_name_at_inc_seq, 
              namelist[[var]]
            )
          increment_with_name <- value_vector[increment_rows]*exp(weight_vector)
          if (var > 1 & inc_seq == 1) name_loc <- name_loc + 1
          conditional_dominance[name_loc, inc_seq] <-
            sum(value_with_name - increment_with_name)
        }
      }
      name_loc <- name_loc + 1
    }
    conditional_dominance
}
#' @title Counts valid combinations by inclusion precedence position
#' @description Internal function to count the number of combinations of names
#' that are inclusive of any focal names (i.e., inclusive) or inclusive of all 
#' focal names (i.e., exclusive) for conditional dominance computations.
#' @param subset_matrix A result of `subset_matrix_contructor` applied 
#' to all names. A logical matrix with column names.
#' @param .nms A named list. Groups the results in `.set` and `.wst` together 
#' and is used to determine valid name combinations.
#' @param inclusive Logical. Inclusive or exclusive result.
#' @return Integer vector. One value per row of `subset_matrix`.
#' @noRd
name_counter <- function(subset_matrix, .nms, inclusive) {
  apply(
    subset_matrix, 1, 
    function(row) {
      val <- 
        sapply(
          .nms, 
          if (inclusive) {
            function(elem) any(elem %in% names(subset_matrix)[row])
          } else {
            function(elem) all(elem %in% names(subset_matrix)[row])
          }
        )
      sum(val)
    }
  )
}
#' @title Computes increment to value given name inclusion
#' @description Internal function to find the location of subsets in 
#' `subset_matrix` that contain a combination of names that can be used to 
#' compute an increment beyond a combination of names with a focal namelist.
#' @param subset_matrix A result of `subset_matrix_contructor` applied 
#' to all names. A logical matrix with column names.
#' @param subset_matrix_for_name A logical matrix. Subset of `subset_matrix` 
#' that identifies combinations at a specific inclusion precedence position 
#' where the `namelist` is included.
#' @param namelist A character vector of names. Focal names to identify 
#' increments.
#' @return Integer matrix with two rows. Top row corresponds to `subset_matrix`
#' row index number of combination with `namelist` included. Bottowm row 
#' corresponds to `subset_matrix` index number of combination with `namelist` 
#' excluded. Each column is a distinct valid increment. 
#' @noRd
find_increments <- function(subset_matrix, subset_matrix_for_name, namelist) {
  subset_matrix_for_name_delta <- subset_matrix_for_name
  subset_matrix_for_name_delta[namelist] <- FALSE
  rows_delta <- 
    apply(
      subset_matrix_for_name_delta,
      1,
      function(row1) {
        which(apply(
          subset_matrix,
          1, 
          function(row2) {
            all(row1 == row2)
          }
        ))
      }
    )
  rows_delta
}
#' @title Constructs map of equation, name, and selector location
#' @description Internal function to associate equation or left hand side names 
#' with term or right hand side names and locations in the selector list.
#' @param .obj `formula` or `formula_list` processed with `formula_parse()`.
#' @param .is_fmllst Logical.
#' @return A list with the following elements:
#' \describe{
#' \item{eq}{List index number for the equation or left hand side 
#' of a `formula_list`.}
#' \item{elem}{{List index number for the term or right hand side 
#' of a `formula_list`.}
#' \item{nm}{Character valued name.}
#' }
#' @noRd
format_selector <- function(.obj, .is_fmllst) {
  if (!.is_fmllst) {
    list_loc <-
      lapply(
        which(!.obj$select_lgl),
        function(el) {
          nm <- .obj$rhs_names[[el]]
          list(eq = 1, elem = el, name = nm)
        }
      )
  } else {
    list_loc <-
      lapply(
        seq_len(length(.obj)),
        function(elem) {
          lapply(
            which(!.obj[[elem]]$select_lgl),
            function(el) {
              nm <- 
                paste(
                  .obj[[elem]]$lhs_names, "~", 
                  .obj[[elem]]$rhs_names[[el]], sep = "")
              list(eq = elem, elem = el, name = nm)
            }
          )
        }
      )
  }
  list_loc
}
#' @title Complete dominance computation
#' @description Internal function to compute complete dominance proportions for 
#' each name pair.
#' @param value_vector A numeric vector.
#' @param subset_matrix A result of `subset_matrix_contructor` applied to 
#' all names. A logical matrix with column names.
#' @param .nms A named list. Groups the results in `.set` and `.wst` together 
#' and is used to determine valid name combinations.
#' @return A square numeric matrix. One name is associated with each column 
#' (dominant name). One name is associaated with each row (non-dominant name).
#' @noRd
compute_complete_dominance <- 
  function(value_vector, subset_matrix, .nms) {
  name_count <- length(.nms)
  complete_dominance <- matrix(nrow = name_count, ncol = name_count)
  all_name_pairs <- utils::combn(1:name_count, 2)
  for (name_pair in seq_len(ncol(all_name_pairs))) {
    selected_name_pair_locs <- 
      all_name_pairs[, name_pair]
    selected_name_pair <- 
      intersect(unlist(.nms[selected_name_pair_locs]), names(subset_matrix))
    unselected_names <- setdiff(names(subset_matrix), selected_name_pair)
    selected_names_matrix <-
      cbind(subset_matrix, seq_len(nrow(subset_matrix)))
    one_names <- unlist(.nms[[selected_name_pair_locs[[1]]]])
    subsets_only_one <- 
      rowSums(subset_matrix[, one_names, drop = FALSE]) == length(one_names)
    two_names <- unlist(.nms[[selected_name_pair_locs[[2]]]])
    subsets_only_two <- 
      rowSums(subset_matrix[, two_names, drop = FALSE]) == length(two_names)
    selected_names_matrix <- 
      selected_names_matrix[xor(subsets_only_one, subsets_only_two), ]
    sorting_matrix <- 
      selected_names_matrix[, c(unselected_names, selected_name_pair)]
    sorting_df <- as.data.frame(sorting_matrix)
    selected_names_sorted <- 
      selected_names_matrix[do.call("order", sorting_df), ]
    first_name_locs <- (seq_len(nrow(selected_names_sorted)) %% 2) == 0
    first_name_index <-
      selected_names_sorted[first_name_locs, ncol(selected_names_sorted)]
    first_name_values <- value_vector[first_name_index]
    second_name_locs <- (seq_len(nrow(selected_names_sorted)) %% 2) == 1
    second_name_index <-
      selected_names_sorted[second_name_locs, ncol(selected_names_sorted)]
    second_name_values <- value_vector[second_name_index]
    sorted_results_pair <- cbind(first_name_values, second_name_values)
    first_vs_second <- sorted_results_pair[, 1] > sorted_results_pair[, 2]
    second_vs_first <- sorted_results_pair[, 1] < sorted_results_pair[, 2]
    complete_dominance[
      selected_name_pair_locs[[2]], selected_name_pair_locs[[1]]
      ] <- mean(second_vs_first)
    complete_dominance[
      selected_name_pair_locs[[1]], selected_name_pair_locs[[2]]
      ] <- mean(first_vs_second)
  }
  complete_dominance
}

#' @name domir-package
#' @title Dominance Analysis Methods
#' @author Joseph Luchman \email{jluchman@gmail.com}
#' @keywords internal
#' "_PACKAGE"
#' @description
#' Dominance analysis methods intended for predictive modeling functions.
#' @details
#' This package implements several methods to compute dominance analysis
#' (Azen & Budescu, 2004; Budescu, 1993). Dominance analysis is a relative
#' importance analysis approach that derives conceptually from Shapley values
#' (e.g.,  Grömping, 200;  Lipovsky & Conklin, 2001) in that it ascribes
#' 'values' from some function to inputs (known as 'names' in the package)
#' to that function.
#'
#' When applied to predictive models, the method compares components of a
#' fit metric ascribed to each 'name' (i.e., independent variable,
#' predictor, feature, or parameter estimate) to each other 'name' in a
#' pairwise fashion to determine a hierarchy of dominance or
#' relative importance.
#' @references
#' \itemize{
#' \item Azen, R., & Budescu, D. V. (2003). The dominance analysis approach
#' for comparing predictors in multiple regression. Psychological Methods,
#' 8(2), 129-148. doi:10.1037/1082-989X.8.2.129
#' \item Budescu, D. V. (1993). Dominance analysis: A new approach to the
#' problem of relative importance of predictors in multiple regression.
#' Psychological Bulletin, 114(3), 542-551. doi:10.1037/0033-2909.114.3.542
#' \item Grömping, U. (2007). Estimators of relative importance in linear
#' regression based on variance decomposition. The American Statistician,
#' 61(2), 139-147. doi:10.1198/000313007X188252
#' \item Lipovetsky, S, & and Conklin, M. (2001). Analysis of regression in
#' game theory approach. Applied Stochastic Models in Business and Industry,
#' 17(4), 319-330. doi:10.1002/asmb.446
#'}
NULL

#' Dominance analysis supporting \code{formula}-based modeling functions
#'
#' Computes dominance statistics for predictive modeling functions that accept a \code{\link{formula}}.
#' 
#' @param formula_overall An object of class \code{\link{formula}} or that can be coerced to class \code{formula} for use in the modeling function in \code{reg}.  The \code{\link{terms}} on the right hand side of this formula are used as separate entries to the dominance analysis.
#' 
#' A valid \code{formula_overall} entry is necessary, even if only submitting entries in \code{sets}, to define a valid left hand side of the prediction equation (see examples).  The function called in \code{reg} must accept one or more responses on the left hand side.
#' @param reg A function implementing the predictive (or "reg"ression) model called. 
#' 
#' String function names (e.g., "lm"), function names (e.g., \code{lm}), or anonymous functions (e.g., \code{function(x) lm(x)}) are acceptable entries.  This argument's contents are passed to \code{\link{do.call}} and thus any function call \code{do.call} would accept is valid.
#' 
#' The predictive model in \code{reg} must accept a \code{formula} object as its first argument or must be adapted to do so with a wrapper function.
#' @param fitstat List providing arguments to call a fit statistic extracting function (see details). The \code{fitstat} list must be of at least length two. 
#' 
#' The first element of \code{fitstat} must be a function implementing the fit statistic extraction. String function names (e.g., "summary"), function names (e.g., \code{summary}), or anonymous functions (e.g., \code{function(x) summary(x)}) are acceptable entries. This element's contents are passed to \code{\link{do.call}} and thus any function call \code{do.call} would accept is valid.
#' 
#'The second element of \code{fitstat} must be the named element of the list or vector produced by the fit extractor function called in the first element of \code{fitstat}.  This element must be a string (e.g., "r.squared").
#' 
#' All list elements beyond the second are submitted as additional arguments to the fit extractor function call.
#' 
#' The fit statistic extractor function in the first list element of \code{fitstat} must accept the model object produced by the predictive modeling function in \code{reg} as its first argument or be adapted to do so with a wrapper function.
#' 
#' The fit statistic produced must be scalar valued (i.e., vector of length 1).
#' @param sets A list with each element comprised of vectors containing variable/factor names or \code{formula} coercible strings. 
#' 
#' Each separate list element-vector in \code{sets} is concatenated (when the list element-vector is of length > 1) and used as an entry to the dominance analysis along with the terms in \code{formula_overall}.
#' @param all A vector of variable/factor names or \code{formula} coercible strings.  The entries in this vector are concatenated (when of length > 1) but are not used in the dominance analysis.  Rather the value of the fit statistic associated with these terms is removed from the dominance analysis; this vector is used like a set of covariates.
#' 
#' The entries in \code{all} are removed from and considered an additional component that explains the fit metric.  As a result, the general dominance statistics will no longer sum to the overall fit metric and the standardized vector will no longer sum to 1.
#' @param conditional Logical.  If \code{FALSE} then conditional dominance matrix is not computed.
#' 
#' If conditional dominance is not desired as an importance criterion, avoiding computing the conditional dominance matrix can save computation time.
#' @param complete Logical.  If \code{FALSE} then complete dominance matrix is not computed.
#' 
#' If complete dominance is not desired as an importance criterion, avoiding computing complete dominance designations can save computation time.
#' @param consmodel A vector of variable/factor names, \code{formula} coercible strings, or other formula terms (i.e., 1 to indicate an intercept).  The entries in this vector are concatenated (when of length > 1) and, like the entries of \code{all}, are not used in the dominance analysis; this vector is used as an adjustment to the baseline value of the overall fit statistic.  
#' 
#' The use of \code{consmodel} changes the interpretation of the the general and conditional dominance statistics.  When \code{consmodel} is used, the general and conditional dominance statistics are reflect the difference between the constant model and the overall fit statistic values.
#'  
#' Typical usage of \code{consmodel} is to pass "1" to set the intercept as the baseline and control for its value when the baseline model's fit statistic value is not 0 (e.g., if using the AIC or BIC as a fit statistic; see examples).
#' 
#' As such, this vector is used to set a baseline for the fit statistic when it is non-0. 
#' @param reverse Logical. If \code{TRUE} then standardized vector, ranks, and complete dominance designations are reversed in their interpretation.  
#' 
#' This argument should be changed to \code{TRUE} if the fit statistic used decreases with better fit to the data (e.g., AIC, BIC). 
#' @param ... Additional arguments passed to the function call in the \code{reg} argument.
#'
#' @section Notes:
#' 
#' `domin` is an R port of the Stata command with the same name (see Luchman, 2021).
#' 
#' `domin` has been superseded by [`domir`].
#'
#' @return Returns an object of \code{\link{class}} "domin".
#' An object of class "domin" is a list composed of the following elements:
#' \describe{
#'  \item{\code{General_Dominance}}{Vector of general dominance statistics.}
#'  \item{\code{Standardized}}{Vector of general dominance statistics normalized to sum to 1.}
#'  \item{\code{Ranks}}{Vector of ranks applied to the general dominance statistics.}
#'  \item{\code{Conditional_Dominance}}{Matrix of conditional dominance statistics.  Each row represents a term; each column represents an order of terms.}
#'  \item{\code{Complete_Dominance}}{Logical matrix of complete dominance designations. The term represented in each row indicates dominance status; the terms represented in each columns indicates dominated-by status.}
#'  \item{\code{Fit_Statistic_Overall}}{Value of fit statistic for the full model.}
#'  \item{\code{Fit_Statistic_All_Subsets}}{Value of fit statistic associated with terms in \code{all}.}
#'  \item{\code{Fit_Statistic_Constant_Model}}{Value of fit statistic associated with terms in \code{consmodel}.}
#'  \item{\code{Call}}{The matched call.}
#'  \item{\code{Subset_Details}}{List containing the full model and descriptions of terms in the full model by source.}
#' }
#'
#' @details \code{domin} automates the computation of all possible combination of entries to the dominance analysis (DA), the creation of \code{formula} objects based on those entries, the modeling calls/fit statistic capture, and the computation of all the dominance statistics for the user.
#' 
#' \code{domin} accepts only a "deconstructed" set of inputs and "reconstructs" them prior to formulating a coherent predictive modeling call.
#' 
#' One specific instance of this deconstruction is in generating the number of entries to the DA. The number of entries is taken as all the \code{terms} from \code{formula_overall} and the separate list element vectors from \code{sets}. The entries themselves are concatenated into a single formula, combined with the entries in \code{all}, and submitted to the predictive modeling function in \code{reg}.  Each different combination of entries to the DA forms a different \code{formula} and thus a different model to estimate.
#' 
#' For example, consider this \code{domin} call:
#' 
#' \code{domin(y ~ x1 + x2, lm, list(summary, "r.squared"), sets = list(c("x3", "x4")), all = c("c1", "c2"), data = mydata))}
#' 
#' This call records three entries and results in seven (i.e., \eqn{2^3 - 1}) different combinations:
#' 
#' \enumerate{
#' \item x1
#' \item x2
#' \item x3, x4
#' \item x1, x2
#' \item x1, x3, x4
#' \item x2, x3, x4
#' \item x1, x2, x3, x4
#' }
#' 
#' \code{domin} parses \code{formula_overall} to obtain all the terms in it and combines them with \code{sets}.  When parsing \code{formula_overall}, only the processing that is available in the \code{stats} package is applied.  Note that \code{domin} is not programmed to process terms of order > 1 (i.e., interactions/products) appropriately (i.e., only include in the presence of lower order component terms). \code{domin} also does not allow \code{offset} terms.
#' 
#' From these combinations, the predictive models are constructed and called. The predictive model call includes the entries in \code{all}, applies the appropriate formula, and reconstructs the function itself. The seven combinations above imply the following series of predictive model calls:
#' 
#' \enumerate{
#' \item \code{lm(y ~ x1 + c1 + c2, data = mydata})
#' \item \code{lm(y ~ x2 + c1 + c2, data = mydata})
#' \item \code{lm(y ~ x3 + x4 + c1 + c2, data = mydata})
#' \item \code{lm(y ~ x1 + x2 + c1 + c2, data = mydata})
#' \item \code{lm(y ~ x1 + x3 + x4 + c1 + c2, data = mydata})
#' \item \code{lm(y ~ x2 + x3 + x4 + c1 + c2, data = mydata})
#' \item \code{lm(y ~ x1 + x2 + x3 + x4 + c1 + c2, data = mydata})
#' }
#'
#' It is possible to use a \code{domin} with only sets (i.e., no IVs in \code{formula_overall}; see examples below). There must be at least two entries to the DA for \code{domin} to run.
#'
#' All the called predictive models are submitted to the fit extractor function implied by the entries in \code{fitstat}. Again applying the example above, all seven predictive models' objects would be individually passed as follows:
#' 
#' \code{summary(lm_obj)["r.squared"]}
#' 
#' where \code{lm_obj} is the model object returned by \code{lm}. 
#' 
#' The entries to \code{fitstat} must be as a list and follow a specific structure: 
#' \code{list(fit_function, element_name, ...)}
#' \describe{
#'  \item{\code{fit_function}}{First element and function to be applied to the object produced by the \code{reg} function}
#'  \item{\code{element_name}}{Second element and name of the element from the object returned by \code{fit_function} to be used as a fit statistic.  The fit statistic must be scalar-valued/length 1}
#'  \item{\code{...}}{Subsequent elements and are additional arguments passed to \code{fit_function}}
#' }
#' 
#' In the case that the model object returned by \code{reg} includes its own fit statistic without the need for an extractor function, the user can apply an anonymous function following the required format to extract it.
#' 
#' @references 
#' 
#' Luchman, J. N. (2021). Relative importance analysis in Stata using dominance analysis: domin and domme. 
#' The Stata Journal, 21, 2. doi: 10.1177/1536867X211025837.
#' 
#' @export 
#' @examples
#' ## Basic linear model with r-square
#' 
#' domin(mpg ~ am + vs + cyl, 
#'   lm, 
#'   list("summary", "r.squared"), 
#'   data = mtcars)
#' 
#' 
#' ## Linear model including sets
#' 
#' domin(mpg ~ am + vs + cyl, 
#'   lm, 
#'   list("summary", "r.squared"), 
#'   data = mtcars, 
#'   sets = list(c("carb", "gear"), c("disp", "wt")))
#'
#'
#' ## Multivariate linear model with custom multivariate r-square function 
#' ## and all subsets variable
#' 
#' Rxy <- function(obj, names, data) {
#'    return(list("r2" = cancor(predict(obj), 
#'        as.data.frame(mget(names, as.environment(data))))[["cor"]][1]^2)) 
#'        }
#'        
#' domin(cbind(wt, mpg) ~ vs + cyl + am, 
#'   lm, 
#'   list(Rxy, "r2", c("mpg", "wt"), mtcars), 
#'   data = mtcars, 
#'   all = c("carb"))
#'
#'
#' ## Sets only
#' 
#' domin(mpg ~ 1, 
#'   lm, 
#'   list("summary", "r.squared"), 
#'   data = mtcars, 
#'   sets = list(c("am", "vs"), c("cyl", "disp"), c("qsec", "carb")))
#'   
#' ## Constant model using AIC
#' 
#' domin(mpg ~ am + carb + cyl, 
#'   lm, 
#'   list(function(x) list(aic = extractAIC(x)[[2]]), "aic"), 
#'   data = mtcars, 
#'   reverse = TRUE, consmodel = "1")

domin <- 
  function(formula_overall, reg, fitstat, sets = NULL, all = NULL, 
           conditional = TRUE, complete = TRUE, consmodel = NULL, reverse = FALSE, ...) {
    
    # Initial exit/warning conditions ---- 
    
    if (!inherits(formula_overall, "formula")) 
      stop(paste(formula_overall, "is not a 'formula' class object."))
    
    if (!is.null(attr(stats::terms(formula_overall), "offset"))) 
      stop("'offset()' terms not allowed in formula object.")
    
    if (!is.list(fitstat)) 
      stop("fitstat is not a list.")
    
    if (length(sets) > 0 & !is.list(sets)) 
      stop("sets is not a list.")
    
    if (is.list(all)) 
      stop("all is a list.  Please submit it as a vector.")
    
    if (!attr(stats::terms(formula_overall), "response")) 
      stop(paste(deparse(formula_overall), "missing a response."))
    
    if (any(attr(stats::terms(formula_overall), "order") > 1))
      warning(paste(deparse(formula_overall), "contains second or higher order terms. domin may not handle them correctly."))
    
    if (length(fitstat) < 2) 
      stop("fitstat requires at least two elements.")
    
    # Process variable lists ----
    
    Indep_Vars <- 
      attr(stats::terms(formula_overall), "term.labels") # obtain IV name vector from `formula_overall`
    
    intercept <- as.logical(attr(stats::terms(formula_overall), "intercept") ) # does the model have an intercept?  Needed for `reformulate`
    
    if (length(sets) > 0) { # if there are sets...
      
      set_aggregated <- 
        sapply(sets, paste0, collapse=" + ") # ...paste together IV names from each set in `formula` format as a vector...
      
      Indep_Vars <- 
        append(Indep_Vars, set_aggregated) # ...append sets vector to end of IV name vector
      
    }
    
    Dep_Var <- 
      attr(stats::terms(formula_overall), "variables")[[2]] # pull out DV name from `formula_overall`
    
    Total_Indep_Vars <- length(Indep_Vars) # count number of IVs and sets in model
    
    ## IV-based exit conditions ----
    if (Total_Indep_Vars < 2) 
      stop(paste("Total of", Total_Indep_Vars, "independent variables or sets. At least 2 needed for useful dominance analysis."))
    
    # Create independent variable/set combination list ----
    
    Combination_Matrix <- 
      expand.grid(
        lapply(1:Total_Indep_Vars, 
               function(x) c(FALSE, TRUE)), 
        KEEP.OUT.ATTRS = FALSE)[-1,] # Logical matrix of inclusion/exclusion for IVs; omit the first, empty row
    
    Total_Models_to_Estimate <- 2**Total_Indep_Vars - 1 # total number of models to estimate
    
    # Define function to call regression models ----
    
    doModel_Fit <- function(Indep_Var_Combin_lgl, Indep_Vars, Dep_Var, # assumes receipt of a logical matrix indicating which IVs to use as well as IV list and DV
                            reg, fitstat, all = NULL, consmodel = NULL, intercept, ...) { # also regression function, fitstat function, with all and consmodel arguments along with an intercept argument for reformulate
      
      Indep_Var_Combination <- Indep_Vars[Indep_Var_Combin_lgl] # select vector of IVs
      
      formula_to_use <- 
        stats::reformulate(c(Indep_Var_Combination, all, consmodel), # 'reformulate' formula to submit to model
                           response = Dep_Var, intercept = intercept)
      
      Model_Result <- # capture the model object ...
        list( # ... as an element of a list, needed for next step calling fit statistic function ...
          do.call(reg, list(formula_to_use, ...) ) # ... from `do.call` invoking the modeling function with formula and all other arguments as the ellipsis
        ) 
      
      if (length(fitstat) > 2) # if there are additional arguments to pass to the fitstat function, indicated by having length of > 2 for this list...
        Model_Result <- 
        append(Model_Result, fitstat[3:length(fitstat)]) # ...append these additional arguments to 'Model_Result' for coming `do.call`
      
      Fit_Value <- do.call(fitstat[[1]], Model_Result) # use first entry of `fitstat` as fitstat function name, use `Model_Result` as results to submit to it
      
      return( Fit_Value[[ fitstat[[2]] ]] ) # ... use second, necessarily named, argument of `fitstat` to select the result of `Fit_Value` to return
      
    }
    
    # Constant model adjustments ----
    
    if (length(consmodel) > 0) { # if there are entries in consmodel...
      
      FitStat_Adjustment <- Cons_Result <- doModel_Fit(NULL, Indep_Vars, Dep_Var, reg, fitstat, consmodel = consmodel, intercept = intercept, ...) # ...obtain their fitstat' value using `doModel_Fit()`
      
    }
    
    else {
      Cons_Result <- NULL # ...otherwise return a null
      FitStat_Adjustment <- 0
    }
    
    # All subsets adjustment ----
    
    if (length(all) > 0) { # if there are entries in all...
      
      FitStat_Adjustment <- All_Result <- doModel_Fit(NULL, Indep_Vars, Dep_Var, reg, fitstat, all = all, consmodel = consmodel, intercept = intercept, ...) # ...obtain their 'fitstat' value as well using `doModel_fit()`...
      
    }
    
    else All_Result <- NULL # ...otherwise return a null
    
    # Obtain all subsets regression results ----
    
    Ensemble_of_Models <- 
      sapply(1:nrow(Combination_Matrix), # for all individual models/rows in the logical combination matrix...
             function(x) { # ... submit the row number ...
               doModel_Fit(unlist(Combination_Matrix[x,]), # ... select a unique pattern of IVs as a logical vector from the combination matrix ...
                           Indep_Vars, Dep_Var, reg, fitstat, # ... and submit all model information
                           all = all, consmodel = consmodel, intercept = intercept, ...)
             },
             simplify = TRUE, USE.NAMES = FALSE)
    
    # Obtain conditional dominance statistics ----
    
    if (conditional) {
      
      Conditional_Dominance <- 
        matrix(nrow = Total_Indep_Vars, ncol = Total_Indep_Vars) # conditional dominance container
      
      Combination_Matrix_Anti <-!Combination_Matrix # complement of logical IV inclusion matrix
      
      IVs_per_Model <- rowSums(Combination_Matrix) # count IVs in each model using logical IV inclusion matrix
      
      Combins_at_Order <- 
        sapply(IVs_per_Model, # for each IV/column, ... 
               function(x) choose(Total_Indep_Vars, x), # ... compute # of combinations associated with each row given the total number of IVs
               simplify = TRUE, USE.NAMES = FALSE)
      
      Combins_at_Order_Prev <- 
        sapply(IVs_per_Model, # for each IV/column, ... 
               function(x) choose(Total_Indep_Vars - 1, x),# ... compute # of combinations associated with each row given the total number of IVs (i.e., # of combinations _not_ including focal IV)
               simplify = TRUE, USE.NAMES = FALSE)
      
      Weighted_Order_Ensemble <- # create a matrix indicating how many unique combinations are associated with each number of IVs in the model; these values are associated with models that have the focal IV (hence multiplied by logical matrix).  This matrix is inverted to make weights that can be used for weighted averaging.  Finally, these weights are multiplied by all the fit statistics
        ((Combination_Matrix*(Combins_at_Order - Combins_at_Order_Prev))**-1)*Ensemble_of_Models
      
      Weighted_Order_Ensemble <- # effectively missing values (Inf) are changed to 0 to remove from summing
        replace(Weighted_Order_Ensemble, 
                Weighted_Order_Ensemble==Inf, 0)
      
      Weighted_Order_Ensemble_Anti <- # create a matrix indicating how many unique combinations are associated with each number of IVs in the model; these values are associated with models that have do not have focal IV and would thus be used to subtract out of models that do (hence the reflection across 0). This matrix is inverted to make weights that can be used for weighted averaging.  Finally, these weights are multiplied by all the fit statistics
        ((Combination_Matrix_Anti*Combins_at_Order_Prev)**-1)*Ensemble_of_Models
      
      Weighted_Order_Ensemble_Anti <-# effectively missing values (Inf) are changed to 0 to remove from summing
        replace(Weighted_Order_Ensemble_Anti, 
                Weighted_Order_Ensemble_Anti==Inf, 0)
      
      for (order in 1:Total_Indep_Vars) {
        
        Conditional_Dominance[, order] <- # the column associated with an order (# of IVs) in the model is replaced by ...
          t( # ... transpose of ...
            colSums(Weighted_Order_Ensemble[IVs_per_Model==order,]) - # ... the sum of the weighted fit stats at a with a specified number of IVs in the model for all models including the focal IV (across all rows) ...
              colSums(Weighted_Order_Ensemble_Anti[IVs_per_Model==(order-1),]) # ... subtracting out the sum of the weighted fit stats with one less than a specified number of IVs in the model for all models that do not contain the focal IV (across all rows)
          )
        
      }
      
      Conditional_Dominance[,1] <- Conditional_Dominance[,1] - FitStat_Adjustment # adjust the first set of conditional dominance statistics for the constant and/or all fitstat adjustments
      
    }
    
    else Conditional_Dominance <- NULL
    
    # Obtain complete dominance statistics ----
    
    if (complete) {
      
      Complete_Dominance <- 
        matrix(data = NA, nrow = Total_Indep_Vars, ncol = Total_Indep_Vars) # complete dominance container; initialize with missing values
      
      Complete_Combinations <- 
        utils::combn(1:Total_Indep_Vars, 2) # produce all pairs of IVs (indicated by column numbers in logical matrix)
      
      for (pair in 1:ncol(Complete_Combinations)) { # looping across all pairs of IVs...
        
        Focal_Cols <- Complete_Combinations[, pair] # identify which columns are associated with the focal unique pair of IVs
        
        NonFocal_Cols <- setdiff(1:Total_Indep_Vars, Focal_Cols) # find the columns associated with the complement of IVs not selected in the unique pair
        
        Select_2IVs <- 
          cbind(Combination_Matrix, 1:nrow(Combination_Matrix))[ # combine logical IV inclusion matrix with vector of row indicators (used to select specific rows in the fitstat vector) ...
            rowSums( Combination_Matrix[,Focal_Cols] )==1, ] #... then take only the rows/models associated with having only one of either of the focal IVs (never both or neither IVs)
        
        Sorted_2IVs <- 
          Select_2IVs[
            do.call("order", # sort/order the logical and row indicator matrix...
                    as.data.frame(Select_2IVs[,c(NonFocal_Cols, Focal_Cols)])), ] #... based on the non included IVs - this forces non-included IVs to "pair" sequentially; that is, models with focal IV 1 are at vector location 'n' and models with focal IV 2 are at vector location 'n + 1' and all the non-focal IVs are identical between those two models (which is required for a valid complete dominance comparison)
        
        Compare_2IVs <-
          cbind( # bind in a matrix as different columns...
            Ensemble_of_Models[ # ... fit statistics ...
              Sorted_2IVs[(1:nrow(Sorted_2IVs) %% 2)==0, ncol(Sorted_2IVs)]], # ... selected based on the row indicators (i.e., last row of the sorted matrix produced above), in all 'n' row positions (i.e., modulus of 0 with frequency 2)...
            Ensemble_of_Models[ # ... as well as the fit statistics ...
              Sorted_2IVs[(1:nrow(Sorted_2IVs) %% 2)==1, ncol(Sorted_2IVs)]] # ... selected based on the row indicators (again, last row of the sorted matrix produced above), in all 'n + 1' row positions (i.e., modulus of 1 with frequency 2)
          )
        
        Complete_Designation <- # compare the matched matrices across columns - and make complete dominance designation
          ifelse(all(Compare_2IVs[,1] > Compare_2IVs[,2]), FALSE,
                 ifelse(all(Compare_2IVs[,1] < Compare_2IVs[,2]), TRUE, NA))
        
        Complete_Dominance[ Focal_Cols[[2]], Focal_Cols[[1]] ] <- #assign designation to one instance of pair 
          Complete_Designation
        
        Complete_Dominance[ Focal_Cols[[1]], Focal_Cols[[2]] ] <- #assign designation to complementary instance of pair
          !Complete_Designation
        
      }
      
    }
    
    else Complete_Dominance <- NULL
    
    if (reverse == TRUE) Complete_Dominance <- !Complete_Dominance # reverse all designations with `reverse`
    
    # Obtain general dominance statistics ----
    
    if (!conditional) { # if there was no conditional dominance computed, generate general dominance as below...
      
      Combination_Matrix_Anti <-!Combination_Matrix # complement of logical IV inclusion matrix
      
      IVs_per_Model <- rowSums(Combination_Matrix) # count IVs in each model using logical IV inclusion matrix
      
      Combins_at_Order <- 
        sapply(IVs_per_Model, # for each IV/column, ... 
               function(x) choose(Total_Indep_Vars, x), # ... compute # of combinations associated with each row given the total number of IVs
               simplify = TRUE, USE.NAMES = FALSE)
      
      Combins_at_Order_Prev <- 
        sapply(IVs_per_Model, # for each IV/column, ... 
               function(x) choose(Total_Indep_Vars - 1, x),# ... compute # of combinations associated with each row given the total number of IVs (i.e., # of combinations _not_ including focal IV)
               simplify = TRUE, USE.NAMES = FALSE)
      
      Indicator_Weight <- # create a matrix indicating how many unique combinations are associated with each number of IVs in the model (surrogate for what conditional dominance statistics do); these values are associated with models that have the focal IV
        Combination_Matrix*(Combins_at_Order - Combins_at_Order_Prev)
      
      Indicator_Weight_Anti <- # create a matrix indicating how many unique combinations are associated with each number of IVs in the model (again, surrogate for what conditional dominance statistics do); these values are associated with models that have do not have focal IV and would thus be used to subtract out of models that do (hence the reflection across 0)
        (Combination_Matrix_Anti*Combins_at_Order_Prev)*-1
      
      Weight_Matrix <- # combine combinations of focal and non-focal counts associated with what the conditional dominance statistics would do with the number of IV average associated with what the general dominance statistics do (arithmetic average of all conditional dominance statistics); these combinations are inverted to make them function as a set of weights that produce averages 
        ((Indicator_Weight + Indicator_Weight_Anti)*Total_Indep_Vars)^-1
      
      General_Dominance <- # general dominance is sum of the product of the weight matrix above and all the fit statistics
        colSums(Ensemble_of_Models*Weight_Matrix)
      
      General_Dominance <- General_Dominance - FitStat_Adjustment/Total_Indep_Vars # adjust the general dominance statistics for the constant and/or all fitstat adjustments in the first/order 1 stats as conditional dominance would do
      
    }
    
    else General_Dominance <- rowMeans(Conditional_Dominance) #... otherwise average conditional dominance by IV/row to produce general dominance statistics
    
    # Obtain overall fit statistic and ranks ----
    
    FitStat <- 
      sum(General_Dominance) + FitStat_Adjustment # adjust overall fit statistic by replacing all subsets component and constant model component
    
    if (reverse == FALSE) General_Dominance_Ranks <- rank(-General_Dominance) # rank general dominance statistic if fitstat value increases (i.e., `reverse` == FALSE)
    else General_Dominance_Ranks <- rank(General_Dominance) # rank general dominance statistic if fitstat value decreases (i.e., `reverse` == TRUE)
    
    # Finalize returned values and attributes ----
    
    if (length(sets) == 0 ) IV_Labels <- 
      attr(stats::terms(formula_overall), "term.labels")
    else IV_Labels <- 
      c( attr(stats::terms(formula_overall), "term.labels"), 
         paste0("set", 1:length(sets)) ) # names for returned values
    
    names(General_Dominance) <- IV_Labels
    names(General_Dominance_Ranks) <- IV_Labels 
    if (conditional) 
      dimnames(Conditional_Dominance) <- list(IV_Labels, paste0("IVs_", 1:length(Indep_Vars)))
    if (complete) 
      dimnames(Complete_Dominance) <- list(paste0("Dmnates_", IV_Labels),  paste0("Dmnated_", IV_Labels))
    
    if (reverse == FALSE) # Standardized if fitstat increases...
      Standardized <- General_Dominance/(FitStat - ifelse(length(Cons_Result) > 0, Cons_Result, 0)) # ...then use normal standardization...
    else Standardized <- -General_Dominance/-(FitStat - ifelse(length(Cons_Result) > 0, Cons_Result, 0)) # ...otherwise reverse the general dominance stats to standardize
    
    return_list <- list(
      "General_Dominance" = General_Dominance,
      "Standardized" = Standardized,
      "Ranks" = General_Dominance_Ranks,
      "Conditional_Dominance" = Conditional_Dominance,
      "Complete_Dominance" = Complete_Dominance,
      "Fit_Statistic_Overall" = FitStat,
      "Fit_Statistic_All_Subsets" = All_Result - ifelse(is.null(Cons_Result), 0, Cons_Result),
      "Fit_Statistic_Constant_Model" = Cons_Result,
      "Call" = match.call(),
      "Subset_Details" = list(
        "Full_Model" = stats::reformulate(c(Indep_Vars, all, consmodel), response = Dep_Var, intercept = intercept),
        "Formula" = attr(stats::terms(formula_overall), "term.labels"), 
        "All" = all,
        "Sets" = sets,
        "Constant" = consmodel
      )
    )
    
    class(return_list) <- c("domin", "list") # apply 'domin' type for S3 method dispatch - list is alternative
    
    return(return_list)
    
  }

#' Print method for \code{domin}
#'
#' Reports formatted results from \code{domin} class object.
#' @param x an object of class "domin".
#' @param ... further arguments passed to or from other methods. Not used currently.
#' @return The "domin" object with altered column and row names for conditional and complete dominance results as displayed in the console.
#' @details The print method for class \code{domin} objects reports out the following results:
#' \itemize{
#'  \item{Fit statistic for the full model.  The fit statistic for the all subsets model is reported here if there are any entries in \code{all}.  The fit statistic for the constant model is reported here if there are any entries in \code{consmodel}.}
#'  \item{Matrix describing general dominance statistics, standardized general dominance statistics, and the ranking of the general dominance statistics}
#'  \item{If \code{conditional} is \code{TRUE}, matrix describing the conditional dominance designations}
#'  \item{If \code{complete} is \code{TRUE}, matrix describing the complete dominance designations}
#'  \item{If following \code{summary.domin}, matrix describing the strongest dominance designations between all independent variables}
#'  \item{If there are entries in \code{sets} and/or \code{all} the terms included in each set as well as the terms in all subsets are reported}}
#'  The \code{domin} print method alters dimension names for readability and they do not display as stored in the original \code{domin} object.
#' @export

print.domin <- function(x, ...) {
  
  cat("Overall Fit Statistic:     ", x[["Fit_Statistic_Overall"]], "\n")
  
  if (length(x[["Fit_Statistic_All_Subsets"]]) > 0) 
    cat("All Subsets Fit Statistic: ", x[["Fit_Statistic_All_Subsets"]], "\n")
  
  if (length(x[["Fit_Statistic_Constant_Model"]]) > 0) 
    cat("Constant Model Fit Statistic: ", x[["Fit_Statistic_Constant_Model"]], "\n")
  
  cat("\n")
  
  cat("General Dominance Statistics:\n")
  
  Display_Std <- 
    t(rbind(x[["General_Dominance"]], x[["Standardized"]], x[["Ranks"]]))
  
  dimnames(Display_Std) <- 
    list(names(x[["Ranks"]]), c("General Dominance", "Standardized", "Ranks"))
  
  print(Display_Std)
  
  cat("\n")
  
  if (length(x[["Conditional_Dominance"]] > 0)) {
    
    cat("Conditional Dominance Statistics:\n")
    
    colnames(x[["Conditional_Dominance"]]) <- 
      paste("IVs:", 1:ncol(x[["Conditional_Dominance"]]))
    
    print(x[["Conditional_Dominance"]])
    
    cat("\n")
    
  }
  
  if (length(x[["Complete_Dominance"]] > 0)) {
    
    cat("Complete Dominance Designations:\n")
    
    colnames(x[["Complete_Dominance"]]) <- 
      gsub("^Dmnated_", "Dmnated?", colnames(x[["Complete_Dominance"]]))
    
    rownames(x[["Complete_Dominance"]]) <- 
      gsub("^Dmnates_", "Dmnates?", rownames(x[["Complete_Dominance"]]))
    
    print(x[["Complete_Dominance"]])
    
    cat("\n")
    
  }
  
  if (length(x[["Strongest_Dominance"]] > 0)) {
    
    cat("Strongest Dominance Designations:")
    
    print(x[["Strongest_Dominance"]])
    
    cat("\n")
    
  }
  
  if (length(x[["Subset_Details"]][["Sets"]]) > 0) {
    
    cat("Components of sets:\n")
    
    for (set in 1:length(x[["Subset_Details"]][["Sets"]])) {
      
      cat(paste0("set", set),":", x[["Subset_Details"]][["Sets"]][[set]], "\n")
      
    }
    
    cat("\n")
    
  }
  
  if (length(x[["Subset_Details"]][["All"]]) > 0) {
    
    cat("All subsets variables:", x[["Subset_Details"]][["All"]])
    
  }
  
  invisible(x)
  
}


#' Summary method for \code{domin}
#'
#' Reports dominance designation results from the \code{domin} class object.
#' @param object an object of class "domin".
#' @param ... further arguments passed to or from other methods. Not used currently.
#' @return The originally submitted "domin" object with an additional \code{Strongest_Dominance} element added.
#' \describe{
#'  \item{\code{Strongest_Dominance}}{Matrix comparing the independent variable in the first row to the independent variable in the third row.  The second row denotes the strongest designation between the two independent variables.}
#' }
#' @details The summary method for class \code{domin} is used for obtaining the strongest dominance designations (i.e., general, conditional, or complete) among the independent variables.
#' @export

summary.domin <- function(object, ...) {
  
  if (length(object[["Strongest_Dominance"]]) == 0) {
    
    reverse <- as.list(object$Call)$reverse
    if (is.null(reverse)) reverse <- FALSE
    
    reverse_cdl <- 
      ifelse(reverse, 
             rep(-1, times = length(object$General_Dominance)), 
             rep(1, times = length(object$General_Dominance)))
    
    reverse_gnl <- ifelse(reverse, -1, 1)
    
    pairs <- utils::combn(names(object$General_Dominance), 2)
    
    pairs <- rbind(pairs[1,], rep("", times = ncol(pairs)), pairs[2,])
    
    location <- 0
    
    for (IV1 in 1:(length(object$General_Dominance) - 1)) {
      
      for (IV2 in (IV1+1):length(object$General_Dominance)) {
        
        location <- location + 1
        
        if (length(object[["Complete_Dominance"]] > 0)) {
          
          if (!is.na(object$Complete_Dominance[IV1, IV2])) {
            pairs[2, location] <- 
              ifelse(object$Complete_Dominance[IV1, IV2], 
                     "completely dominates",
                     "is completely dominated by")
            
            next
            
          }
          
        }
        
        if (length(object[["Conditional_Dominance"]] > 0)) {
          
          if (all(object$Conditional_Dominance[IV1,]*reverse_cdl > 
                  object$Conditional_Dominance[IV2,]*reverse_cdl)) { 
            
            pairs[2, location] <- "conditionally dominates"
            
            next
            
          }
          
          else if (all(object$Conditional_Dominance[IV1,]*reverse_cdl < 
                       object$Conditional_Dominance[IV2,]*reverse_cdl)) {
            
            pairs[2, location] <- "is conditionally dominated by"
            
            next
            
          }
          
        }
        
        pairs[2, location] <- 
          ifelse(object$General_Dominance[[IV1]]*reverse_gnl > 
                   object$General_Dominance[[IV2]]*reverse_gnl, 
                 "generally dominates", 
                 ifelse(object$General_Dominance[[IV1]]*reverse_gnl < 
                          object$General_Dominance[[IV2]]*reverse_gnl,
                        "is generally dominated by", 
                        "has no dominance designation with"))
        
      }
      
    }
    
    rownames(pairs) <- rep("", times = 3)
    
    colnames(pairs) <- rep("", times = ncol(pairs))
    
    res <- append(object, list(Strongest_Dominance = pairs))
    
    class(res) <- c("domin", "list")
    
    return(res)
    
  }
  
  else return(object)
  
}

library(systemfit)
library(datasets)

# domir.formula and domin tests ----

vs_mgn <- c( 
  cor(mtcars$mpg, predict(lm(mpg ~ vs, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ cyl, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + carb, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ carb, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ cyl + carb, data = mtcars)))^2)

cyl_mgn <- c(
  cor(mtcars$mpg, predict(lm(mpg ~ cyl, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ cyl + carb, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ carb, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs + carb, data = mtcars)))^2)

carb_mgn <- c( 
  cor(mtcars$mpg, predict(lm(mpg ~ carb, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ vs + carb, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ cyl + carb, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ cyl, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl, data = mtcars)))^2)

vs_c <- c(vs_mgn[[1]], mean(vs_mgn[2:3]), vs_mgn[[4]])
cyl_c <- c(cyl_mgn[[1]], mean(cyl_mgn[2:3]), cyl_mgn[[4]])
carb_c <- c(carb_mgn[[1]], mean(carb_mgn[2:3]), carb_mgn[[4]])

cdl_names <- list(c("vs", "cyl", "carb"), paste0("IVs_", 1:3))
cdl_names_new <- list(c("vs", "cyl", "carb"), paste0("include_at_", 1:3))

cdl_test <- matrix(c(vs_c, cyl_c, carb_c), nrow = 3, ncol = 3, 
                   byrow = TRUE, dimnames = cdl_names)

cdl_test_new <- matrix(c(vs_c, cyl_c, carb_c), nrow = 3, ncol = 3, 
                   byrow = TRUE, dimnames = cdl_names_new)

test_obj <- domin(mpg ~ vs + cyl + carb, "lm", list("summary", "r.squared"), 
                  data = mtcars)

test_obj_new <- domir(mpg ~ vs + cyl + carb, 
                      function(fml, data) {
                        res <- summary(lm(fml, data = data))
                        return(res[["r.squared"]])
                      },
                  data = mtcars)

test_that("Test Conditional Dominance Computation: domin", {
  expect_equal(test_obj$Conditional_Dominance, cdl_test
  )}
)

test_that("Test Conditional Dominance Computation: domir", {
  expect_equal(test_obj_new$Conditional_Dominance, cdl_test_new
  )}
)

gen_test <- rowMeans(cdl_test)

names(gen_test) <- cdl_names[[1]]

test_that("Test General Dominance Computation: domin", {
  expect_equal(test_obj$General_Dominance, gen_test
  )}
)

test_that("Test General Dominance Computation: domir", {
  expect_equal(test_obj_new$General_Dominance, gen_test
  )}
)

cyl_vs_cpt <- mean(c(cyl_mgn[[3]] > vs_mgn[[3]], 
                     cyl_mgn[[1]] > vs_mgn[[1]]))

carb_vs_cpt <- mean(c(carb_mgn[[3]] > vs_mgn[[2]], 
                    carb_mgn[[1]] > vs_mgn[[1]]))

carb_cyl_cpt <- mean(c(carb_mgn[[2]] > cyl_mgn[[2]], 
                     carb_mgn[[1]] > cyl_mgn[[1]]))

cpt_test <- matrix(c(NA, cyl_vs_cpt, carb_vs_cpt, 
                     1-cyl_vs_cpt, NA, carb_cyl_cpt, 
                     1-carb_vs_cpt, 1-carb_cyl_cpt, NA),
                   nrow = 3, ncol = 3)

dimnames(cpt_test) <- list(
  paste0(c("vs", "cyl", "carb"), "_>"),
  paste0(">_", c("vs", "cyl", "carb"))
)

dmn_trns <- function(val) {
  ifelse(val == 1, TRUE, ifelse(val == 0, FALSE, NA))
}
cpt_test_dmn <- matrix(c(NA, dmn_trns(cyl_vs_cpt), dmn_trns(carb_vs_cpt), 
                         dmn_trns(1-cyl_vs_cpt), NA, dmn_trns(carb_cyl_cpt), 
                         dmn_trns(1-carb_vs_cpt), dmn_trns(1-carb_cyl_cpt), NA),
                   nrow = 3, ncol = 3)

dimnames(cpt_test_dmn) <- list(
  paste0("Dmnates_", c("vs", "cyl", "carb")),
  paste0("Dmnated_", c("vs", "cyl", "carb"))
)

test_that("Test Complete Dominance Designation: domin", {
  expect_equal(test_obj$Complete_Dominance, cpt_test_dmn
  )})

test_that("Test Complete Dominance Designation: domir", {
  expect_equal(test_obj_new$Complete_Dominance, cpt_test
  )})

test_that("Test Overall Fit Statistic Value: domin", {
  expect_equal(test_obj$Fit_Statistic_Overall, 
               cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb, data = mtcars)))^2
  )})

test_that("Test Overall Fit Statistic Value: domir", {
  expect_equal(test_obj_new$Value,
               cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb, data = mtcars)))^2
  )})

# domir.formula_list tests ----

baseline_ll <- 
  logLik(systemfit(list(mpg ~ 1, qsec ~ 1), data = mtcars))[[1]]

vs_fl_mgn <- c( 
  (1 - logLik(systemfit(list(mpg ~ vs, qsec ~ 1), data = mtcars))[[1]]/baseline_ll),
  (1 - logLik(systemfit(list(mpg ~ vs + cyl, qsec ~ 1), data = mtcars))[[1]]/baseline_ll) - 
    (1 - logLik(systemfit(list(mpg ~ cyl, qsec ~ 1), data = mtcars))[[1]]/baseline_ll),
  (1 - logLik(systemfit(list(mpg ~ vs, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) - 
    (1 - logLik(systemfit(list(mpg ~ 1, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) , 
  (1 - logLik(systemfit(list(mpg ~ vs + cyl, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) - 
    (1 - logLik(systemfit(list(mpg ~ cyl, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) )

cyl_fl_mgn <- c( 
  (1 - logLik(systemfit(list(mpg ~ cyl, qsec ~ 1), data = mtcars))[[1]]/baseline_ll),
  (1 - logLik(systemfit(list(mpg ~ vs + cyl, qsec ~ 1), data = mtcars))[[1]]/baseline_ll) - 
    (1 - logLik(systemfit(list(mpg ~ vs, qsec ~ 1), data = mtcars))[[1]]/baseline_ll),
  (1 - logLik(systemfit(list(mpg ~ cyl, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) - 
    (1 - logLik(systemfit(list(mpg ~ 1, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) , 
  (1 - logLik(systemfit(list(mpg ~ vs + cyl, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) - 
    (1 - logLik(systemfit(list(mpg ~ vs, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) )

carb_fl_mgn <- c( 
  (1 - logLik(systemfit(list(mpg ~ 1, qsec ~ carb), data = mtcars))[[1]]/baseline_ll),
  (1 - logLik(systemfit(list(mpg ~ vs, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) - 
    (1 - logLik(systemfit(list(mpg ~ vs, qsec ~ 1), data = mtcars))[[1]]/baseline_ll),
  (1 - logLik(systemfit(list(mpg ~ cyl, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) - 
    (1 - logLik(systemfit(list(mpg ~ cyl, qsec ~ 1), data = mtcars))[[1]]/baseline_ll) , 
  (1 - logLik(systemfit(list(mpg ~ vs + cyl, qsec ~ carb), data = mtcars))[[1]]/baseline_ll) - 
    (1 - logLik(systemfit(list(mpg ~ vs + cyl, qsec ~ 1), data = mtcars))[[1]]/baseline_ll) )

vs_fl_c <- c(vs_fl_mgn[[1]], mean(vs_fl_mgn[2:3]), vs_fl_mgn[[4]])
cyl_fl_c <- c(cyl_fl_mgn[[1]], mean(cyl_fl_mgn[2:3]), cyl_fl_mgn[[4]])
carb_fl_c <- c(carb_fl_mgn[[1]], mean(carb_fl_mgn[2:3]), carb_fl_mgn[[4]])

cdl_fl_names_new <- list(c("mpg~vs", "mpg~cyl", "qsec~carb"), paste0("include_at_", 1:3))

cdl_fl_test_new <- matrix(c(vs_fl_c, cyl_fl_c, carb_fl_c), nrow = 3, ncol = 3, 
                       byrow = TRUE, dimnames = cdl_fl_names_new)

test_fl_obj_new <- 
  domir(formula_list(
    mpg ~ vs + cyl, 
    qsec ~ carb), 
        function(fml, data, b_ll) {
          res <- logLik(
            systemfit(fml, data = data)
          )[[1]]
          (1 - res/b_ll)
        },
        data = mtcars, b_ll = baseline_ll)

test_that("Test Conditional Dominance Computation: domir.formula_list", {
  expect_equal(test_fl_obj_new$Conditional_Dominance, cdl_fl_test_new
  )}
)

gen_fl_test <- rowMeans(cdl_fl_test_new)

names(gen_fl_test) <- cdl_fl_names_new[[1]]

test_that("Test General Dominance Computation: domir.formula_list", {
  expect_equal(test_fl_obj_new$General_Dominance, gen_fl_test
  )}
)

cyl_vs_cpt_fl <- 
  mean(c(cyl_fl_mgn[[3]] > vs_fl_mgn[[3]], 
             cyl_fl_mgn[[1]] > vs_fl_mgn[[1]])) 

carb_vs_cpt_fl <- 
  mean(c(carb_fl_mgn[[3]] > vs_fl_mgn[[2]], 
             carb_fl_mgn[[1]] > vs_fl_mgn[[1]]))

carb_cyl_cpt_fl <- 
  mean(c(carb_fl_mgn[[2]] > cyl_fl_mgn[[2]],
             carb_fl_mgn[[1]] > cyl_fl_mgn[[1]]))

cpt_fl_test <- 
  matrix(c(NA, cyl_vs_cpt_fl, carb_vs_cpt_fl, 
           1-cyl_vs_cpt_fl, NA, carb_cyl_cpt_fl, 
           1-carb_vs_cpt_fl, 1-carb_cyl_cpt_fl, NA),
         nrow = 3, ncol = 3)

dimnames(cpt_fl_test) <- list(
  paste0(cdl_fl_names_new[[1]], "_>"),
  paste0(">_", cdl_fl_names_new[[1]])
)

test_that("Test Complete Dominance Designation: domir.formula_list", {
  expect_equal(test_fl_obj_new$Complete_Dominance, cpt_fl_test
  )})

test_that("Test Overall Fit Statistic Value: domir.formula_list", {
  expect_equal(
    test_fl_obj_new$Value,
    (1 - logLik(systemfit(list(mpg ~ vs + cyl, qsec ~ carb), data = mtcars))[[1]]/baseline_ll)
  )})

#' @title A [`list`] composed of `formulas`
#' @name formula_list
#' @description
#' Defines a list object composed of `formula`s. The purpose of this 
#' class of object is to impose structure of the list to ensure that it 
#' can be used to obtain response-term pairs and will be able to be 
#' parsed in [`domir`].
#' @param ... `formula`s, possibly named
#' @details
#' The `formula_list` requires that each element of the list is a `formula` 
#' and that each `formula` is unique with a different, non-`NULL` 
#' dependent variable/response.
#' @return A `list` of class `formula_list`.
#' @rdname formula_list
#' @export
formula_list <- function(...) {
  .obj <- list(...)
  not_fml <- sapply(.obj, Negate(inherits), what = "formula")
  if (any(not_fml))
    stop(
      paste(c("List element", which(not_fml), "not of class 'formula'."),
            collapse = " "),
      call. = TRUE
    )
  DV_list <-
    sapply(
      .obj,
      function(elem) {
        tryCatch(
          (function() { if (attr(stats::terms(elem), "response")==1)
            as.list(attr(stats::terms(elem), "variables"))[[
              attr(stats::terms(elem), "response")+1
            ]]
            else NA
          })(),
          error = 
            function(err) 
              stop(deparse(elem), " is an invalid formula.", call. = FALSE)
        )
      }
    )
  DV_problem <- (is.na(DV_list) | duplicated(DV_list, incomparables = c(NA)))
  if (any(DV_problem))
    stop(
      paste(c("List element", which(DV_problem),
              "missing a response or duplicated a response."),
            collapse = " "),
      call. = TRUE
    )
  class(.obj) <- c("formula_list", "list")
  .obj
}
#' @title Translate `formula_list` into `Formula::Formula`
#' @name fmllst2Fml
#' @description
#' Translates [`formula_list`] objects into a [`Formula::Formula`]
#' @param fmllst A `formula_list` classed object.
#' @param drop_lhs An integer vector.
#'
#' Used as a selection vector to remove response names prior to
#' generating the `Formula` object. This vector must be composed of
#' integers (e.g., 1L and not 1).
#'
#' This is useful for some `Formulas` that do not have a separate
#' responses for each left hand side model part (e.g., [`pscl::zeroinfl`]) but 
#' are required to have separate responses by `formula_list`.
#' @return A `Formula::Formula` object.
#' @rdname fmllst2Fml
#' @export
fmllst2Fml <- function(fmllst, drop_lhs = NULL) {
  if (!inherits(fmllst, "formula_list"))
    stop("Submitted object is not of class 'formula_list'.", call. = FALSE)
  if (!requireNamespace("Formula"))
    stop("Package '{Formula}' not available.", call. = FALSE)
  list_parsed <-
    lapply(fmllst, formula_parse)
  if (!is.null(drop_lhs)) {
    if (!is.atomic(drop_lhs) || !is.integer(drop_lhs) || !length(drop_lhs)) {
      stop(paste(
        "'drop_lhs' is not a integer vector.",
        "'drop_lhs' is a(n)", class(drop_lhs), "of mode", mode(drop_lhs),
        "with a length of", length(drop_lhs), "."
      ),
      call. = FALSE)
    }
    if ( !all(drop_lhs %in% 1:length(list_parsed)) ) {
      bad_drop_lhs <-
        drop_lhs[which(!(drop_lhs %in% 1:length(list_parsed)))]
      stop(paste("Values",
                 paste(bad_drop_lhs, collapse = " "),
                 "in 'drop_lhs' are not valid element positions."),
           call. = FALSE)
    }
   
    keep_lhs <- which(!(1:length(list_parsed) %in% drop_lhs))
  } else {
    keep_lhs <- 1:length(list_parsed)
  }
 
  Fml <-
    paste(
      sapply(
        list_parsed[keep_lhs],
        function(elem) {
          elem$lhs_names
        }
      ),
      collapse = "|"
    )
 
  Fml <-
    paste(
      c(Fml, " ~ ",
        paste(
          sapply(
            list_parsed,
            function(elem) {
              rhs_sum <-
                paste(elem$rhs_names, collapse = "+")
              if (rhs_sum == "") rhs_sum <- "1"
              rhs_sum
            }
          ),
          collapse = "|"
        )
      ),
      collapse = ""
    )
 
  Formula::as.Formula(Fml)
 
}

library(pscl)
library(datasets)

# domir.formula and domin tests ----

## with sets ----

vs_mgn <- c( 
  cor(mtcars$mpg, predict(lm(mpg ~ vs, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ cyl, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ carb + am, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ cyl + carb + am, data = mtcars)))^2)

cyl_mgn <- c(
  cor(mtcars$mpg, predict(lm(mpg ~ cyl, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ cyl + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ carb + am, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs + carb + am, data = mtcars)))^2)

set_mgn <- c( 
  cor(mtcars$mpg, predict(lm(mpg ~ carb + am, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ vs + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ cyl + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ cyl, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl, data = mtcars)))^2)

vs_c <- c(vs_mgn[[1]], mean(vs_mgn[2:3]), vs_mgn[[4]])
cyl_c <- c(cyl_mgn[[1]], mean(cyl_mgn[2:3]), cyl_mgn[[4]])
set_c <- c(set_mgn[[1]], mean(set_mgn[2:3]), set_mgn[[4]])

cdl_names <- list(c("vs", "cyl", "set1"), paste0("IVs_", 1:3))
cdl_names_new <- list(c("vs", "cyl", "set1"), paste0("include_at_", 1:3))

cdl_test <- matrix(c(vs_c, cyl_c, set_c), nrow = 3, ncol = 3, 
                   byrow = TRUE, dimnames = cdl_names)

cdl_test_new <- matrix(c(vs_c, cyl_c, set_c), nrow = 3, ncol = 3, 
                   byrow = TRUE, dimnames = cdl_names_new)

test_obj <- domin(mpg ~ vs + cyl, "lm", list("summary", "r.squared"), 
                  data = mtcars, sets = list(c("carb", "am")))

test_obj_new <- domir(mpg ~ vs + cyl + carb + am, 
                      function(fml, data) {
                        res <- summary(lm(fml, data = data))
                        return(res[["r.squared"]])
                      },
                      data = mtcars, 
                      .set = list(~ carb + am))

test_that("Test Use of Sets with Conditional Dominance: domin", {
  expect_equal(test_obj$Conditional_Dominance, cdl_test
  )}
)

test_that("Test Use of Sets with Conditional Dominance: domir", {
  expect_equal(test_obj_new$Conditional_Dominance, cdl_test_new
  )}
)

all_test <- cor(mtcars$mpg, predict(lm(mpg ~ am, data = mtcars)))^2

vs_mgn2 <- c( 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + am, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ cyl + am, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ carb + am, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ cyl + carb + am, data = mtcars)))^2)

cyl_mgn2 <- c(
  cor(mtcars$mpg, predict(lm(mpg ~ cyl + am, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs + am, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ cyl + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ carb + am, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs + carb + am, data = mtcars)))^2)

carb_mgn2 <- c( 
  cor(mtcars$mpg, predict(lm(mpg ~ carb + am, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ vs + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs + am, data = mtcars)))^2,
  cor(mtcars$mpg, predict(lm(mpg ~ cyl + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ cyl + am, data = mtcars)))^2, 
  cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb + am, data = mtcars)))^2 - 
    cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + am, data = mtcars)))^2)

vs_c2 <- c(vs_mgn2[[1]] - all_test, mean(vs_mgn2[2:3]), vs_mgn2[[4]])
cyl_c2 <- c(cyl_mgn2[[1]] - all_test, mean(cyl_mgn2[2:3]), cyl_mgn2[[4]])
carb_c2 <- c(carb_mgn2[[1]] - all_test, mean(carb_mgn2[2:3]), carb_mgn2[[4]])

cdl_names2 <- list(c("vs", "cyl", "carb"), paste0("IVs_", 1:3))

cdl_names2_new <- list(c("vs", "cyl", "carb"), paste0("include_at_", 1:3))

cdl_test2 <- matrix(c(vs_c2, cyl_c2, carb_c2), nrow = 3, ncol = 3, 
                   byrow = TRUE, dimnames = cdl_names2)

cdl_test2_new <- matrix(c(vs_c2, cyl_c2, carb_c2), nrow = 3, ncol = 3, 
                    byrow = TRUE, dimnames = cdl_names2_new)

test_obj2 <- domin(mpg ~ vs + cyl + carb, "lm", list("summary", "r.squared"), 
                  data = mtcars, all = c("am"))

test_obj2_new <- domir(mpg ~ vs + cyl + carb + am, 
                      function(fml, data) {
                        res <- summary(lm(fml, data = data))
                        return(res[["r.squared"]])
                      },
                      data = mtcars, 
                      .all = ~ am)
## with all subests  ----
test_that("Test Use of All with Conditional Dominance: domin", {
  expect_equal(test_obj2$Conditional_Dominance, cdl_test2
  )}
)

test_that("Test Use of All with Conditional Dominance: domir", {
  expect_equal(test_obj2_new$Conditional_Dominance, cdl_test2_new
  )}
)

test_that("Test All Subsets Fitstat Value: domin", {
  expect_equal(test_obj2$Fit_Statistic_All_Subsets, all_test
  )}
)

test_that("Test All Subsets Fitstat Value: domir", {
  expect_equal(test_obj2_new$Value_All, all_test
  )}
)

test_obj3 <- domin(mpg ~ vs + cyl, "lm", list("summary", "r.squared"), 
                  data = mtcars, sets = list(c("carb", "am")), complete = FALSE)

test_obj3_new <- domir(mpg ~ vs + cyl + carb + am, 
                      function(fml, data) {
                        res <- summary(lm(fml, data = data))
                        return(res[["r.squared"]])
                      },
                      data = mtcars, #.cpt = FALSE,
                      .set = list(~ carb + am))

### confirm complete can be nullified ----

test_that("Test Complete Dominance as \"off\": domin", {
  expect_null(test_obj3$Complete_Dominance
  )})

# -- no longer used for domir --
# test_that("Test Complete Dominance as \"off\": domir", {
#   expect_null(test_obj3_new$Complete_Dominance
#   )})

## constant model + reverse ----

vs_cns_mgn <- c( extractAIC(lm(mpg ~ vs, data = mtcars))[[2]] - 
                   extractAIC(lm(mpg ~ 1, data = mtcars))[[2]],
                 extractAIC(lm(mpg ~ vs + cyl, data = mtcars))[[2]] - 
                   extractAIC(lm(mpg ~ cyl, data = mtcars))[[2]], 
                 extractAIC(lm(mpg ~ vs + carb, data = mtcars))[[2]] - 
                   extractAIC(lm(mpg ~ carb, data = mtcars))[[2]], 
                 extractAIC(lm(mpg ~ vs + cyl + carb, data = mtcars))[[2]] - 
                   extractAIC(lm(mpg ~ cyl + carb, data = mtcars))[[2]] )

cyl_cns_mgn <- c( extractAIC(lm(mpg ~ cyl, data = mtcars))[[2]]- 
                    extractAIC(lm(mpg ~ 1, data = mtcars))[[2]],
                 extractAIC(lm(mpg ~ vs + cyl, data = mtcars))[[2]] - 
                   extractAIC(lm(mpg ~ vs, data = mtcars))[[2]], 
                 extractAIC(lm(mpg ~ cyl + carb, data = mtcars))[[2]] - 
                   extractAIC(lm(mpg ~ carb, data = mtcars))[[2]], 
                 extractAIC(lm(mpg ~ vs + cyl + carb, data = mtcars))[[2]] - 
                   extractAIC(lm(mpg ~ vs + carb, data = mtcars))[[2]] )

carb_cns_mgn <- c( extractAIC(lm(mpg ~ carb, data = mtcars))[[2]]- 
                    extractAIC(lm(mpg ~ 1, data = mtcars))[[2]],
                  extractAIC(lm(mpg ~ vs + carb, data = mtcars))[[2]] - 
                    extractAIC(lm(mpg ~ vs, data = mtcars))[[2]], 
                  extractAIC(lm(mpg ~ cyl + carb, data = mtcars))[[2]] - 
                    extractAIC(lm(mpg ~ cyl, data = mtcars))[[2]], 
                  extractAIC(lm(mpg ~ vs + cyl + carb, data = mtcars))[[2]] - 
                   extractAIC(lm(mpg ~ vs + cyl, data = mtcars))[[2]] )

vs_cns_c <- c(vs_cns_mgn[[1]], mean(vs_cns_mgn[2:3]), vs_cns_mgn[[4]])
cyl_cns_c <- c(cyl_cns_mgn[[1]], mean(cyl_cns_mgn[2:3]), cyl_cns_mgn[[4]])
carb_cns_c <- c(carb_cns_mgn[[1]], mean(carb_cns_mgn[2:3]), carb_cns_mgn[[4]])

cdl_cns_names <- list(c("vs", "cyl", "carb"), paste0("IVs_", 1:3))
cdl_cns_names_new <- list(c("vs", "cyl", "carb"), paste0("include_at_", 1:3))

cdl_cns_test <- matrix(c(vs_cns_c, cyl_cns_c, carb_cns_c), nrow = 3, ncol = 3, 
                   byrow = TRUE, dimnames = cdl_cns_names)

cdl_cns_test_new <- matrix(c(vs_cns_c, cyl_cns_c, carb_cns_c), nrow = 3, ncol = 3, 
                       byrow = TRUE, dimnames = cdl_cns_names_new)

test_obj4 <- domin(mpg ~ vs + cyl + carb, lm, 
                   list(function(x) list(AIC = extractAIC(x)[[2]]), "AIC"), 
                   data=mtcars, reverse = TRUE, consmodel = "1") 

test_obj4_new <- domir(mpg ~ vs + cyl + carb, 
                       function(fml, data) {
                         res <- lm(fml, data = data)
                         return(extractAIC(res)[[2]])
                       },
                   data=mtcars, .rev = TRUE, .adj = TRUE) 

test_that("Test Use of Constant Model with Conditional Dominance: domin", {
  expect_equal(test_obj4$Conditional_Dominance, cdl_cns_test
  )}
)

test_that("Test Use of Adjustment with Conditional Dominance: domir", {
  expect_equal(test_obj4_new$Conditional_Dominance, cdl_cns_test_new
  )}
)

test_that("Test Constant Model Fitstat Value: domin", {
  expect_equal(test_obj4$Fit_Statistic_Constant_Model, 
               extractAIC(lm(mpg ~ 1, data = mtcars))[[2]]
  )}
)

test_that("Test Adjustment Value: domir", {
  expect_equal(test_obj4_new$Value_Adjust, 
               extractAIC(lm(mpg ~ 1, data = mtcars))[[2]]
  )}
)

cyl_vs_cpt <- mean(c(cyl_cns_mgn[[3]] < vs_cns_mgn[[3]], 
                         cyl_cns_mgn[[1]] < vs_cns_mgn[[1]]))

carb_vs_cpt <- mean(c(carb_cns_mgn[[3]] < vs_cns_mgn[[2]], 
                          carb_cns_mgn[[1]] < vs_cns_mgn[[1]]))

carb_cyl_cpt <- mean(c(carb_cns_mgn[[2]] < cyl_cns_mgn[[2]],
                           carb_cns_mgn[[1]] < cyl_cns_mgn[[1]]))

cpt_rev_test <- matrix(c(NA, cyl_vs_cpt, carb_vs_cpt, 
                     1-cyl_vs_cpt, NA, carb_cyl_cpt, 
                     1-carb_vs_cpt, 1-carb_cyl_cpt, NA),
                   nrow = 3, ncol = 3)

dimnames(cpt_rev_test) <- list(
  paste0(c("vs", "cyl", "carb"), "_>"),
  paste0(">_", c("vs", "cyl", "carb"))
)

dmn_trns <- function(val) {
  ifelse(val == 1, TRUE, ifelse(val == 0, FALSE, NA))
}
cpt_rev_test_dmn <- matrix(c(NA, dmn_trns(cyl_vs_cpt), dmn_trns(carb_vs_cpt), 
                         dmn_trns(1-cyl_vs_cpt), NA, dmn_trns(carb_cyl_cpt), 
                         dmn_trns(1-carb_vs_cpt), dmn_trns(1-carb_cyl_cpt), NA),
                       nrow = 3, ncol = 3)

dimnames(cpt_rev_test_dmn) <- list(
  paste0("Dmnates_", c("vs", "cyl", "carb")),
  paste0("Dmnated_", c("vs", "cyl", "carb"))
)

test_that("Test Reversed Complete Dominance Designation: domin", {
  expect_equal(test_obj4$Complete_Dominance, cpt_rev_test_dmn
  )})

test_that("Test Reversed Complete Dominance Designation: domir", {
  expect_equal(test_obj4_new$Complete_Dominance, cpt_rev_test
  )})

gen_rev_test <- rank(rowMeans(cdl_cns_test))

names(gen_rev_test) <- cdl_cns_names[[1]]

test_that("Test Reversed General Dominance Ranking: domin", {
  expect_equal(test_obj4$Ranks, gen_rev_test
  )}
)

test_that("Test Reversed General Dominance Ranking: domir", {
  expect_equal(test_obj4_new$Ranks, gen_rev_test
  )}
)

test_obj_nocdl <- domin(mpg ~ vs + cyl, "lm", list("summary", "r.squared"), 
                  data = mtcars, sets = list(c("carb", "am")), conditional = FALSE)

test_obj_nocdl_new <- domir(mpg ~ vs + cyl + carb + am, 
                            function(fml, data) {
                              res <- summary(lm(fml, data = data))
                              return(res[["r.squared"]])
                            },
                            data = mtcars, 
                            .set = list(~ carb + am)) #, 
                            #.cdl = FALSE)

### confirm conditional can be nullified ----

test_that("'Conditional is False' General Dominance: domin", {
  expect_equal(test_obj_nocdl$General_Dominance, test_obj$General_Dominance
  )}
)

# -- no longer used for domir --
# test_that("'Conditional is False' General Dominance: domir", {
#   expect_equal(test_obj_nocdl_new$General_Dominance, test_obj_new$General_Dominance
#   )}
# )

# domir.formula_list ----

adj_mtcars <- 
  transform(mtcars, 
            zi = am*gear)

adj_val <- 
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ 1, infert ~ 1), drop_lhs = 2L),
      data = adj_mtcars) )[[1]]

all_val <- 
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ wt, infert ~ 1), drop_lhs = 2L),
      data = adj_mtcars) )[[1]]
    
vsdrat_fl_mgn <- c( 
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ vs + drat + wt, infert ~ 1), drop_lhs = 2L),
      data = adj_mtcars))[[1]],
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ vs + drat + cyl + hp + wt, infert ~ 1), drop_lhs = 2L),
      data = adj_mtcars))[[1]] - 
    logLik(
      zeroinfl(
        fmllst2Fml(formula_list(zi ~ cyl + hp + wt, infert ~ 1), drop_lhs = 2L),
        data = adj_mtcars))[[1]],
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ vs + drat + wt, infert ~ carb), drop_lhs = 2L),
      data = adj_mtcars))[[1]] - 
    logLik(
      zeroinfl(
        fmllst2Fml(formula_list(zi ~ wt, infert ~ carb), drop_lhs = 2L),
        data = adj_mtcars))[[1]],
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ vs + drat + cyl + hp + wt, infert ~ carb), drop_lhs = 2L),
      data = adj_mtcars))[[1]] - 
    logLik(
      zeroinfl(
        fmllst2Fml(formula_list(zi ~ cyl + hp + wt, infert ~ carb), drop_lhs = 2L),
        data = adj_mtcars))[[1]] )

cylhp_fl_mgn <- c( 
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ cyl + hp + wt, infert ~ 1), drop_lhs = 2L),
      data = adj_mtcars))[[1]],
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ vs + drat + cyl + hp + wt, infert ~ 1), drop_lhs = 2L),
      data = adj_mtcars))[[1]] - 
    logLik(
      zeroinfl(
        fmllst2Fml(formula_list(zi ~ vs + drat + wt, infert ~ 1), drop_lhs = 2L),
        data = adj_mtcars))[[1]],
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ cyl + hp + wt, infert ~ carb), drop_lhs = 2L),
      data = adj_mtcars))[[1]] - 
    logLik(
      zeroinfl(
        fmllst2Fml(formula_list(zi ~ wt, infert ~ carb), drop_lhs = 2L),
        data = adj_mtcars))[[1]],
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ vs + drat + cyl + hp + wt, infert ~ carb), drop_lhs = 2L),
      data = adj_mtcars))[[1]] - 
    logLik(
      zeroinfl(
        fmllst2Fml(formula_list(zi ~ vs + drat + wt, infert ~ carb), drop_lhs = 2L),
        data = adj_mtcars))[[1]] )

carbi_fl_mgn <- c( 
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ wt, infert ~ carb), drop_lhs = 2L),
      data = adj_mtcars))[[1]],
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ vs + drat + wt, infert ~ carb), drop_lhs = 2L),
      data = adj_mtcars))[[1]] - 
    logLik(
      zeroinfl(
        fmllst2Fml(formula_list(zi ~ vs + drat + wt, infert ~ 1), drop_lhs = 2L),
        data = adj_mtcars))[[1]],
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ cyl + hp + wt, infert ~ carb), drop_lhs = 2L),
      data = adj_mtcars))[[1]] - 
    logLik(
      zeroinfl(
        fmllst2Fml(formula_list(zi ~ cyl + hp + wt, infert ~ 1), drop_lhs = 2L),
        data = adj_mtcars))[[1]],
  logLik(
    zeroinfl(
      fmllst2Fml(formula_list(zi ~ vs + drat + cyl + hp + wt, infert ~ carb), drop_lhs = 2L),
      data = adj_mtcars))[[1]] - 
    logLik(
      zeroinfl(
        fmllst2Fml(formula_list(zi ~ vs + drat + cyl + hp + wt, infert ~ 1), drop_lhs = 2L),
        data = adj_mtcars))[[1]] )

vsdrat_fl_c <- c(vsdrat_fl_mgn[[1]] - all_val, mean(vsdrat_fl_mgn[2:3]), vsdrat_fl_mgn[[4]])
cylhp_fl_c <- c(cylhp_fl_mgn[[1]] - all_val, mean(cylhp_fl_mgn[2:3]), cylhp_fl_mgn[[4]])
carbi_fl_c <- c(carbi_fl_mgn[[1]] - all_val, mean(carbi_fl_mgn[2:3]), carbi_fl_mgn[[4]])

cdl_fl_names_set <- list(c("infert~carb", "set1", "set2"), paste0("include_at_", 1:3))

cdl_fl_test_set <- matrix(c(carbi_fl_c, vsdrat_fl_c, cylhp_fl_c), nrow = 3, ncol = 3, 
                          byrow = TRUE, dimnames = cdl_fl_names_set)

test_fl_obj_set <- 
  domir(formula_list(
    zi ~ vs + drat + cyl + hp + wt, 
    infert ~ carb), 
    function(fml, data) {
      
      Fml <- 
        fmllst2Fml(fml, drop_lhs = 2L)
      
      logLik(
        zeroinfl(Fml, data = data)
      )[[1]]
      
    },
    .set = 
      list(formula_list(zi ~ vs + drat), formula_list(zi ~ cyl + hp)),
    .all = formula_list(zi ~ wt),
    data = adj_mtcars,
    .adj = TRUE)

test_that("Test Use of Sets with Conditional Dominance: domir.formula_list", {
  expect_equal(test_fl_obj_set$Conditional_Dominance, cdl_fl_test_set
  )}
)

test_that("Test All Subsets Fitstat Value: domir.formula_list", {
  expect_equal(test_fl_obj_set$Value_All, (all_val - adj_val)
  )}
)

test_that("Test Adjustment Fitstat Value: domir.formula_list", {
  expect_equal(test_fl_obj_set$Value_Adjust, adj_val
  )}
)

#' @title Dominance analysis methods
#' @name domir
#' @description
#' Parses input object to obtain a list of names, determines which 
#' combinations of the names are valid, submits valid name 
#' combinations to a function, and computes dominance 
#' values based on the returned values from the function.
#' @param .obj A `formula` or `formula_list`.
#' Parsed to produce a list of names. All valid combinations names from 
#' the list are [`sapply`]-ed to `.fct`.
#' 
#' The combinations of names submitted to `.fct` are formatted to be of
#' the same [`class`] as `.obj` and must be submitted to
#' `.fct` as its first, unnamed argument.
#' @param .fct A [`function`] or string function name.
#' Applied to all valid combinations of names parsed from `.obj`.
#' Must return a length 1/scalar, numeric, atomic vector.
#' @param .set A `list`.
#' Each element of the list must be the same class as `.obj`.
#' Elements of the list can be named.
#' Names parsed from the elements of the list must also be in `.obj`.
#' @param .wst A `list`.
#' Each element of the list must be the same class as `.obj`.
#' Names parsed from the elements of the list must also be in `.obj`.
#' @param .all A `formula` or `formula_list`.
#' Must be the same class as `.obj`.
#' Parsed names must also be in `.obj`.
#' @param .adj Logical.
#' If `TRUE` then a model including only an intercept is submitted to `.fct`
#' and the value returned is subtracted from the values returned from all
#' subsets in the dominance analysis.
#' @param .cdl `NULL`.
#' Depreciated. Use `.cdl = FALSE` in `domir`'s `print` method to suppress 
#' display of conditional dominance values.
#' @param .cpt `NULL`.
#' Depreciated. Use `.cpt = FALSE` in `domir`'s `print` method to suppress 
#' display of complete dominance values.
#' @param .rev Logical.
#' If `TRUE` then standardized vector, ranks, and complete dominance
#' designations are reversed in their interpretation.
#' @param .cst Object of class c("SOCKcluster", "cluster") from
#' [`parallel-package`].
#'
#' When non-`NULL`, will alter the method for collecting values from all
#' combinations of names from using [`sapply`] to [`parallel::parSapply`].
#' @param .prg Logical.
#' If `TRUE` then a progress bar is displayed during collection of values
#' to indicate progress.
#' @param ... Passes arguments to other methods during method dispatch;
#' passes arguments to the function in `.fct` during function execution.
#' @return Returns an object of [`class`] "domir" composed of:
#' \describe{
#'  \item{`General_Dominance`}{Vector of general dominance values.}
#'  \item{`Standardized`}{Vector of general dominance values standardized
#'  to sum to 1.}
#'  \item{`Ranks`}{Vector of ranks applied to the general dominance values.}
#'  \item{`Conditional_Dominance`}{Matrix of conditional dominance values.
#'  Each row represents a name in `.obj`;
#'  each column represents a number of names included in `.fct`.}
#'  \item{`Complete_Dominance`}{Matrix of proportions of subsets where the
#'  name in the row has a larger value than the name in the column.
#'  These proportions determine complete dominance when a value of
#'  1 or 0.}
#'  \item{`Value`}{Value returned by `.fct` with all names included.}
#'  \item{`Value_All`}{Value of `.fct` associated with names included
#'  in `.all`;
#'  when `.adj` is `TRUE`, this value will be adjusted for `Value_Adjust`.}
#'  \item{`Value_Adjust`}{Value of `.fct` returned when no names are included.}
#'  \item{`Call`}{The matched call.}
#' }
#' @details
#' ## Name Parsing
#' `.obj` is first parsed into a list of names.
#' How the name list is parsed depends on `.obj`'s class.
#' ### `formula`
#' The `formula` method creates a name list using all terms in the formula.
#' The terms are obtained using [`terms.formula`]. 
#' All processing that is normally applied to the right hand side of a 
#' formula is implemented (see [`formula`]).
#'
#' A response/left hand side is not required but, if present, is
#' included in all `formula`s passed to `.fct`.
#' ### `formula_list`
#' The [`formula_list`] creates a name list out of response-term pairs.
#' The terms are obtained using `terms.formula` applied to each individual
#' formula in the list.
#'
#' The `formula_list` methods make it possible to implement dominance analysis
#' for parameter estimates in multivariate predictive models
#' (e.g., Luchman, Xie, & Kaplan, 2020).
#' ### Additional Details
#' `formula`s and `formula_list` elements are assumed to have an intercept
#' except if explicitly removed with a `- 1` in the `formula`(s) in `.obj`.
#' If removed, the intercept will be removed in all `formula`(s) in each
#' `sapply`-ed subset to `.fct`.
#' If [`offset`]s are included, they are passed, like intercepts, while
#' `sapply`-ing subsets to `.fct`.
#' ## Changing Combination Generation
#' Parsed names are used independently for creating combinations of names 
#' to submit to`.fct`. 
#' When using `.set`, `.wst`, and `.all`, the way combinations are 
#' created changes.
#' Names in `.set`, `.wst`, and `.all` must also be present in `.obj`.
#' ### `.set`
#' `.set` binds together names such that they are considered to be one name 
#' when creating combinations. 
#' The names in each `.set` then pool their returned value together with the 
#' other members of their set.
#'
#' By default, sets are referred to by set names in all output.
#' Unnamed `.set` list elements are called 'set@' where '@' is an integer 
#' indicating the set's position in the list. 
#' The user can give each set in the `.set` list a name that will be used in 
#' place of the default.
#' Names of `.set`s cannot contain the period character '.'.
#'
#' `.set` implements the grouped dominance analysis method described by
#' Luchman (2026).
#' ### `.wst`
#' `.wst` binds together names such that they are considered to be one name 
#' when creating combinations with other names that are not in their `.wst`.
#' All combinations of names of the members of names within a `.wst` 
#' are considered valid.
#' The names in each `.wst` then pool their returned value together with the 
#' other members of their set when compared to names outside of the `.wst` 
#' but do not pool their returned value when compared to names within their
#' `.wst`.
#' 
#' `.wst` list elements cannot be named.
#'
#' `.wst` implements the within-group dominance analysis method described by
#' Luchman (2026).
#' `.wst` and `.set` can be used together.
#' ### `.all`
#' `.all` binds together names such that they are considered to be one name 
#' when creating combinations and are always included first, before all 
#' other names.
#' The names in `.all` will then be included in 'all' valid combinations 
#' of names.
#' The names in `.all` are included after the combination with no names but 
#' before any other combinations.
#' The value associated with `.all` is also removed from the 
#' dominance analysis and is reported along with the overall value 
#' including all names.
#'
#' The `formula` method for `.all` does not allow the formula in `.all` to have
#' a left hand side.
#' ### `.adj`
#' By default, `domir` assumes that the combination of no names has a 
#' value of 0.
#' `.adj` indicates that the there is an 'adjustment' needed such that the 
#' no names combination has a non-0 value and an intercept-only model should 
#' be supplied to `.fct`.
#'
#' The `formula` method will submit an intercept-only formula to `.fct`.
#' The `formula_list` method creates a separate, intercept-only subset for each
#' of the `formula`s in the list.
#' Both the `formula` and `formula_list` methods will respect the user's
#' removal of an intercept and or inclusion of an `offset`.
#' ### Additional Details
#' All methods submit combinations of names as an object of the same class as
#' `.obj`. 
#' A `formula` in `.obj` will submit all combinations of names as
#' `formula`s to `.fct`. 
#' A `formula_list` in `.obj` will submit all
#' combinations of subsets of names as `formula_list`s to `.fct`.
#' In the case that `.fct` requires a different `class` (e.g.,
#' a character vector of names, a [`Formula::Formula`] see [`fmllst2Fml`]) the
#' subsets of names will have to be processed in `.fct` to obtain the correct
#' `class`.
#'
#' The `formula` or `formula_list` of names will be submitted to `.fct` as the 
#' first, unnamed argument.
#' ## `.fct` as Analysis Pipeline
#' `.fct` is expected to be a complete analysis pipeline that receives a
#' subset of names of the same `class` as `.obj` and uses these names in the
#' `class` as submitted to generate a returned value of the appropriate
#' type to dominance analyze.
#' At current, only atomic (i.e., non-`list`), numeric scalars (i.e.,
#' vectors of length 1) are allowed as returned values.
#' `domir` is designed for use with predictive models and assumes the returned 
#' value is a scalar-valued fit statistic/metric.
#'
#' The `.fct` argument is strict about names submitted and returned value
#' requirements for functions used. 
#' A series of checks to ensure the submitted names and returned value adhere 
#' to these requirements.
#' The checks include whether the `.obj` can be submitted to `.fct` without
#' producing an error and whether the returned value from `.fct` is a length 1,
#' atomic, numeric vector.
#' In most circumstances, the user will have to make their own named or
#' anonymous function to supply as `.fct` to satisfy the checks.
#' # Notes
#' Prior to version 1.1.0, the `formula` method allowed a `formula`
#' to be submitted to `.adj`.
#' Submitting any argument other than a logical is now defunct.
#'
#' The `formula` and `formula_list` methods can be used to pass responses,
#' intercepts, and `offset`s to all combinations of names.
#' If the user seeks to include other model components integral to
#' estimation (i.e., a random effect term in [`lme4::glmer()`]) include them as
#' [`update`][update.formula] to the submitted `formula` or `formula_list`
#' embedded in `.fct`.
#'
#' Second-order or higher terms (i.e., interactions like `~ a*b`) are parsed
#' by default but not used differently from first-order terms for generating 
#' valid combinations. 
#' The values ascribed to such names may not be valid unless the user ensures 
#' that second-order and higher term names are used appropriately in `.fct`.
#' @export
#' @examples
#' ## Linear model returning r-square
#' lm_r2 <-
#'   function(fml, data) {
#'     lm_res <- lm(fml, data = data)
#'     summary(lm_res)[["r.squared"]]
#'  }
#'
#' domir(mpg ~ am + vs + cyl, lm_r2, data = mtcars)
#'
#' ## Linear model including set
#' domir(
#'   mpg ~ am + vs + cyl + carb + gear + disp + wt,
#'   lm_r2,
#'   .set = list(~ carb + gear, ~ disp + wt),
#'   data = mtcars
#' )
#'
#' ## Multivariate regression with multivariate r-square and
#' ## all subsets variable
#' if (requireNamespace("performance", quietly = TRUE)) {
#'   mlm_rxy <-
#'     function(fml, data) {
#'       mlm_res <- lm(fml, data = data)
#'       performance::r2_mlm(mlm_res)[["Symmetric Rxy"]]
#'     }
#'
#'   domir(
#'     cbind(wt, mpg) ~ vs + cyl + am + carb,
#'     mlm_rxy,
#'     .all = ~ carb,
#'     data = mtcars
#'   )
#' }
#'
#' ## Named sets
#' domir(
#'   mpg ~ am + gear + cyl + vs + qsec + drat,
#'   lm_r2,
#'   data = mtcars,
#'   .set =
#'     list(
#'       trns = ~ am + gear,
#'       eng = ~ cyl + vs,
#'       misc = ~ qsec + drat
#'     )
#' )
#'
#' ## Linear model returning AIC
#' lm_aic <-
#'   function(fml, data) {
#'     lm_res <- lm(fml, data = data)
#'     AIC(lm_res)
#'  }
#'
#' domir(
#'   mpg ~ am + carb + cyl,
#'   lm_aic,
#'   .adj = TRUE,
#'   .rev = TRUE,
#'   data = mtcars
#'  )
#'
#' ## 'systemfit' with 'formula_list' method returning AIC
#' if (requireNamespace("systemfit", quietly = TRUE)) {
#'   domir(
#'     formula_list(mpg ~ am + cyl + carb, qsec ~ wt + cyl + carb),
#'     function(fml) {
#'       res <- systemfit::systemfit(fml, data = mtcars)
#'       AIC(res)
#'     },
#'     .adj = TRUE, .rev = TRUE
#'   )
#' }
#'
#' ## within-set or within-group dominance analysis
#' domir(
#'   mpg ~ am + gear + cyl + vs + qsec + drat,
#'   lm_r2,
#'   data = mtcars,
#'   .wst =
#'     list(
#'       ~ am + gear,
#'       ~ cyl + vs,
#'       ~ qsec + drat
#'     )
#' )
#' @references
#' \itemize{
#' \item Luchman, J. N., Lei, X., & Kaplan, S. (2020). Relative Importance
#' Analysis with Multivariate Models: Shifting the Focus from Independent
#' Variables to Parameter Estimates. Journal of Applied Structural Equation
#' Modeling, 4(2), 1-20. doi:https://doi.org/10.47263/JASEM.4(2)02
#' \item Luchman, J. N. (2026). Determining Relative Importance with
#' Independent Variable Groups: An Alternative Dominance Analysis Method.
#' Journal of Behavioral Data Science, 6(1), 1-26.
#' doi:https://doi.org/10.35566/jbds/luchman
#'}
#'
domir <- function(.obj, ...) {
  UseMethod("domir")
}
#' @rdname domir
#' @exportS3Method
domir.formula <- function(
    .obj, .fct,
    .set = NULL, .wst = NULL,
    .all = NULL, .adj = FALSE,
    .cdl = NULL, .cpt = NULL,
    .rev = FALSE, .cst = NULL, .prg = FALSE, ...) {
  domir_arg_checker(.rev, .cpt, .cdl, .prg, .cst)
  fml_parsed <- formula_parse(.obj)
  if (length(fml_parsed$rhs_names) == 0)
    stop("The formula in '.obj' must have one or more terms.", call. = FALSE)
  entire_namelist_value <- formula_output_check(.obj, .fct, TRUE, ...)
  adj_checker(.adj, fml_parsed, TRUE)
  adj_value <- est_adj_value(.adj, fml_parsed, .fct, TRUE, ...)
  all_parsed <- all_checker(.all, fml_parsed, TRUE)
  fml_parsed <- fml_all_update(all_parsed, fml_parsed, TRUE)
  all_value <- est_all_value(.all, fml_parsed, .fct, adj_value, TRUE, ...)
  set_checker(.set, fml_parsed, "'.set'", TRUE)
  sets_parsed <- set_element_checker(.set, fml_parsed, "'.set'", TRUE)
  set_checker(.wst, fml_parsed, "'.wst'", TRUE)
  wsts_parsed <- set_element_checker(.wst, fml_parsed, "'.wst'", TRUE)
  check_namelists(fml_parsed, sets_parsed, wsts_parsed, all_parsed, TRUE)
  names_for_dominance <-
    determine_dominance_names(fml_parsed, sets_parsed, wsts_parsed, TRUE)
  return_list <-
    dominance_scalar(
      fml_parsed, .fct, names_for_dominance, entire_namelist_value,
      adj_value, all_value,
      .rev, .cst, .prg, list(...), FALSE)
  names_for_printing <- determine_display_names(names_for_dominance, .set)
  return_list <- name_return_list(return_list, names_for_printing)
  if (!.adj) adj_value <- NULL
  if (is.null(.all)) all_value <- NULL
  return_list <-
    append(
      return_list,
    list(
      Value = entire_namelist_value,
      Value_All = all_value,
      Value_Adjust = adj_value,
      Call = match.call()
    )
  )
  class(return_list) <- c("domir")
  return_list
}
#' @rdname domir
#' @exportS3Method
domir.formula_list <- function(
    .obj, .fct,
    .set = NULL, .wst = NULL, .all = NULL, .adj = FALSE,
    .cdl = NULL, .cpt = NULL,
    .rev = FALSE, .cst = NULL, .prg = FALSE, ...) {
  domir_arg_checker(.rev, .cpt, .cdl, .prg, .cst)
  fmllst_parsed <- lapply(.obj, formula_parse)
  rhs_term_counts <- 
    sapply(fmllst_parsed, function(elem) length(elem$rhs_names))
  if (any(rhs_term_counts == 0)) {
    stop(
      paste("Each formula in '.obj' must have one or more terms.",
            "Formulas", paste(which(rhs_term_counts == 0), collapse = " "),
            "have no terms."),
      call. = FALSE
      )
  }
  entire_namelist_value <- formula_output_check(.obj, .fct, FALSE, ...)
  adj_checker(.adj, fmllst_parsed, FALSE)
  adj_value <- est_adj_value(.adj, fmllst_parsed, .fct, FALSE, ...)
  all_parsed <- all_checker(.all, fmllst_parsed, FALSE)
  fmllst_parsed <- fml_all_update(all_parsed, fmllst_parsed, FALSE)
  all_value <- est_all_value(.all, fmllst_parsed, .fct, adj_value, FALSE, ...) 
  set_checker(.set, fmllst_parsed, "'.set'", FALSE)
  sets_parsed <- set_element_checker(.set, fmllst_parsed, "'.set'", FALSE)
  set_checker(.wst, fmllst_parsed, "'.wst'", FALSE)
  wsts_parsed <- set_element_checker(.wst, fmllst_parsed, "'.wst'", FALSE)
  check_namelists(fmllst_parsed, sets_parsed, wsts_parsed, all_parsed, FALSE)
  names_for_dominance <-
    determine_dominance_names(fmllst_parsed, sets_parsed, wsts_parsed, FALSE)
  return_list <-
    dominance_scalar(
      fmllst_parsed, .fct, names_for_dominance, entire_namelist_value,
      adj_value, all_value,
      .rev, .cst, .prg, list(...), TRUE)
  names_for_printing <- determine_display_names(names_for_dominance, .set)
  return_list <- name_return_list(return_list, names_for_printing)
  if (!.adj) adj_value <- NULL
  if (is.null(.all)) all_value <- NULL
  return_list <-
    append(
      return_list,
      list(
        Value = entire_namelist_value,
        Value_All = all_value,
        Value_Adjust = adj_value,
        Call = match.call()
      )
    )
  class(return_list) <- c("domir")
  return_list
}
#' @title Formula parsing function
#' @description Internal formula parsing function to facilitate
#' re-construction of a formula in an external function using `reformulate()`.
#' @noRd
#' @param .obj A `formula`.
#' @returns A list composed of:
#' \describe{
#'  \item{`rhs_names`}{Character vector of names from the RHS/right hand side
#'  of the formula.}
#'  \item{`lhs_names`}{`call` vector of names from the LHS/left hand side
#'  of the formula. Note that this element is not a character vector.}
#'  \item{`intercept_lgl`}{Logical vector indicating whether the `formula`
#'  has an intercept.}
#'  \item{`offset`}{Character vector of offset terms to be included in the
#'  reconstructed `formula`.}
#'  \item{`select_lgl`}{Logical vector for use by `domir` to indicate whether
#'  names from the `rhs_names` list will be included in a submodel.}
#' }
formula_parse <- function(.obj) {
  if (is.null(.obj)) return(NULL)
  rhs_names <-
    tryCatch(
      attr(stats::terms(.obj), "term.labels"),
      error = function(err) {
        stop(deparse(.obj), " is an invalid formula.", call. = FALSE)
      }
    )
  intercept_lgl <- as.logical(attr(stats::terms(.obj), "intercept"))
  if (!is.null(attr(stats::terms(.obj), "offset"))) {
    offset_locs <- attr(stats::terms(.obj), "offset")
    offset <- rownames(attr(stats::terms(.obj), "factors"))[offset_locs]
    if (is.null(offset))
      offset <-
      sapply(
        (offset_locs + 1),
        function(loc) attr(stats::terms(.obj), "variables")[[loc]]
      )
  } else {
    offset <- NULL
  }
  if (attr(stats::terms(.obj), "response") == 1) {
    lhs_names <- attr(stats::terms(.obj), "variables")[[2]]
  } else {
    lhs_names <- NULL
  }
  select_lgl <- rep(FALSE, times = length(rhs_names))
  list(rhs_names = rhs_names,
       lhs_names = lhs_names,
       intercept_lgl = intercept_lgl,
       offset = offset,
       select_lgl = select_lgl)
}
#' @title formula output checking function
#' @description Internal function which ensures that `.fct` does not
#' produce errors when `.obj` is applied to it and that `.fct` produces an
#' atomic scalar-valued numeric result.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param .obj A `formula` or `formula_list`.
#' @param .fct A `function` or string function name.
#' @param .is_fml Logical.
#' @param ... Passes arguments to the function in `.fct`.
#' @returns Result of the `.fct` call using `.obj`.
formula_output_check <- function(.obj, .fct, .is_fml, ...) {
  if (.is_fml) {
    evaluate_fml <- do.call(eval(.fct), append(.obj, list(...)))
  } else {
    evaluate_fml <- do.call(eval(.fct), append(list(.obj), list(...)))
  }
  test_model <-
    tryCatch(
      evaluate_fml,
      error = function(err) {
        stop("'.fct' produced an error when applied to '.obj'.\n",
             "The error was:\n", err, call. = FALSE)
      }
    )
  if (!is.numeric(test_model) || !is.vector(test_model) ||
      !is.atomic(test_model) || length(test_model) != 1)
    stop("Result of '.fct' is not an atomic, numeric, scalar object ",
         "(vector with a 'length()' value of 1).", call. = FALSE)
  test_model
}
#' @title `domir` argument checking function
#' @description Internal function to ensure that the arguments unrelated to
#' names are formulated correctly.
#' @noRd
#' @param .rev Logical.
#' @param .cpt `NULL`.
#' @param .cdl `NULL`.
#' @param .prg Logical.
#' @param .cst Object of class c("SOCKcluster", "cluster") from
#' [`parallel-package`]
#' @returns `NULL`
domir_arg_checker <-
  function(.rev, .cpt, .cdl, .prg, .cst) {
    lgl_args <- sapply(list(.rev, .prg), is.logical)
    if (!all(lgl_args))
      stop(
        paste(
          c(".rev", ".prg")[which(!lgl_args)],
          collapse = " "
        ),
        " must be logical.", call. = FALSE
      )
    if (!all(c(is.null(.cpt), is.null(.cdl))))
      warning("'.cpt' and '.cdl' are depreciated arguments to 'domir' as of ",
              "version 1.3.\nUse '.cdl' and '.cpt' as arguments to 'print()' ",
              "instead.", call. = FALSE)
    if (!is.null(.cst) && .prg)
      stop("Progress bars do not yet work with parallelized value ",
           "estimation.", call. = FALSE)
    if (!is.null(.cst) && !inherits(.cst, "SOCKcluster"))
      stop(
        "Object in '.cst' not a cluster from package 'parallel'.",
        call. = FALSE
      )
    NULL
  }
#' @title `.adj` argument checking function
#' @description Internal function which ensures that the arguments to `.adj`
#' methods are formulated correctly.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param .adj Logical.
#' @param fml_parsed `formula` or `formula_list` processed with
#' `formula_parse()`.
#' @param .is_fml Logical.
#' @returns `NULL`
adj_checker <- function(.adj, fml_parsed, .is_fml) {
  if (!is.logical(.adj) || (length(.adj) > 1))
    stop("'.adj' argument must be logical of length 1.", call. = FALSE)
  if (.is_fml) fml_parsed <- list(fml_parsed)
  rmv_intercept_locs <- sapply(fml_parsed, function(elem) elem$intercept_lgl)
  if (any(!rmv_intercept_locs) && .adj)
    stop("'.adj' cannot be estimated with intercepts removed from '.obj'.", 
         call. = FALSE)
  NULL
}
#' @title Return an intercept-only or 'no names' value
#' @description Internal function to estimates the value associated with no
#'  value generating names included.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param .adj Logical.
#' @param fml_parsed `formula` or `formula_list` processed with
#' `formula_parse()`.
#' @param .fct A `function` or string function name.
#' @param .is_fml Logical.
#' @param ... Passes arguments to the function in `.fct`.
#' @returns Result of `.fct` call applied to `.obj` with no names.
est_adj_value <- function(.adj, fml_parsed, .fct, .is_fml, ...) {
  if (.adj && .is_fml) {
    fml <-
      stats::reformulate(
        c("1", fml_parsed$offset),
        response = fml_parsed$lhs_names,
        intercept = fml_parsed$intercept_lgl)
    value <- formula_output_check(fml, .fct, .is_fml, ...)
  } else if (.adj && !.is_fml) {
    fml_lst <-
      lapply(
        fml_parsed,
        function(fml) {
          stats::reformulate(
            c("1", fml$offset),
            response = fml$lhs_names,
            intercept = fml$intercept_lgl)
        }
      )
    fml_lst <- do.call("formula_list", fml_lst)
    value <- formula_output_check(fml_lst, .fct, .is_fml, ...)
  } else {
    value <- 0
  }
  value
}
#' @title `.all` argument checking and parsing function
#' @description Internal function to check `.all` argument and process it using
#' `formula_parse()`.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param .all A `formula` or `formula_list`.
#' @param fml_parsed `formula` or `formula_list` processed with
#' `formula_parse()`.
#' @param .is_fml Logical.
#' @returns A `formula_parse` object applied to `.all` object.
all_checker <- function(.all, fml_parsed, .is_fml) {
  if (is.null(.all)) return(NULL)
  if (!inherits(.all, "formula") && .is_fml) {
    stop("'.all' must be a 'formula'.", call. = FALSE)
  } else if (!inherits(.all, "formula_list") && !.is_fml) {
    stop("'.all' must be a 'formula_list'.", call. = FALSE)
  }
  if (.is_fml) {
    all_pre_parse <- list(.all)
  } else {
    all_pre_parse <- .all
  }
  all_parsed <- lapply(all_pre_parse, function(elem) formula_parse(elem))
  if (any(sapply(all_parsed, function(elem) length(elem$rhs_names) == 0)))
    stop("Formulas in '.all' must have one or more terms.", call. = FALSE)
  if (
    any(sapply(all_parsed, function(elem) !is.null(elem$lhs_names))) && .is_fml
  ) {
    stop("Left hand side names not allowed in '.all' formulas.", call. = FALSE)
  } else if (.is_fml) {
    all_parsed[[1]]$lhs_names <- fml_parsed$lhs_names
  }
  if (any(sapply(all_parsed, function(elem) !is.null(elem$offset))))
    stop("Offsets not allowed in '.all' formulas.", call. = FALSE)
  if (any(sapply(all_parsed, function(elem) !elem$intercept_lgl)))
    stop("Removing intercepts not allowed in '.all' formulas.", call. = FALSE)
  if (.is_fml) all_parsed <- all_parsed[[1]]
  all_parsed
}
#' @title Adjusts `fml_parsed` object for names in `.all`
#' @description Internal function to adjust `.obj` object
#' processed by `formula_parse()` to include names in `.all` in all subsets
#' of names.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param .all A `formula` or `formula_list`.
#' @param fml_parsed `formula` or `formula_list` processed with
#' `formula_parse()`.
#' @param .is_fml Logical.
#' @returns An updated `formula_parse` object.
fml_all_update <- function(all_parsed, fml_parsed, .is_fml) {
  if (is.null(all_parsed)) return(fml_parsed)
  else if (.is_fml) {
    fml_parsed <- list(fml_parsed)
    all_parsed <- list(all_parsed)
  }
  invalid_all_names <-
    setdiff(
      Reduce(
        union,
        sapply(
          all_parsed,
          function(elem)
            paste(deparse(elem$lhs_names), "~",
                  elem$rhs_names)), NULL),
      Reduce(
        union,
        sapply(
          fml_parsed,
          function(elem) 
            paste(deparse(elem$lhs_names), "~", elem$rhs_names)), NULL)
    )
  if (length(invalid_all_names) > 0) {
    invalid_all_names <-
      ifelse(
        .is_fml,
        paste(
          sapply(
            invalid_all_names,
            function(elem)
              substr(
                elem,
                nchar(deparse(all_parsed[[1]]$lhs_names)) + 4,
                nchar(elem))
          ),
          collapse = ", "
        ),
        paste(invalid_all_names, collapse = ", ")
      )
    stop("Name(s) in '.all' not found in '.obj' formulas.",
         call. = FALSE)
  }
  fml_parsed <-
    lapply(
      fml_parsed,
      function(elem) {
        lhs_names_all <-
          unlist(sapply(all_parsed, function(el) deparse(el$lhs_names)))
        pairs_all <-
          Reduce(
            union,
            sapply(
              all_parsed,
              function(el)
                paste(deparse(el$lhs_names), "~",
                      el$rhs_names)), NULL)
        if (deparse(elem$lhs_names) %in% lhs_names_all) {
          iv_dv_pairs <-
            paste(deparse(elem$lhs_names), "~", elem$rhs_names)
          which_to_true <- iv_dv_pairs %in% pairs_all
          elem_adj <- elem
          elem_adj$select_lgl <- which_to_true
          return(elem_adj)
        } else {
          return(elem)
        }
      }
    )
  if (.is_fml) fml_parsed <- fml_parsed[[1]]
  fml_parsed
}
#' @title Return an all subsets names value
#' @description Internal function to estimates the value associated with the
#' names defined to be in all subsets.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param .all `formula` or `formula_list`.
#' @param fml_parsed `formula` or `formula_list` processed with
#' `formula_parse()`.
#' @param .fct A `function` or string function name.
#' @param .adj A scalar value produced by `est_adj_value()`.
#' @param .is_fml Logical.
#' @param ... Passes arguments to the function in `.fct`.
#' @returns Result of `.fct` call applied to `.obj` using only the names
#' defined in `.all`.
est_all_value <- function(.all, fml_parsed, .fct, .adj, .is_fml, ...) {
  if (!is.null(.all) && .is_fml) {
    fml <-
      stats::reformulate(
        c(fml_parsed$rhs_names[fml_parsed$select_lgl], fml_parsed$offset),
        response = fml_parsed$lhs_names,
        intercept = fml_parsed$intercept_lgl)
    value <- formula_output_check(fml, .fct, .is_fml, ...)
  } else if (!is.null(.all) && !.is_fml)  {
    fml_lst <-
      lapply(
        fml_parsed,
        function(fml) {
          rhs <- c(fml$rhs_names[fml$select_lgl], fml$offset)
          if (length(rhs) == 0) rhs <- "1"
          stats::reformulate(
            rhs,
            response = fml$lhs_names,
            intercept = fml$intercept_lgl)
        }
      )
    fml_lst <- do.call("formula_list", fml_lst)
    value <- formula_output_check(fml_lst, .fct, .is_fml, ...)
  } else {
    value <- 0
  }
  ifelse(is.null(.all), value, value - .adj)
}
#' @title `.set` and `.wst` argument checking function
#' @description Internal function to check `.set` and `.wst` arguments.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param .set A list of `formula`s or `formula_list`s.
#' @param fml_parsed `formula` or `formula_list` processed with
#' `formula_parse()`.
#' @param .typ A character vector. Must be '.set' or '.wst'.
#' @param .is_fml Logical.
#' @returns NULL
set_checker <- function(.set, fml_parsed, .typ, .is_fml) {
  if (is.null(.set)) return(NULL)
  if (!is.list(.set)) stop(.typ, " must be a list.", call. = FALSE)
  if (length(.set) == 0) stop(.typ, " is an empty list.", call. = FALSE)
  if (.typ == "'.wst'" && !is.null(names(.set))) {
    stop("Names for '.wst's are not allowed.", call. = FALSE)
  } else if (any(grepl("\\.", names(.set)))) {
    stop("Names for '.set' cannot contain the period character '.'.", # !! this necessary?
         call. = FALSE)
  }
  .obj <- ifelse(.is_fml, "formula", "formula_list")
  not_fmls <- sapply(.set, function(elem) {!inherits(elem, .obj)})
  if (any(not_fmls)) {
    which_not_fml <- seq_len(length(not_fmls))[not_fmls]
    stop("List element(s):\n", paste(which_not_fml, collapse = " "),
         "\nof ", .typ, " are not the same type as '.obj'.", call. = FALSE)
  }
  if (.typ == "'.set'") {
    fml_parsed <- list(fml_parsed)
    set_names <- set_labeller(.set, FALSE)
    set_namelist_overlap <-
      intersect(
        set_names,
        unlist(lapply(fml_parsed, function(elem) elem$rhs_names))
      )
    if (length(set_namelist_overlap) > 0)
      stop("Name(s) '", paste(set_namelist_overlap, collapse = "','"),
           "' overlap with names of sets. Give these sets new names.",
           call. = FALSE)
  }
  NULL
}
#' @title `.set` and `.wst` list element checking and parsing function
#' @description Internal function to check individual `.set` and `.wst`
#' arguments and process them using `formula_parse()`.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param .set A list of `formula`s or `formula_list`s.
#' @param fml_parsed `formula` or `formula_list` processed with
#' `formula_parse()`.
#' @param .typ A character vector. Must be '.set' or '.wst'.
#' @param .is_fml Logical.
#' @returns A `formula_parse` object `lapply`-ed applied to the `.set` object.
set_element_checker <- function(.set, fml_parsed, .typ, .is_fml) {
  if (is.null(.set)) return(NULL)
  if (.is_fml) {
    sets_parsed <- lapply(.set, function(elem) list(formula_parse(elem)))
  } else {
    sets_parsed <- lapply(.set, function(elem) lapply(elem, formula_parse))
  }
  sets_check <-
    lapply(
      sets_parsed,
      function(eq) {
        eq_res <-
          lapply(
            eq,
            function(elem) {
              list(
                rhs = length(elem$rhs_names) == 0,
                lhs = length(elem$lhs_names) > 0,
                offset = !is.null(elem$offset),
                intercept = elem$intercept_lgl
              )
            }
          )
        names(eq_res) <- seq_len(length(eq_res))
        eq_res
      }
    )
  rhs_counts_sets <-
    unlist(lapply(sets_check, function(eq) lapply(eq, function(elem) elem$rhs)))
  if (any(unlist(rhs_counts_sets)))
    stop("Formulas in ", .typ, " must have one or more terms.", call. = FALSE)
  bad_lhs <-
    unlist(lapply(sets_check, function(eq) lapply(eq, function(elem) elem$lhs)))
  if (any(bad_lhs) && .is_fml)
    stop("Left hand sides not allowed in ", .typ, " formulas.", call. = FALSE)
  bad_offset <-
    unlist(lapply(
      sets_check, function(eq) lapply(eq, function(elem) elem$offset)
    ))
  if (any(bad_offset))
    stop("Offsets not allowed in ", .typ, " formulas.", call. = FALSE)
  bad_intercept <-   
    unlist(lapply(
      sets_check,
      function(eq) lapply(eq, function(elem) !elem$intercept)
    ))
  if (any(bad_intercept))
    stop("Removing intercepts not allowed in ", .typ, " formulas.",
         call. = FALSE)
  if (.is_fml) sets_parsed <- lapply(sets_parsed, function(elem) elem[[1]])
  sets_parsed
}
#' @title Name checking function
#' @description Internal function to check for overlap between `.set`, `.wst`,
#' and `.all`. Also checks to ensire all names in `.set` and `.wst` are also
#' in `.obj`.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param fml_parsed `formula` or `formula_list` processed with
#' `formula_parse()`.
#' @param sets_parsed `.set` processed with `element_set_checker()`.
#' @param wsts_parsed `.wst` processed with `element_set_checker()`.
#' @param all_parsed `.all` processed with `all_checker()`.
#' @param .is_fml Logical.
#' @returns NULL
check_namelists <-
  function(fml_parsed, sets_parsed, wsts_parsed, all_parsed, .is_fml) {
    if (.is_fml) {
      fml_parsed$lhs_names <- "..domir"
      fml_parsed <- list(fml_parsed)
      if (!is.null(sets_parsed)) {
        sets_parsed <-
          lapply(
            sets_parsed,
            function(elem) {
              elem$lhs_names <- "..domir"
              list(elem)
            }
          )
      }
      if (!is.null(wsts_parsed)) {
        wsts_parsed <-
          lapply(
            wsts_parsed,
            function(elem) {
              elem$lhs_names <- "..domir"
              list(elem)
            }
          )
      }
      if (!is.null(all_parsed)) {
        all_parsed$lhs_names <- "..domir"
        all_parsed <- list(all_parsed)
      }
    }
    namelist <-
      lapply(
        fml_parsed,
        function(elem) paste(elem$lhs_names, "~", elem$rhs_names, sep = "")
      )
    sets_namelists <-
      lapply(
        sets_parsed,
        function(elem) {
          lapply(
            elem,
            function(el) {
              paste(el$lhs_names, "~", el$rhs_names, sep = "")
            }
          )
        }
      )
    wsts_namelists <-
      lapply(
        wsts_parsed,
        function(elem) {
          lapply(
            elem,
            function(el) {
              paste(el$lhs_names, "~", el$rhs_names, sep = "")
            }
          )
        }
      )
    if (
      length(setdiff(unlist(namelist), unlist(wsts_namelists))) == 0 &&
      length(wsts_namelists) == 1
    )
      stop("All names in '.obj' cannot be grouped into a single '.wst'.",
           call. = FALSE)
    all_namelist <- lapply(all_parsed, function(elem) elem$rhs_names)
    if (length(intersect(unlist(all_namelist), unlist(sets_namelists))) > 0) {
      stop("The same names cannot be in both '.set' and '.all'.", call. = FALSE)
    }
    if (length(intersect(unlist(all_namelist), unlist(wsts_namelists))) > 0) {
      stop("The same names cannot be in both '.wst' and '.all'.", call. = FALSE)
    }
    if (length(intersect(unlist(sets_namelists), unlist(wsts_namelists))) > 0) {
      stop("The same names cannot be in both '.set' and '.wst'.", call. = FALSE)
    }
    if (length(setdiff(unlist(sets_namelists), unlist(namelist))) > 0) {
      stop("Names in '.set' missing from '.obj'.", call. = FALSE)
    }
    if (length(setdiff(unlist(wsts_namelists), unlist(namelist))) > 0) {
      stop("Names in '.wst' missing from '.obj'.", call. = FALSE)
    }
    if (any(duplicated(unlist(sets_namelists)))) {
      stop("Duplicated names in '.set's.", call. = FALSE)
    }
    if (any(duplicated(unlist(wsts_namelists)))) {
      stop("Duplicated names in '.wst's.", call. = FALSE)
    }
    NULL
  }

#' @title `.set` and `.wst` labelling function
#' @description Internal function to apply labels to `.set`s and `.wst`s.
#' Applies names defined in `.set` by the user.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param .set A `formula` or `formula_list` in the format required by either
#' `.set` or `.wst` arguments.
#' @param .is_wst Logical.
#' @returns Character vector.
set_labeller <- function(.set, .is_wst) {
  if (is.null(.set)) return(NULL)
  name_type <- ifelse(.is_wst, "wst", "set")
  if (is.null(names(.set))) {
    set_labels <- paste(name_type, seq_len(length(.set)), sep = "")
  } else {
    set_labels <- names(.set)
  }
  missing_set_labels <- which(set_labels == "")
  if (length(missing_set_labels) > 0)
    set_labels[missing_set_labels] <- paste("set", missing_set_labels, sep = "")
  set_labels
}
#' @title Name to `.obj` structure coordinating method
#' @description Internal function to link names to `formula`s or
#' `formula_list`s in a way that can be referred to across normal,
#' `.set`, and `.wst` structures.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param fml_parsed `formula` or `formula_list` processed with
#' `formula_parse()`.
#' @param sets_parsed `.set` processed with `set_element_checker()`.
#' @param wsts_parsed `.wst` processed with `set_element_checker()`.
#' @param .is_fml Logical.
#' @returns Data frame with three columns: 'eq', 'elem', and 'name'.
#' 'eq' is the integer formula location of the name; always 1 for `formula`s.
#' 'elem' is the integer term location in the name in the formula.
#' 'name' a character.
determine_dominance_names <-
  function(fml_parsed, sets_parsed, wsts_parsed, .is_fml) {
    if (.is_fml) {
      fml_parsed <- list(fml_parsed)
      if (!is.null(sets_parsed))
        sets_parsed <-
          lapply(sets_parsed, function(elem) list(elem))
      if (!is.null(wsts_parsed))
        wsts_parsed <-
          lapply(wsts_parsed, function(elem) list(elem))
    }
    namelist <-
      unlist(
        lapply(
          fml_parsed,
          function(elem) {
            dv <- NULL
            if (!.is_fml) dv <- paste(elem$lhs_names, "~", sep = "")
            paste(dv, elem$rhs_names[!elem$select_lgl], sep = "")
          }
        )
      )
    if (is.null(sets_parsed)) {
      sets_namelists <- NULL
    } else {
      sets_namelists <-
        lapply(
          sets_parsed,
          function(elem)
            unlist(lapply(
              elem,
              function(el) {
                dv <- NULL
                if (!.is_fml) dv <- paste(el$lhs_names, "~", sep = "")
                paste(dv, el$rhs_names, sep = "")
              }
            ))
        )
      set_names <- paste("set", seq_len(length(sets_parsed)), sep = "")
      names(sets_namelists) <- set_names
    }
  if (is.null(wsts_parsed)) {
    wsts_namelists <- NULL
  } else {
    wsts_namelists <-
      lapply(
        wsts_parsed,
        function(elem)
          unlist(lapply(
            elem,
            function(el) {
              dv <- NULL
              if (!.is_fml) dv <- paste(el$lhs_names, "~", sep = "")
              paste(dv, el$rhs_names, sep = "")
            }
          ))
      )
    wst_names <- paste("wst", seq_len(length(wsts_parsed)), sep = "")
    names(wsts_namelists) <- wst_names
  }
  submitter_list <-
    setdiff(namelist, c(unlist(sets_namelists), unlist(wsts_namelists)))
  if (length(submitter_list) > 0) {
    submitter_list <- as.list(submitter_list)
    names(submitter_list) <-
      paste("var", seq_len(length(submitter_list)), sep = "")
  } else {
    submitter_list <- list()
  }
  submitter_list <- append(submitter_list, sets_namelists)
  submitter_list <- append(submitter_list, wsts_namelists)
  number_dominance_names <-
    sum(grepl("^var|^set", names(submitter_list))) +
    sum(
      unlist(sapply(wsts_namelists, function(elem) length(elem)))
    )
  if (number_dominance_names < 2) {
    stop("At least two names or sets of names are needed for a ",
         "dominance analysis.", call. = FALSE)
  }
  submitter_list
}
#' @title Formatting for names given `.obj` structure
#' @description Internal function to to apply labels output.
#' Applies names defined in `.set` by the user.
#' Designed to accommodate both `formula` and `formula_list` objects.
#' @noRd
#' @param namelist Character vector of names from `.obj`.
#' @param .set A `formula` or `formula_list` in the format required by the
#' `.set` argument.
#' @returns Character vector.
determine_display_names <- function(namelist, .set) {
    regular_names <- namelist[grepl("var", names(namelist))]
    if (length(regular_names) > 0) names(regular_names) <- NULL
    set_names <- set_labeller(.set, FALSE)
    if (length(set_names) > 0) names(set_names) <- NULL
    wst_names <- namelist[grepl("wst", names(namelist))]
    if (length(wst_names) > 0) names(wst_names) <- NULL
    c(unlist(regular_names), set_names, unlist(wst_names))
  }
#' @title Formatting for names in returned list
#' @description Internal function to to apply names to the elements of the
#' list returned by `domir`.
#' @noRd
#' @param return_list A list of dominance analysis results.
#' @param names_for_printing Character vector.
#' @returns `return_list` with updated names.
name_return_list <- function(return_list, names_for_printing) {
  names(return_list$general) <- names_for_printing
  names(return_list$ranks) <- names_for_printing
  names(return_list$standard) <- names_for_printing
  rownames(return_list$conditional) <- names_for_printing
  colnames(return_list$conditional) <-
    paste("include_at_", seq_len(ncol(return_list$conditional)), sep = "")
  if (!is.null(return_list$complete)) {
    rownames(return_list$complete) <- paste(names_for_printing, "_>", sep = "")
    colnames(return_list$complete) <- paste(">_", names_for_printing, sep = "")
  }
  names(return_list) <-
    c("General_Dominance", "Conditional_Dominance", "Complete_Dominance",
      "Ranks", "Standardized")
  return_list
}
#' @title Print method for `domir`
#' @description Reports formatted results from `domir` class object.
#' @param x an object of class "domir".
#' @param .cdl Logical.
#' If `TRUE` then conditional dominance statistics will be reported.
#' @param .cpt Logical.
#' If `TRUE` then complete dominance proportions will be reported.
#' @param ... further arguments passed to [`print.default`].
#' @return The submitted "domir" object, invisibly.
#' @details The print method for class `domir` objects reports out the
#' following results:
#' \itemize{
#'  \item{Value when all elements are included in `obj`.}
#'  \item{Value for the elements included in `.all`, if any.}
#'  \item{Value for the elements included in `.adj`, if any.}
#'  \item{Matrix describing general dominance values, standardized
#'  general dominance values, and the ranking of the general
#'  dominance values.}
#'  \item{Matrix describing the conditional dominance values
#'  if `.cdl` is `TRUE`.}
#'  \item{Matrix describing the complete dominance designations
#'  if `.cpt` is `TRUE`.}
#'  \item{If following [`summary.domir`], matrix describing the strongest
#'  dominance designations between all elements if both `.cdl` and `.cpt`
#'  are `TRUE`.}}
#'
#'  The `domir` print method alters dimension names for readability and they
#'  do not display as stored in the `domir` object.
#'
#' @exportS3Method
print.domir <- function(x, .cdl = TRUE, .cpt = TRUE, ...) {
  # if (!is.null(as.list(x$Call)$.prg))
  #   switch(
  #     as.character(as.logical(deparse(as.list(x$Call)$.prg))),
  #     `TRUE` = cat("\n"),
  #     `FALSE` = NULL
  #   )
  # ~~ to remove: begin ~~
  temp_cpt <- as.list(x$Call)$.cpt
  temp_cdl <- as.list(x$Call)$.cdl
  if (!is.null(temp_cpt)) .cpt <- as.logical(temp_cpt)
  if (!is.null(temp_cdl)) .cdl <- as.logical(temp_cdl)
  # ~~ to remove: end ~~
  cat("\nOverall Value:     ", x[["Value"]], "\n")
  if (length(x[["Value_All"]]) > 0)
    cat("All Subset Value:  ", x[["Value_All"]], "\n")
  if (length(x[["Value_Adjust"]]) > 0)
    cat("Adjustment Value:  ",
        x[["Value_Adjust"]], "\n")
  cat("\n")
  cat("General Dominance Values:\n")
  display_std <-
    t(rbind(x[["General_Dominance"]], x[["Standardized"]], x[["Ranks"]]))
  dimnames(display_std) <-
    list(names(x[["Ranks"]]), c("General Dominance", "Standardized", "Ranks"))
  print(display_std, ...)
  cat("\n")
  if (length(x[["Conditional_Dominance"]] > 0) && .cdl) {
    cat("Conditional Dominance Values:\n")
    colnames(x[["Conditional_Dominance"]]) <-
      paste("Include At:", seq_len(ncol(x[["Conditional_Dominance"]])))
    print(x[["Conditional_Dominance"]], ...)
    cat("\n")
  }
  if (length(x[["Complete_Dominance"]] > 0) && .cpt) {
    cat("Complete Dominance Proportions:\n")
    colnames(x[["Complete_Dominance"]]) <-
      gsub("^>_", "> ", colnames(x[["Complete_Dominance"]]))
    rownames(x[["Complete_Dominance"]]) <-
      gsub("_>$", " >", rownames(x[["Complete_Dominance"]]))
    print(x[["Complete_Dominance"]])
    cat("\n")
  }
  if (length(x[["Strongest_Dominance"]] > 0) && .cpt && .cdl) {
    cat("Strongest Dominance Designations:")
    print(x[["Strongest_Dominance"]])
    cat("\n")
  }
  invisible(x)
}
#' @title Summary method for `domir`
#' @description Reports dominance designation results from the `domir`
#' class object.
#' @param object an object of class "domir".
#' @param ... further arguments passed to or from other methods.
#' Not used currently.
#' @return The submitted "domir" object with an additional
#' `Strongest_Dominance` element added.
#' \describe{
#'  \item{\code{Strongest_Dominance}}{Matrix comparing the element in the first
#'  row to the element in the third row.  The second row denotes the strongest
#'  designation between the two names.}
#' }
#'
#' @details The summary method for class `domir` objects is used for obtaining
#' the strongest dominance designations (i.e., general, conditional, or
#' complete) among all pairs of dominance analyzed names.
#'
#' @exportS3Method
summary.domir <- function(object, ...) {
  if (length(object[["Strongest_Dominance"]]) == 0) {
    reverse <- as.list(object$Call)$.rev
    if (is.null(reverse)) reverse <- FALSE
    reverse_cdl <-
      ifelse(reverse,
             rep(-1, times = length(object$General_Dominance)),
             rep(1, times = length(object$General_Dominance)))
    reverse_gnl <- ifelse(reverse, -1, 1)
    pairs <- utils::combn(names(object$General_Dominance), 2)
    pairs <- rbind(pairs[1, ], rep("", times = ncol(pairs)), pairs[2, ])
    location <- 0
    for (IV1 in 1:(length(object$General_Dominance) - 1)) {
      for (IV2 in (IV1 + 1):length(object$General_Dominance)) {
        location <- location + 1
        if (length(object[["Complete_Dominance"]] > 0)) {
          if ((object$Complete_Dominance[IV1, IV2] %in% c(0, 1))) {
            pairs[2, location] <-
              ifelse(object$Complete_Dominance[IV1, IV2] == 1,
                     "completely dominates",
                     "is completely dominated by")
            next
          }
        }
        if (length(object[["Conditional_Dominance"]] > 0)) {
          if (all(object$Conditional_Dominance[IV1, ] * reverse_cdl >
                    object$Conditional_Dominance[IV2, ] * reverse_cdl)) {
            pairs[2, location] <- "conditionally dominates"
            next
          } else if (all(object$Conditional_Dominance[IV1, ] * reverse_cdl <
                           object$Conditional_Dominance[IV2, ] * reverse_cdl)) {
            pairs[2, location] <- "is conditionally dominated by"
            next
          }
        }
        pairs[2, location] <-
          ifelse(object$General_Dominance[[IV1]] * reverse_gnl >
                   object$General_Dominance[[IV2]] * reverse_gnl,
                 "generally dominates",
                 ifelse(object$General_Dominance[[IV1]] * reverse_gnl <
                          object$General_Dominance[[IV2]] * reverse_gnl,
                        "is generally dominated by",
                        "has no dominance designation with"))
      }
    }
    rownames(pairs) <- rep("", times = 3)
    colnames(pairs) <- rep("", times = ncol(pairs))
    res <- append(object, list(Strongest_Dominance = pairs))
    class(res) <- c("domir")
    return(res)
  } else {
    return(object)
  }
}

library(pscl)
library(datasets)

# domir.formula ----

value_vec <- 
  c(
    cor(mtcars$mpg, predict(lm(mpg ~ vs, data = mtcars)))^2,
    cor(mtcars$mpg, predict(lm(mpg ~ cyl, data = mtcars)))^2,
    cor(mtcars$mpg, predict(lm(mpg ~ carb, data = mtcars)))^2,
    cor(mtcars$mpg, predict(lm(mpg ~ am, data = mtcars)))^2,
    cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl, data = mtcars)))^2,
    cor(mtcars$mpg, predict(lm(mpg ~ carb + am, data = mtcars)))^2,
    cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb, data = mtcars)))^2,
    cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + am, data = mtcars)))^2,
    cor(mtcars$mpg, predict(lm(mpg ~ carb + am + vs, data = mtcars)))^2,
    cor(mtcars$mpg, predict(lm(mpg ~ carb + am + cyl, data = mtcars)))^2,
    cor(mtcars$mpg, predict(lm(mpg ~ vs + cyl + carb + am, data = mtcars)))^2
  )

wst1_vec <- 
  c(
    value_vec[[1]]/2 + (value_vec[[5]] - value_vec[[2]])/2,
    value_vec[[2]]/2 + (value_vec[[5]] - value_vec[[1]])/2,
    value_vec[[3]]/2 + (value_vec[[6]] - value_vec[[4]])/2,
    value_vec[[4]]/2 + (value_vec[[6]] - value_vec[[3]])/2
  )

wst2_vec <- 
  c(
    (value_vec[[9]] - value_vec[[6]])/2 + (value_vec[[11]] - value_vec[[10]])/2,
    (value_vec[[10]] - value_vec[[6]])/2 + (value_vec[[11]] - value_vec[[9]])/2,
    (value_vec[[7]] - value_vec[[5]])/2 + (value_vec[[11]] - value_vec[[8]])/2,
    (value_vec[[8]] - value_vec[[5]])/2 + (value_vec[[11]] - value_vec[[7]])/2
  )

cdl_names <- list(c("vs", "cyl", "carb", "am"), paste0("include_at_", 1:2))

cdl_test <- 
  matrix(c(wst1_vec, wst2_vec), nrow = 4, ncol = 2, dimnames = cdl_names)

test_obj <- domir(mpg ~ vs + cyl + carb + am, 
                      function(fml, data) {
                        res <- summary(lm(fml, data = data))
                        return(res[["r.squared"]])
                      },
                      data = mtcars, 
                      .wst = list(~ vs + cyl, ~ carb + am))

test_that("Test Use of W-sets with Conditional Dominance: domir.formula", {
  expect_equal(test_obj$Conditional_Dominance, cdl_test
  )}
)

---
title: "Conceptual Introduction to Dominance Analysis"
subtitle: "Examples and Implementation with `domir`"
author: Joseph Luchman
date: "`r Sys.Date()`"
bibliography: domir_basics.bib
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Conceptual Introduction to Dominance Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

The purpose of this vignette is to briefly discuss the conceptual underpinnings of the relative importance method implemented in the **domir** package and provide several extensive examples that illustrate these concepts as applied to data.

This vignette is intended to serve as a refresher for users familiar with these concepts as well as an brief introduction to them for those who are not.

By the end of this vignette, the reader should have a sense for what the key relative importance method is attempting to do as well as an understanding of how it is accomplished as applied to data.

# Conceptual Introduction

The relative importance method implemented in the **domir** package produces results that are relatively easy to interpret but does so in a way that is computationally intensive in as implemented.

The discussion below outlines the conceptual origins of the method, what the relative importance method does, and some details about how the DA method is implemented in the **domir** package.

## Dominance Analysis

The focus of the **domir** package is on dominance analysis (DA). DA is a method that resolves the indeterminacy of trying to ascribe a the results from a predictive model's fit metric, referred to as a 'value' in the package, to individual predictive factors (i.e., independent variables/IVs, predictors, features), referred to as 'names' in the package.

A challenge for many predictive models and fit metrics are that there is no way to analytically decompose the fit statistic/metric given correlations between the predictive factors that are naturally present in the data or are introduced by the model. When there is no way to analytically separate the fit statistic to ascribe it to predictive factors, a methodological approach could be applied where values are ascribed by including the factors in the model sequentially. As each predictive factor is included, the change in the fit metric is ascribed to that predictive factor.

One issue with the sequential approach is that the sequence chosen to ascribe the fit statistic to predictive factors determines how much of the fit statistic is ascribed to the factor. When the analyst has good reason to choose a specific inclusion order, this approach produces a useful result.

Using a single inclusion order can be problematic when there is *not* a good reason to choose one specific inclusion order over another. When the inclusion order is effectively arbitrary, changing the order changes the values ascribed to the predictive factors in ways that have implications on inferences from the model.

A solution to this problem is to consider all possible ways of including the predictive factor. This method for resolving this issue is the approach used by Shapley value decomposition from Cooperative Game Theory (see @gromping2007estimators for a discussion) which seeks to find a solution to the problem of how to subdivide payoffs to players in a cooperative game based on their relative contribution when it is not possible to separate relative contributions analytically.

DA uses the idea of comparing across inclusion orders as a methodological, and almost experimental design-like, approach to determining importance. DA also extends on the classic Shapley value decomposition methodology by adding two, more difficult to achieve, importance criteria. All three different importance criteria, known as dominance designations, are discussed in greater detail in the context of an example discussed below after a brief introduction to the `domir` function.

## DA Implementation with `domir`

The `domir` function is an API for applying DA to predictive modeling functions in R. The sections below will use this function to illustrate how DA is implemented and discuss conceptual details of each computation.

```{r, include = FALSE, results='hide'}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
#devtools::load_all()
library(ggplot2)
library(dplyr)
library(purrr)
library(forcats)
library(tidyr)
library(stringr)
```

# Concepts in Application

The purpose of the **domir** package is to apply DA to predictive models. This section builds on the last by providing an example predictive model with which to illustrate the computation and interpretation of the dominance results produced by DA.

DA was developed originally using linear regression (`lm`) with the explained variance $R^2$ metric as a fit statistic [@budescu1993dominance]. The examples below use this model and fit statistic as both are widely used and understood in statistics and data science.

Consider this model using the *mtcars* data in the **datasets** package.

```{r setup_lm}
library(datasets)

lm_cars <- 
  lm(mpg ~ am + cyl + carb, data = mtcars)

summary(lm_cars)
```

The results show that all three IVs are statistically significant at the traditional level (i.e., $p < .05$) and that, in total, the predictors---*am, cyl*, and *carb*---explain \~80% of the variance in *mpg*.

I intend to conduct a DA on this model using `domir` and implement the DA as follows:

```{r setup_domir}
library(domir)

domir(
  mpg ~ am + cyl + carb, 
  function(formula) {
    lm_model <- lm(formula, data = mtcars)
    summary(lm_model)[["r.squared"]]
  }
)
```

The `domir` function above prints out results in four sections:

1.  fit statistic results
2.  general dominance statistics
3.  conditional dominance statistics
4.  complete dominance proportions

Below I "replay" and discuss each result in turn.

## Fit Statistic Results

```         
Overall Value:      0.8113023
```

The first result `domin` prints is related to the overall fit statistic value for the model. In game theory terms, this value is the total payoff all players/IVs produced in the cooperative game/model.

The value produced serves as the fit statistic "to be decomposed" by the DA and is limiting value for how much each IV will be able to explain. The DA will ascribe the three IVs in this model separate components of this \~$.8113$ value related to their contributions to predicting *mpg*.

Other fit statistic value adjustments are reported in this section as well in particular those associated with the all subsets and constant model adjustments (*to be discussed further below; sections under development*) when used in the DA.

## General Dominance Statistics

```         
General Dominance Values:
     General Dominance Standardized Ranks
am           0.2156848    0.2658501     2
cyl          0.4173094    0.5143698     1
carb         0.1783081    0.2197801     3
```

The second result printed reports the *general dominance statistics* related to how the overall fit statistic's value is divided up among the IVs. These also represent the Shapley value decompositions of the fit statistic showing how each player/IV is ascribed a component of the payoff/fit statistic from the game/model.

The *General Dominance* column of statistics can be interpreted in terms of the fit metric it decomposed. For example, *am* has a value of \~$0.2157$ which means *am* is associated with an $R^2$ of about twenty-two percentage points of *mpg*'s variance given the predictive model and other IVs.

The *Standardized* column of statistics expresses the general dominance statistic value as a percentage of the overall fit statistic value and thus sums to 100%. *am*'s contributions to the $R^2$'s total value is \~27% (i.e., $\frac{.2157}{.8113} = .2659$).

The final *Ranks* column is most relevant to the focal purpose of determining the relative importance of the IVs in this model as it provides a rank ordering of the IVs based on their general dominance statistics. It is here that DA moves beyond Shapley value decomposition in that DA, and this rank ordering based on general dominance statistics, allows for applying labels to the relationships between IVs in terms of their importance.

*am* is ranked second because it has a smaller general dominance statistic than the first ranked *cyl*. As such, *am* "is generally dominated by" *cyl*. By contrast, *am* is ranked higher than third ranked *carb* and thus *am* "generally dominates" *carb*. These labels are known as *general dominance designations*.

### Points of Note: General Dominance

The general dominance statistics always sum to the value of the overall fit statistic and, because they represent parts of a whole, are widely considered the easiest of the dominance statistics to interpret (i.e., as compared to conditional and complete dominance discussed next).

The general dominance statistics, however simple to interpret, are the least stringent of the dominance statistics/designations (the reasons why are discussed later). Thus, the requirements to assign the "generally dominates" label to a relationship between two IVs are the easiest to fulfill.

## Conditional Dominance Statistics

```         
Conditional Dominance Values:
     Include At: 1 Include At: 2 Include At: 3
am       0.3597989     0.2164938    0.07076149
cyl      0.7261800     0.4181185    0.10762967
carb     0.3035184     0.1791172    0.05228872
```

The third section reported on by `domin` prints the *conditional dominance statistics* associated with each IV. Each IV has a separate conditional dominance statistic related to position at which it is included in the sequence of IVs in the model.

The conditional dominance matrix can be used to designate importance in a way that is more stringent/harder to achieve than the general dominance statistics. To determine importance with the conditional dominance matrix each IV is compared to each other in a 'row-wise' fashion. If the value of each entry for a row/IV is greater than the value of another row/IV at the same position (i.e., comparing IVs at the same inclusion position) than an IV is said to "conditionally dominate" the other IV. The matrix above shows that *am* "is conditionally dominated by" *cyl* as its conditional dominance statistics are smaller than *cyl*'s at positions 1, 2, and 3. Conversely, *am* "conditionally dominates" *carb* as its conditional dominance statistics are greater than *carb*'s at positions 1, 2, and 3.

### Points of Note: Conditional Dominance

Conditional dominance statistics provide more information about each IV than general dominance statistics as they more clearly reveal the effect that IV redundancy has on prediction for each IV. Conditional dominance statistics show the average increase in predictive usefulness associated with an IV when it is included at a specific position in the sequence of IVs. As the position gets more later in the model, the contribution any one IV can make tends to grow more limited. This limiting effect with more IVs is reflected in the trajectory of conditional dominance statistics.

The increase in complexity with conditional dominance over that of general dominance also results in a more stringent set of comparisons. Because the label "conditionally dominates" is only ascribed to a relationship that shows more contribution to the fit metric at all positions of the conditional dominance matrix, it is a more difficult criterion to achieve and is therefore a stronger designation.

Note that conditional dominance implies general dominance--but the reverse is not true. An IV can generally, but not conditionally, dominate another IV.

## Complete Dominance Proportions

```         
Complete Dominance Proportions:
       > am > cyl > carb
am >     NA     0      1
cyl >     1    NA      1
carb >    0     0     NA
```

The fourth section reported on by `domir` prints the *complete dominance proportions* associated with each IV pair. Each IV is compared to each other IV and has two entries in this matrix. The IV noted in the row labels is the 'dominating' IV as is implied by the greater than symbol (i.e., $>$) preferring it. The IV noted in the column labels is the 'dominated' IV as is implied by the greater than symbol not preferring it. The values reported are the proportion of sub-models in which the IV in the row obtains a larger value than the IV in the column.

The complete dominance designations are useful beyond the general and conditional dominance results as they are the most stringent sets of comparisons. Complete dominance reflects the relative performance of each IV to another IV in all the sub-models where their relative predictive performance can be compared. When a value of 1 is obtained, the IV in the row is said to "completely dominate" the IV in the column. Conversely, when a value of 0 is obtained, the IV in the row is said to be "completely dominated by" the IV in the column.

### Points of Note: Complete Dominance

Complete dominance designations are the most stringent of the designations as *all* comparable sub-models for an IV must be larger than its comparison IV for the "conditionally dominates" label to be ascribed to their relationship. The sorts of sub-models that qualify as 'comparable' is discussed later.

Also note that complete dominance implies both conditional and general dominance but, again, not the reverse. An IV can conditionally or generally, but not completely, dominate another IV.

# Dominance Statistics and Designations: Key Computational Details

The DA methodology currently implemented in **domir** is a relatively assumption-free and model agnostic but computationally expensive methodology that follows from the way Shapley value decomposition was originally formulated.

The sections below begin by providing an analogy for how to think about the computation of DA results, outline exactly how each dominance statistic and designation is determined, as well as extend the example above by applying each computation in the context of the example.

## Computational Implementation: All Possible Orders

DA requires evaluating the contribution IVs make to prediction given all possible orders in which they are included in the prediction model. As was noted above, this is an experimental design-like approach where all possible combinations of the IVs included or excluded are estimated as sub-models. When there are $p$ IVs in the model there will be $2^p$ sub-models estimated. The experimental design-like approach of the method makes it widely applicable across predictive models and fit statistic values but is computationally expensive as each additional IV added to the model results in a geometric increase in the number of required sub-models.

## DA Results: The Full-Factorial Design

The DA results related to the `lm` model with three IVs discussed above is composed of `r 2**3` sub-models and their $R^2$ values. The `domir` function, if supplied a predictive modeling function that can record each sub-model's results, can be adapted to capture each sub-model's $R^2$ value along with the IVs that comprise it.

The code below constructs a wrapper function to export results from each sub-model to an external data frame. The code to produce these results is complex and each line is commented to note its purpose. This wrapper function then replaces `lm` in the call to `domir` so that, as the DA is executed, all the sub-models' data are captured for the illustration to come.

```{r capture_r2s}
lm_capture <- 
  function(formula, data, ...) { # wrapper program that accepts formula, data, and ellipsis arguments
    count <<- count + 1 # increment counter in enclosing environment
    lm_obj <- lm(formula, data = data, ...) # estimate 'lm' model and save object
    DA_results[count, "formula"] <<- 
      deparse(formula) # record string version of formula passed in 'DA_results' in enclosing environment
    DA_results[count, "R^2"] <<- 
      summary(lm_obj)[["r.squared"]] # record R^2 in 'DA_results' in enclosing environment
    summary(lm_obj)[["r.squared"]] # return R^2
  }

count <- 0 # initialize the count indicating the row in which the results will fill-in

DA_results <- # container data frame in which to record results
  data.frame(formula = rep("", times = 2^3-1), 
             `R^2` = rep(NA, times = 2^3-1), 
             check.names = FALSE)

lm_da <- domir(mpg ~ am + cyl + carb, # implement the DA with the wrapper
               lm_capture, 
               data = mtcars)

DA_results
```

The printed result from *DA_results* shows that `domir` runs `r nrow(DA_results)` sub-models; each a different combination of the IVs. Note that, by default, the sub-model where all IVs are excluded is assumed to result in a fit statistic value of 0 and is not estimated directly (which can be changed with the `.adj` argument).

The $R^2$ values recorded in *DA_results* are used to compute the dominance statistics and designations reported on above.

### Complete Dominance Proportions

Complete dominance proportions between two IVs are computed by:

$$C_{X_vX_z} =\, \frac{\Sigma^{2^{p-2}}_{j=1}{ \{\begin{matrix} if\, F_{X_v\; \cup\;S_j}\, > F_{X_z\; \cup\;S_j}\, \,then\, 1\, \\ if\, F_{X_v\; \cup\;S_j}\, \le F_{X_z\; \cup\;S_j}\,then\, \,0\end{matrix} }}{2^{p-2}}$$

Where $X_v$ and $X_z$ are two IVs, $S_j$ is a distinct set of the other IVs in the model not including $X_v$ and $X_z$ which can include the null set ($\emptyset$) with no other IVs, and $F$ is a model fit statistic. This computation is then the proportion of all comparable sub-models where $X_v$ is greater than $X_z$.

The results from *DA_results* can then be used to compute the complete dominance proportions. The comparison begins with the results for `am` and `cyl`.

```{r cpt_am_cyl, echo=FALSE}
knitr::kable(
  cbind(DA_results[grepl("am", DA_results$formula) & !grepl("cyl", DA_results$formula) ,],
        DA_results[!grepl("am", DA_results$formula) & grepl("cyl", DA_results$formula) ,]), 
  row.names = FALSE, caption = "Complete Dominance Comparisons: `am` versus `cyl` ", digits = 3)
```

The rows in the table above are aligned such that comparable models are in the rows. As applied to this example, the $S_j$ sets are $\emptyset$ (i.e., the null set) with no other IVs and the set also including *carb*.

The $R^2$ values across the comparable models show that *cyl* has larger $R^2$ values than, and thus completely dominates, *am*.

```{r cpt_am_carb, echo=FALSE}
knitr::kable(cbind(DA_results[grepl("am", DA_results$formula) & !grepl("carb", DA_results$formula) ,], DA_results[!grepl("am", DA_results$formula) & grepl("carb", DA_results$formula) ,]), row.names = FALSE, caption = "Complete Dominance Comparisons: `am` versus `carb` ", digits = 3)
```

Here the $S_j$ sets are, again, $\emptyset$ and the set also including *cyl*.

The $R^2$ values across the comparable models show that *am* has larger $R^2$ values than and completely dominates *carb*.

```{r cpt_cyl_carb, echo=FALSE}
knitr::kable(cbind(DA_results[grepl("cyl", DA_results$formula) & !grepl("carb", DA_results$formula) ,], DA_results[!grepl("cyl", DA_results$formula) & grepl("carb", DA_results$formula) ,]), row.names = FALSE, caption = "Complete Dominance Comparisons: `cyl` versus `carb` ", digits = 3)
```

Finally, the $S_j$ sets are the $\emptyset$ and the set also including *am*.

The $R^2$ values across the comparable models show that *cyl* has larger $R^2$ values than and completely dominates *carb*.

Each of these three sets of comparisons are represented in the *Complete_Dominance* matrix as a series of proportions. Note that the diagonal of the matrix is `NA` values as it is conceptually useless to compare the IV to itself.

```{r lm_complete}
lm_da$Complete_Dominance
```

### Computing Conditional Dominance

Conditional dominance statistics are computed as:

$$C^i_{X_v} = \Sigma^{\begin{bmatrix}p-1\\i-1\end{bmatrix}}_{i=1}{\frac{F_{X_v\; \cup\; S_i}\, - F_{S_i}}{\begin{bmatrix}p-1\\i-1\end{bmatrix}}}$$

Where $S_i$ is a subset of IVs not including $X_v$ and $\begin{bmatrix}p-1\\i-1\end{bmatrix}$ is the number of distinct combinations produced choosing the number of elements in the bottom value ($i-1$) given the number of elements in the top value ($p-1$; i.e., the value produced by `choose(p-1, i-1)`).

In effect, the formula above amounts to an average of the differences between each model containing $X_v$ from the comparable model not containing it by the number of IVs in the model total. These values then reflect the effect of including the IV at a specific order in the model. As applied to the results from *DA_results*, *am*'s conditional dominance statistics are computed with the following differences:

```{r cdl_am, echo=FALSE}
first_order <- 
  cbind(DA_results[DA_results$formula == "mpg ~ am",], data.frame(`formula subtrahend` = "mpg ~ 1", `R^2 subtrahend` = 0, difference = DA_results[DA_results$formula == "mpg ~ am", "R^2"], check.names = FALSE))

names(first_order)[1:2] <- c("formula minuend", "R^2 minuend")

second_order <- 
  cbind(DA_results[grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),], 
        DA_results[!grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),], 
        data.frame(difference = 
                     DA_results[grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"] - DA_results[!grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),"R^2"]))

names(second_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

third_order <- 
  cbind(DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),],
        DA_results[!grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),],
        data.frame(difference = 
                     DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),"R^2"] - DA_results[!grepl("am", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"]))

names(third_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

knitr::kable(first_order, row.names = FALSE, caption = "Conditional Dominance Computations: `am` with One IV/Alone", digits = 3)

knitr::kable(second_order, row.names = FALSE, caption = "Conditional Dominance Computations: `am` with Two IVs", digits = 3)

knitr::kable(third_order, row.names = FALSE, caption = "Conditional Dominance Computations: `am` with Three IVs/Full Model", digits = 3)
```

The rows of each table represent a difference to be recorded for the conditional dominance statistics computation. In the position one, two, and three comparison tables, the model with *am* is presented first (as the minuend) and the comparable model without *am* is presented second (as the subtrahend)---for the first IV comparison table, this model is the intercept only model that, as is noted above, is assumed to have a value of 0. The difference is presented last.

By table, the differences are averaged resulting in the `r round(lm_da$Conditional_Dominance["am", "include_at_1"], digits = 3)` value when first, the ``` r round(lm_da$Conditional_Dominance["am", "``include_at_``2"], digits = 3) ``` when second, and ``` r round(lm_da$Conditional_Dominance["am", "``include_at_``3"], digits = 3) ``` when third.

Next the computations for *cyl* are reported.

```{r cdl_cyl, echo=FALSE}
first_order <- 
  cbind(DA_results[DA_results$formula == "mpg ~ cyl",], data.frame(`formula subtrahend` = "mpg ~ 1", `R^2 subtrahend` = 0, difference = DA_results[DA_results$formula == "mpg ~ cyl", "R^2"], check.names = FALSE))

names(first_order)[1:2] <- c("formula minuend", "R^2 minuend")

second_order <- 
  cbind(DA_results[grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),], 
        DA_results[!grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),], 
        data.frame(difference = 
                     DA_results[grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"] - DA_results[!grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),"R^2"]))

names(second_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

third_order <- 
  cbind(DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),],
        DA_results[!grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),],
        data.frame(difference = 
                     DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),"R^2"] - DA_results[!grepl("cyl", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"]))

names(third_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

knitr::kable(first_order, row.names = FALSE, caption = "Conditional Dominance Computations: `cyl` with One IV/Alone", digits = 3)

knitr::kable(second_order, row.names = FALSE, caption = "Conditional Dominance Computations: `cyl` with Two IVs", digits = 3)

knitr::kable(third_order, row.names = FALSE, caption = "Conditional Dominance Computations: `cyl` with Three IVs/Full Model", digits = 3)
```

Again, the differences are averaged resulting in the `r round(lm_da$Conditional_Dominance["cyl", "include_at_1"], digits = 3)` value when first, the `r round(lm_da$Conditional_Dominance["cyl", "include_at_2"], digits = 3)` when second, and `r round(lm_da$Conditional_Dominance["cyl", "include_at_3"], digits = 3)` when third.

Finally, the computations for *carb*.

```{r cdl_carb, echo=FALSE}
first_order <- 
  cbind(DA_results[DA_results$formula == "mpg ~ carb",], data.frame(`formula subtrahend` = "mpg ~ 1", `R^2 subtrahend` = 0, difference = DA_results[DA_results$formula == "mpg ~ carb", "R^2"], check.names = FALSE))

names(first_order)[1:2] <- c("formula minuend", "R^2 minuend")

second_order <- 
  cbind(DA_results[grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),], 
        DA_results[!grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),], 
        data.frame(difference = 
                     DA_results[grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"] - DA_results[!grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==2),"R^2"]))

names(second_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

third_order <- 
  cbind(DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),],
        DA_results[!grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),],
        data.frame(difference = 
                     DA_results[sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==4),"R^2"] - DA_results[!grepl("carb", DA_results$formula) & sapply(DA_results$formula, function(x) length(all.vars(as.formula(x)))==3),"R^2"]))

names(third_order)[1:4] <- c("formula minuend", "R^2 minuend", "formula subtrahend", "R^2 subtrahend")

knitr::kable(first_order, row.names = FALSE, caption = "Conditional Dominance Computations: `carb` with One IV/Alone", digits = 3)

knitr::kable(second_order, row.names = FALSE, caption = "Conditional Dominance Computations: `carb` with Two IVs", digits = 3)

knitr::kable(third_order, row.names = FALSE, caption = "Conditional Dominance Computations: `carb` with Three IVs/Full Model", digits = 3)
```

And again, the differences are averaged resulting in the `r round(lm_da$Conditional_Dominance["carb", "include_at_1"], digits = 3)` value when first, the `r round(lm_da$Conditional_Dominance["carb", "include_at_2"], digits = 3)` when second, and `r round(lm_da$Conditional_Dominance["carb", "include_at_3"], digits = 3)` when third.

These nine values then populate the conditional dominance statistic matrix.

```{r lm_conditional}
lm_da$Conditional_Dominance
```

The conditional dominance matrix's values can then be compared by creating a series of logical designations indicating whether each IV conditionally dominates each other.

Below the comparisons begin with *am* and *cyl*

```{r cdl_am_cyk, echo=FALSE}
knitr::kable(data.frame(t(lm_da$Conditional_Dominance[c("am", "cyl"),]), comparison= lm_da$Conditional_Dominance["am",] > lm_da$Conditional_Dominance["cyl",]), caption = "Conditional Dominance Designation: `am` Compared to `cyl`", digits = 3)
```

The table above is a transpose of the conditional dominance statistic matrix with an additional *comparison* column indicating whether the first IV/*am*'s conditional dominance statistic at that inclusion position is greater than the second IV/*cyl*'s at that same position.

Conditional dominance is determined by all values being `TRUE` or `FALSE`; in this case, *cyl* is seen to conditionally dominate *am* as all values are `FALSE`.

Next is the comparison between *am* and *carb*

```{r cdl_am_carb, echo=FALSE}
knitr::kable(data.frame(t(lm_da$Conditional_Dominance[c("am", "carb"),]), comparison= lm_da$Conditional_Dominance["am",] > lm_da$Conditional_Dominance["carb",]), caption = "Conditional Dominance Designation: `am` Compared to `carb`", digits = 3)
```

Here *am* conditionally dominates *carb* as all values are `TRUE`.

The final comparison between *cyl* and *carb*

```{r cdl_cyl_carb, echo=FALSE}
knitr::kable(data.frame(t(lm_da$Conditional_Dominance[c("cyl", "carb"),]), comparison= lm_da$Conditional_Dominance["cyl",] > lm_da$Conditional_Dominance["carb",]), caption = "Conditional Dominance Designation: `cyl` Compared to `carb`", digits = 3)
```

*cyl* also conditionally dominates *carb* as all values are `TRUE`.

Another way of looking at conditional dominance is by graphing the trajectory of each IV across all positions in the conditional dominance matrix. If an IV's line crosses another IV's line, then a conditional dominance relationship between those two IVs cannot be determined. A graphic depicting the trajectory of the three IVs in the focal model is depicted below.

```{r condit_gph, echo=FALSE}
lm_da |> pluck("Conditional_Dominance") |> as_tibble(rownames = "pred") |> pivot_longer(names_to = "ivs", values_to = "stat", cols = starts_with("Inclu")) |> mutate(ivs = fct_relabel(ivs, ~ str_replace(., "_", ": "))) |> ggplot(aes(x = ivs, y = stat, group = pred, color= pred)) + geom_line() + ylab("Conditional Dominance Statistic Value") + xlab("Number of Independent Variables") + labs(color = "Independent\nVariable") + theme_linedraw() + scale_color_viridis_d() 
```

The graph above confirms that all three IV's lines never cross and thus have a clear set of conditional dominance designations.

As was mentioned in the **Concepts in Application** section, because we knew all three IVs have complete dominance designations relative to one another, they necessarily also had conditional dominance designations relative to one another.

### Computing General Dominance

General dominance is computed as:

$$C_{X_v} = \Sigma^p_{i=1}{\frac{C^i_{X_v}}{p}}$$

Where, $C^{i}_{X_x}$ are the conditional dominance statistics for $X_v$ with $i$ IVs. Hence, the general dominance statistics are the arithmetic average of all the conditional dominance statistics for an IV.

When applied to *am*'s results, the general dominance statistic value is:

```{r gen_am, echo=FALSE}
knitr::kable(data.frame(t(as.data.frame(lm_da$Conditional_Dominance["am",])), `general dominance` = lm_da$General_Dominance[["am"]], check.names = FALSE), row.names = FALSE, caption = "General Dominance Computations: `am`", digits = 3)
```

Next to *cyl*.

```{r gen_cyl, echo=FALSE}
knitr::kable(data.frame(t(as.data.frame(lm_da$Conditional_Dominance["cyl",])), `general dominance` = lm_da$General_Dominance[["cyl"]], check.names = FALSE), row.names = FALSE, caption = "General Dominance Computations: `cyl`", digits = 3)
```

And lastly *carb*.

```{r gen_carb, echo=FALSE}
knitr::kable(data.frame(t(as.data.frame(lm_da$Conditional_Dominance["carb",])), `general dominance` = lm_da$General_Dominance[["carb"]], check.names = FALSE), row.names = FALSE, caption = "General Dominance Computations: `carb`", digits = 3)
```

Taken as a set, these values represent the general dominance statistic/Shapley value decomposition vector:

```{r lm_general}
lm_da$General_Dominance
```

The general dominance statistic vector can also be used in a way similar to that of the complete and conditional dominance designations by ranking each value.

```{r gen_rank, echo = FALSE}
knitr::kable(data.frame(IV = names(lm_da$General_Dominance), `general dominance` = lm_da$General_Dominance, ranks = lm_da$Ranks, check.names = FALSE), row.names = FALSE, caption = "General Dominance Designations", digits = 3)
```

The rank ordering above shows that *am* is generally dominated by *cyl*, *am* generally dominates *carb*, and *cyl* also generally dominates *carb*.

Again because we knew all three IVs have complete and conditional dominance designations relative to one another, they necessarily also had general dominance designations relative to one another.

#### General Dominance/Shapley Values: Weighted Average of All Submodels

It is also worth pointing out a subtle feature of the general dominance statistics that tends to be more explicit discussions about the Shapley value decomposition. This feature is that each general dominance statistic is a weighted average of **all** $2^p$ fit statistics.

To see how this is the case, first recall the computations related to obtaining conditional dominance statistics for the *am* IV. If you look at all the entries in the three tables, all 8 models are included either as a minuend or a subtrahend. The general dominance statistics are then just an average of these three conditional dominance statistics. Hence, the general dominance statistics include the value for **every** model.

The conditional dominance statistics for *cyl* and *carb* re-arrange these same models but otherwise use the same information to produce their general dominance statistics.

## Strongest Dominance Designations

Whereas all dominance designations have been made in the example above, the strongest designation between two IVs is likely of primary interest as the strongest designation, as is noted above, implies all weaker designations.

To access the strongest dominance designations, the DA object can be submitted to the `summary` function.

```{r lm_strongest}
summary(lm_da)$Strongest_Dominance
```

The result the `summary` function produces in the *Strongest_Dominance* element is consistent with expectation in that all three IV interrelationships have complete dominance designations between them.

# Parting Thoughts

"Relative importance" as a concept is used in many different ways in statistics and data science. In the author's view, a crucial, but rarely acknowledged, difference between DA and many of the relative importance statistics produced by methods other than DA, are that many other methods are probably most useful for model selection and not for model evaluation. In making a distinction between model selection and importance, I follow the work of @azen2001criticality who distinguish between the concept of IV criticality and IV importance.

## Criticality: Model Selection

In many cases, methods that focus on relative importance are probably best used for model selection. When applied to model selection, a method would identify when an IV should be included in the model or not. The process of determining whether or not an IV should be included in the model is described by Azen et al. as reflecting *IV Criticality*.

In the view of the author, methods such as posterior inclusion probability, Akaike weights, and permutation importance are actually IV criticality, as opposed to importance, measures. These methods are criticality methods as they tend to be informative for identifying whether an IV has trivial or non-trivial contribution to prediction but is less informative for identifying the magnitude of their contribution.

## Importance: Model Evaluation

Model evaluation differs from model selection in that it seeks not to determine whether IVs should be in the model, but rather compares them in terms of their impact in the model conditional on their being included. Thus, importance methods assume that a model has passed through a model selection phase and that all the predictors in the model have non-trivial effects.

The DA method implemented by the `domir` function is an importance method in this sense in that it assumes that the predictive model used is "pre-selected" or has passed through model selection procedures and the user is confident that the IVs model and are, in fact, reasonable to include. The results from DA and similar methods then provide more information about relative contribution to prediction which assists in model evaluation.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/domir.r
\name{domir}
\alias{domir}
\alias{domir.formula}
\alias{domir.formula_list}
\title{Dominance analysis methods}
\usage{
domir(.obj, ...)

\method{domir}{formula}(
  .obj,
  .fct,
  .set = NULL,
  .wst = NULL,
  .all = NULL,
  .adj = FALSE,
  .cdl = NULL,
  .cpt = NULL,
  .rev = FALSE,
  .cst = NULL,
  .prg = FALSE,
  ...
)

\method{domir}{formula_list}(
  .obj,
  .fct,
  .set = NULL,
  .wst = NULL,
  .all = NULL,
  .adj = FALSE,
  .cdl = NULL,
  .cpt = NULL,
  .rev = FALSE,
  .cst = NULL,
  .prg = FALSE,
  ...
)
}
\arguments{
\item{.obj}{A \code{formula} or \code{formula_list}.
Parsed to produce a list of names. All valid combinations names from
the list are \code{\link{sapply}}-ed to \code{.fct}.

The combinations of names submitted to \code{.fct} are formatted to be of
the same \code{\link{class}} as \code{.obj} and must be submitted to
\code{.fct} as its first, unnamed argument.}

\item{...}{Passes arguments to other methods during method dispatch;
passes arguments to the function in \code{.fct} during function execution.}

\item{.fct}{A \code{\link{function}} or string function name.
Applied to all valid combinations of names parsed from \code{.obj}.
Must return a length 1/scalar, numeric, atomic vector.}

\item{.set}{A \code{list}.
Each element of the list must be the same class as \code{.obj}.
Elements of the list can be named.
Names parsed from the elements of the list must also be in \code{.obj}.}

\item{.wst}{A \code{list}.
Each element of the list must be the same class as \code{.obj}.
Names parsed from the elements of the list must also be in \code{.obj}.}

\item{.all}{A \code{formula} or \code{formula_list}.
Must be the same class as \code{.obj}.
Parsed names must also be in \code{.obj}.}

\item{.adj}{Logical.
If \code{TRUE} then a model including only an intercept is submitted to \code{.fct}
and the value returned is subtracted from the values returned from all
subsets in the dominance analysis.}

\item{.cdl}{\code{NULL}.
Depreciated. Use \code{.cdl = FALSE} in \code{domir}'s \code{print} method to suppress
display of conditional dominance values.}

\item{.cpt}{\code{NULL}.
Depreciated. Use \code{.cpt = FALSE} in \code{domir}'s \code{print} method to suppress
display of complete dominance values.}

\item{.rev}{Logical.
If \code{TRUE} then standardized vector, ranks, and complete dominance
designations are reversed in their interpretation.}

\item{.cst}{Object of class c("SOCKcluster", "cluster") from
\code{\link{parallel-package}}.

When non-\code{NULL}, will alter the method for collecting values from all
combinations of names from using \code{\link{sapply}} to \code{\link[parallel:clusterApply]{parallel::parSapply}}.}

\item{.prg}{Logical.
If \code{TRUE} then a progress bar is displayed during collection of values
to indicate progress.}
}
\value{
Returns an object of \code{\link{class}} "domir" composed of:
\describe{
\item{\code{General_Dominance}}{Vector of general dominance values.}
\item{\code{Standardized}}{Vector of general dominance values standardized
to sum to 1.}
\item{\code{Ranks}}{Vector of ranks applied to the general dominance values.}
\item{\code{Conditional_Dominance}}{Matrix of conditional dominance values.
Each row represents a name in \code{.obj};
each column represents a number of names included in \code{.fct}.}
\item{\code{Complete_Dominance}}{Matrix of proportions of subsets where the
name in the row has a larger value than the name in the column.
These proportions determine complete dominance when a value of
1 or 0.}
\item{\code{Value}}{Value returned by \code{.fct} with all names included.}
\item{\code{Value_All}}{Value of \code{.fct} associated with names included
in \code{.all};
when \code{.adj} is \code{TRUE}, this value will be adjusted for \code{Value_Adjust}.}
\item{\code{Value_Adjust}}{Value of \code{.fct} returned when no names are included.}
\item{\code{Call}}{The matched call.}
}
}
\description{
Parses input object to obtain a list of names, determines which
combinations of the names are valid, submits valid name
combinations to a function, and computes dominance
values based on the returned values from the function.
}
\details{
\subsection{Name Parsing}{

\code{.obj} is first parsed into a list of names.
How the name list is parsed depends on \code{.obj}'s class.
\subsection{\code{formula}}{

The \code{formula} method creates a name list using all terms in the formula.
The terms are obtained using \code{\link{terms.formula}}.
All processing that is normally applied to the right hand side of a
formula is implemented (see \code{\link{formula}}).

A response/left hand side is not required but, if present, is
included in all \code{formula}s passed to \code{.fct}.
}

\subsection{\code{formula_list}}{

The \code{\link{formula_list}} creates a name list out of response-term pairs.
The terms are obtained using \code{terms.formula} applied to each individual
formula in the list.

The \code{formula_list} methods make it possible to implement dominance analysis
for parameter estimates in multivariate predictive models
(e.g., Luchman, Xie, & Kaplan, 2020).
}

\subsection{Additional Details}{

\code{formula}s and \code{formula_list} elements are assumed to have an intercept
except if explicitly removed with a \code{- 1} in the \code{formula}(s) in \code{.obj}.
If removed, the intercept will be removed in all \code{formula}(s) in each
\code{sapply}-ed subset to \code{.fct}.
If \code{\link{offset}}s are included, they are passed, like intercepts, while
\code{sapply}-ing subsets to \code{.fct}.
}

}

\subsection{Changing Combination Generation}{

Parsed names are used independently for creating combinations of names
to submit to\code{.fct}.
When using \code{.set}, \code{.wst}, and \code{.all}, the way combinations are
created changes.
Names in \code{.set}, \code{.wst}, and \code{.all} must also be present in \code{.obj}.
\subsection{\code{.set}}{

\code{.set} binds together names such that they are considered to be one name
when creating combinations.
The names in each \code{.set} then pool their returned value together with the
other members of their set.

By default, sets are referred to by set names in all output.
Unnamed \code{.set} list elements are called 'set@' where '@' is an integer
indicating the set's position in the list.
The user can give each set in the \code{.set} list a name that will be used in
place of the default.
Names of \code{.set}s cannot contain the period character '.'.

\code{.set} implements the grouped dominance analysis method described by
Luchman (2026).
}

\subsection{\code{.wst}}{

\code{.wst} binds together names such that they are considered to be one name
when creating combinations with other names that are not in their \code{.wst}.
All combinations of names of the members of names within a \code{.wst}
are considered valid.
The names in each \code{.wst} then pool their returned value together with the
other members of their set when compared to names outside of the \code{.wst}
but do not pool their returned value when compared to names within their
\code{.wst}.

\code{.wst} list elements cannot be named.

\code{.wst} implements the within-group dominance analysis method described by
Luchman (2026).
\code{.wst} and \code{.set} can be used together.
}

\subsection{\code{.all}}{

\code{.all} binds together names such that they are considered to be one name
when creating combinations and are always included first, before all
other names.
The names in \code{.all} will then be included in 'all' valid combinations
of names.
The names in \code{.all} are included after the combination with no names but
before any other combinations.
The value associated with \code{.all} is also removed from the
dominance analysis and is reported along with the overall value
including all names.

The \code{formula} method for \code{.all} does not allow the formula in \code{.all} to have
a left hand side.
}

\subsection{\code{.adj}}{

By default, \code{domir} assumes that the combination of no names has a
value of 0.
\code{.adj} indicates that the there is an 'adjustment' needed such that the
no names combination has a non-0 value and an intercept-only model should
be supplied to \code{.fct}.

The \code{formula} method will submit an intercept-only formula to \code{.fct}.
The \code{formula_list} method creates a separate, intercept-only subset for each
of the \code{formula}s in the list.
Both the \code{formula} and \code{formula_list} methods will respect the user's
removal of an intercept and or inclusion of an \code{offset}.
}

\subsection{Additional Details}{

All methods submit combinations of names as an object of the same class as
\code{.obj}.
A \code{formula} in \code{.obj} will submit all combinations of names as
\code{formula}s to \code{.fct}.
A \code{formula_list} in \code{.obj} will submit all
combinations of subsets of names as \code{formula_list}s to \code{.fct}.
In the case that \code{.fct} requires a different \code{class} (e.g.,
a character vector of names, a \code{\link[Formula:Formula]{Formula::Formula}} see \code{\link{fmllst2Fml}}) the
subsets of names will have to be processed in \code{.fct} to obtain the correct
\code{class}.

The \code{formula} or \code{formula_list} of names will be submitted to \code{.fct} as the
first, unnamed argument.
}

}

\subsection{\code{.fct} as Analysis Pipeline}{

\code{.fct} is expected to be a complete analysis pipeline that receives a
subset of names of the same \code{class} as \code{.obj} and uses these names in the
\code{class} as submitted to generate a returned value of the appropriate
type to dominance analyze.
At current, only atomic (i.e., non-\code{list}), numeric scalars (i.e.,
vectors of length 1) are allowed as returned values.
\code{domir} is designed for use with predictive models and assumes the returned
value is a scalar-valued fit statistic/metric.

The \code{.fct} argument is strict about names submitted and returned value
requirements for functions used.
A series of checks to ensure the submitted names and returned value adhere
to these requirements.
The checks include whether the \code{.obj} can be submitted to \code{.fct} without
producing an error and whether the returned value from \code{.fct} is a length 1,
atomic, numeric vector.
In most circumstances, the user will have to make their own named or
anonymous function to supply as \code{.fct} to satisfy the checks.
}
}
\section{Notes}{
Prior to version 1.1.0, the \code{formula} method allowed a \code{formula}
to be submitted to \code{.adj}.
Submitting any argument other than a logical is now defunct.

The \code{formula} and \code{formula_list} methods can be used to pass responses,
intercepts, and \code{offset}s to all combinations of names.
If the user seeks to include other model components integral to
estimation (i.e., a random effect term in \code{\link[lme4:glmer]{lme4::glmer()}}) include them as
\code{\link[=update.formula]{update}} to the submitted \code{formula} or \code{formula_list}
embedded in \code{.fct}.

Second-order or higher terms (i.e., interactions like \code{~ a*b}) are parsed
by default but not used differently from first-order terms for generating
valid combinations.
The values ascribed to such names may not be valid unless the user ensures
that second-order and higher term names are used appropriately in \code{.fct}.
}

\examples{
## Linear model returning r-square
lm_r2 <-
  function(fml, data) {
    lm_res <- lm(fml, data = data)
    summary(lm_res)[["r.squared"]]
 }

domir(mpg ~ am + vs + cyl, lm_r2, data = mtcars)

## Linear model including set
domir(
  mpg ~ am + vs + cyl + carb + gear + disp + wt,
  lm_r2,
  .set = list(~ carb + gear, ~ disp + wt),
  data = mtcars
)

## Multivariate regression with multivariate r-square and
## all subsets variable
if (requireNamespace("performance", quietly = TRUE)) {
  mlm_rxy <-
    function(fml, data) {
      mlm_res <- lm(fml, data = data)
      performance::r2_mlm(mlm_res)[["Symmetric Rxy"]]
    }

  domir(
    cbind(wt, mpg) ~ vs + cyl + am + carb,
    mlm_rxy,
    .all = ~ carb,
    data = mtcars
  )
}

## Named sets
domir(
  mpg ~ am + gear + cyl + vs + qsec + drat,
  lm_r2,
  data = mtcars,
  .set =
    list(
      trns = ~ am + gear,
      eng = ~ cyl + vs,
      misc = ~ qsec + drat
    )
)

## Linear model returning AIC
lm_aic <-
  function(fml, data) {
    lm_res <- lm(fml, data = data)
    AIC(lm_res)
 }

domir(
  mpg ~ am + carb + cyl,
  lm_aic,
  .adj = TRUE,
  .rev = TRUE,
  data = mtcars
 )

## 'systemfit' with 'formula_list' method returning AIC
if (requireNamespace("systemfit", quietly = TRUE)) {
  domir(
    formula_list(mpg ~ am + cyl + carb, qsec ~ wt + cyl + carb),
    function(fml) {
      res <- systemfit::systemfit(fml, data = mtcars)
      AIC(res)
    },
    .adj = TRUE, .rev = TRUE
  )
}

## within-set or within-group dominance analysis
domir(
  mpg ~ am + gear + cyl + vs + qsec + drat,
  lm_r2,
  data = mtcars,
  .wst =
    list(
      ~ am + gear,
      ~ cyl + vs,
      ~ qsec + drat
    )
)
}
\references{
\itemize{
\item Luchman, J. N., Lei, X., & Kaplan, S. (2020). Relative Importance
Analysis with Multivariate Models: Shifting the Focus from Independent
Variables to Parameter Estimates. Journal of Applied Structural Equation
Modeling, 4(2), 1-20. doi:https://doi.org/10.47263/JASEM.4(2)02
\item Luchman, J. N. (2026). Determining Relative Importance with
Independent Variable Groups: An Alternative Dominance Analysis Method.
Journal of Behavioral Data Science, 6(1), 1-26.
doi:https://doi.org/10.35566/jbds/luchman
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/domin.r
\name{domin}
\alias{domin}
\title{Dominance analysis supporting \code{formula}-based modeling functions}
\usage{
domin(
  formula_overall,
  reg,
  fitstat,
  sets = NULL,
  all = NULL,
  conditional = TRUE,
  complete = TRUE,
  consmodel = NULL,
  reverse = FALSE,
  ...
)
}
\arguments{
\item{formula_overall}{An object of class \code{\link{formula}} or that can be coerced to class \code{formula} for use in the modeling function in \code{reg}.  The \code{\link{terms}} on the right hand side of this formula are used as separate entries to the dominance analysis.

A valid \code{formula_overall} entry is necessary, even if only submitting entries in \code{sets}, to define a valid left hand side of the prediction equation (see examples).  The function called in \code{reg} must accept one or more responses on the left hand side.}

\item{reg}{A function implementing the predictive (or "reg"ression) model called.

String function names (e.g., "lm"), function names (e.g., \code{lm}), or anonymous functions (e.g., \code{function(x) lm(x)}) are acceptable entries.  This argument's contents are passed to \code{\link{do.call}} and thus any function call \code{do.call} would accept is valid.

The predictive model in \code{reg} must accept a \code{formula} object as its first argument or must be adapted to do so with a wrapper function.}

\item{fitstat}{List providing arguments to call a fit statistic extracting function (see details). The \code{fitstat} list must be of at least length two.

The first element of \code{fitstat} must be a function implementing the fit statistic extraction. String function names (e.g., "summary"), function names (e.g., \code{summary}), or anonymous functions (e.g., \code{function(x) summary(x)}) are acceptable entries. This element's contents are passed to \code{\link{do.call}} and thus any function call \code{do.call} would accept is valid.

The second element of \code{fitstat} must be the named element of the list or vector produced by the fit extractor function called in the first element of \code{fitstat}.  This element must be a string (e.g., "r.squared").

All list elements beyond the second are submitted as additional arguments to the fit extractor function call.

The fit statistic extractor function in the first list element of \code{fitstat} must accept the model object produced by the predictive modeling function in \code{reg} as its first argument or be adapted to do so with a wrapper function.

The fit statistic produced must be scalar valued (i.e., vector of length 1).}

\item{sets}{A list with each element comprised of vectors containing variable/factor names or \code{formula} coercible strings.

Each separate list element-vector in \code{sets} is concatenated (when the list element-vector is of length > 1) and used as an entry to the dominance analysis along with the terms in \code{formula_overall}.}

\item{all}{A vector of variable/factor names or \code{formula} coercible strings.  The entries in this vector are concatenated (when of length > 1) but are not used in the dominance analysis.  Rather the value of the fit statistic associated with these terms is removed from the dominance analysis; this vector is used like a set of covariates.

The entries in \code{all} are removed from and considered an additional component that explains the fit metric.  As a result, the general dominance statistics will no longer sum to the overall fit metric and the standardized vector will no longer sum to 1.}

\item{conditional}{Logical.  If \code{FALSE} then conditional dominance matrix is not computed.

If conditional dominance is not desired as an importance criterion, avoiding computing the conditional dominance matrix can save computation time.}

\item{complete}{Logical.  If \code{FALSE} then complete dominance matrix is not computed.

If complete dominance is not desired as an importance criterion, avoiding computing complete dominance designations can save computation time.}

\item{consmodel}{A vector of variable/factor names, \code{formula} coercible strings, or other formula terms (i.e., 1 to indicate an intercept).  The entries in this vector are concatenated (when of length > 1) and, like the entries of \code{all}, are not used in the dominance analysis; this vector is used as an adjustment to the baseline value of the overall fit statistic.

The use of \code{consmodel} changes the interpretation of the the general and conditional dominance statistics.  When \code{consmodel} is used, the general and conditional dominance statistics are reflect the difference between the constant model and the overall fit statistic values.

Typical usage of \code{consmodel} is to pass "1" to set the intercept as the baseline and control for its value when the baseline model's fit statistic value is not 0 (e.g., if using the AIC or BIC as a fit statistic; see examples).

As such, this vector is used to set a baseline for the fit statistic when it is non-0.}

\item{reverse}{Logical. If \code{TRUE} then standardized vector, ranks, and complete dominance designations are reversed in their interpretation.

This argument should be changed to \code{TRUE} if the fit statistic used decreases with better fit to the data (e.g., AIC, BIC).}

\item{...}{Additional arguments passed to the function call in the \code{reg} argument.}
}
\value{
Returns an object of \code{\link{class}} "domin".
An object of class "domin" is a list composed of the following elements:
\describe{
\item{\code{General_Dominance}}{Vector of general dominance statistics.}
\item{\code{Standardized}}{Vector of general dominance statistics normalized to sum to 1.}
\item{\code{Ranks}}{Vector of ranks applied to the general dominance statistics.}
\item{\code{Conditional_Dominance}}{Matrix of conditional dominance statistics.  Each row represents a term; each column represents an order of terms.}
\item{\code{Complete_Dominance}}{Logical matrix of complete dominance designations. The term represented in each row indicates dominance status; the terms represented in each columns indicates dominated-by status.}
\item{\code{Fit_Statistic_Overall}}{Value of fit statistic for the full model.}
\item{\code{Fit_Statistic_All_Subsets}}{Value of fit statistic associated with terms in \code{all}.}
\item{\code{Fit_Statistic_Constant_Model}}{Value of fit statistic associated with terms in \code{consmodel}.}
\item{\code{Call}}{The matched call.}
\item{\code{Subset_Details}}{List containing the full model and descriptions of terms in the full model by source.}
}
}
\description{
Computes dominance statistics for predictive modeling functions that accept a \code{\link{formula}}.
}
\details{
\code{domin} automates the computation of all possible combination of entries to the dominance analysis (DA), the creation of \code{formula} objects based on those entries, the modeling calls/fit statistic capture, and the computation of all the dominance statistics for the user.

\code{domin} accepts only a "deconstructed" set of inputs and "reconstructs" them prior to formulating a coherent predictive modeling call.

One specific instance of this deconstruction is in generating the number of entries to the DA. The number of entries is taken as all the \code{terms} from \code{formula_overall} and the separate list element vectors from \code{sets}. The entries themselves are concatenated into a single formula, combined with the entries in \code{all}, and submitted to the predictive modeling function in \code{reg}.  Each different combination of entries to the DA forms a different \code{formula} and thus a different model to estimate.

For example, consider this \code{domin} call:

\code{domin(y ~ x1 + x2, lm, list(summary, "r.squared"), sets = list(c("x3", "x4")), all = c("c1", "c2"), data = mydata))}

This call records three entries and results in seven (i.e., \eqn{2^3 - 1}) different combinations:

\enumerate{
\item x1
\item x2
\item x3, x4
\item x1, x2
\item x1, x3, x4
\item x2, x3, x4
\item x1, x2, x3, x4
}

\code{domin} parses \code{formula_overall} to obtain all the terms in it and combines them with \code{sets}.  When parsing \code{formula_overall}, only the processing that is available in the \code{stats} package is applied.  Note that \code{domin} is not programmed to process terms of order > 1 (i.e., interactions/products) appropriately (i.e., only include in the presence of lower order component terms). \code{domin} also does not allow \code{offset} terms.

From these combinations, the predictive models are constructed and called. The predictive model call includes the entries in \code{all}, applies the appropriate formula, and reconstructs the function itself. The seven combinations above imply the following series of predictive model calls:

\enumerate{
\item \code{lm(y ~ x1 + c1 + c2, data = mydata})
\item \code{lm(y ~ x2 + c1 + c2, data = mydata})
\item \code{lm(y ~ x3 + x4 + c1 + c2, data = mydata})
\item \code{lm(y ~ x1 + x2 + c1 + c2, data = mydata})
\item \code{lm(y ~ x1 + x3 + x4 + c1 + c2, data = mydata})
\item \code{lm(y ~ x2 + x3 + x4 + c1 + c2, data = mydata})
\item \code{lm(y ~ x1 + x2 + x3 + x4 + c1 + c2, data = mydata})
}

It is possible to use a \code{domin} with only sets (i.e., no IVs in \code{formula_overall}; see examples below). There must be at least two entries to the DA for \code{domin} to run.

All the called predictive models are submitted to the fit extractor function implied by the entries in \code{fitstat}. Again applying the example above, all seven predictive models' objects would be individually passed as follows:

\code{summary(lm_obj)["r.squared"]}

where \code{lm_obj} is the model object returned by \code{lm}.

The entries to \code{fitstat} must be as a list and follow a specific structure:
\code{list(fit_function, element_name, ...)}
\describe{
\item{\code{fit_function}}{First element and function to be applied to the object produced by the \code{reg} function}
\item{\code{element_name}}{Second element and name of the element from the object returned by \code{fit_function} to be used as a fit statistic.  The fit statistic must be scalar-valued/length 1}
\item{\code{...}}{Subsequent elements and are additional arguments passed to \code{fit_function}}
}

In the case that the model object returned by \code{reg} includes its own fit statistic without the need for an extractor function, the user can apply an anonymous function following the required format to extract it.
}
\section{Notes}{


\code{domin} is an R port of the Stata command with the same name (see Luchman, 2021).

\code{domin} has been superseded by \code{\link{domir}}.
}

\examples{
## Basic linear model with r-square

domin(mpg ~ am + vs + cyl, 
  lm, 
  list("summary", "r.squared"), 
  data = mtcars)


## Linear model including sets

domin(mpg ~ am + vs + cyl, 
  lm, 
  list("summary", "r.squared"), 
  data = mtcars, 
  sets = list(c("carb", "gear"), c("disp", "wt")))


## Multivariate linear model with custom multivariate r-square function 
## and all subsets variable

Rxy <- function(obj, names, data) {
   return(list("r2" = cancor(predict(obj), 
       as.data.frame(mget(names, as.environment(data))))[["cor"]][1]^2)) 
       }
       
domin(cbind(wt, mpg) ~ vs + cyl + am, 
  lm, 
  list(Rxy, "r2", c("mpg", "wt"), mtcars), 
  data = mtcars, 
  all = c("carb"))


## Sets only

domin(mpg ~ 1, 
  lm, 
  list("summary", "r.squared"), 
  data = mtcars, 
  sets = list(c("am", "vs"), c("cyl", "disp"), c("qsec", "carb")))
  
## Constant model using AIC

domin(mpg ~ am + carb + cyl, 
  lm, 
  list(function(x) list(aic = extractAIC(x)[[2]]), "aic"), 
  data = mtcars, 
  reverse = TRUE, consmodel = "1")
}
\references{
Luchman, J. N. (2021). Relative importance analysis in Stata using dominance analysis: domin and domme.
The Stata Journal, 21, 2. doi: 10.1177/1536867X211025837.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/domin.r
\name{print.domin}
\alias{print.domin}
\title{Print method for \code{domin}}
\usage{
\method{print}{domin}(x, ...)
}
\arguments{
\item{x}{an object of class "domin".}

\item{...}{further arguments passed to or from other methods. Not used currently.}
}
\value{
The "domin" object with altered column and row names for conditional and complete dominance results as displayed in the console.
}
\description{
Reports formatted results from \code{domin} class object.
}
\details{
The print method for class \code{domin} objects reports out the following results:
\itemize{
\item{Fit statistic for the full model.  The fit statistic for the all subsets model is reported here if there are any entries in \code{all}.  The fit statistic for the constant model is reported here if there are any entries in \code{consmodel}.}
\item{Matrix describing general dominance statistics, standardized general dominance statistics, and the ranking of the general dominance statistics}
\item{If \code{conditional} is \code{TRUE}, matrix describing the conditional dominance designations}
\item{If \code{complete} is \code{TRUE}, matrix describing the complete dominance designations}
\item{If following \code{summary.domin}, matrix describing the strongest dominance designations between all independent variables}
\item{If there are entries in \code{sets} and/or \code{all} the terms included in each set as well as the terms in all subsets are reported}}
The \code{domin} print method alters dimension names for readability and they do not display as stored in the original \code{domin} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/domir.r
\name{print.domir}
\alias{print.domir}
\title{Print method for \code{domir}}
\usage{
\method{print}{domir}(x, .cdl = TRUE, .cpt = TRUE, ...)
}
\arguments{
\item{x}{an object of class "domir".}

\item{.cdl}{Logical.
If \code{TRUE} then conditional dominance statistics will be reported.}

\item{.cpt}{Logical.
If \code{TRUE} then complete dominance proportions will be reported.}

\item{...}{further arguments passed to \code{\link{print.default}}.}
}
\value{
The submitted "domir" object, invisibly.
}
\description{
Reports formatted results from \code{domir} class object.
}
\details{
The print method for class \code{domir} objects reports out the
following results:
\itemize{
\item{Value when all elements are included in \code{obj}.}
\item{Value for the elements included in \code{.all}, if any.}
\item{Value for the elements included in \code{.adj}, if any.}
\item{Matrix describing general dominance values, standardized
general dominance values, and the ranking of the general
dominance values.}
\item{Matrix describing the conditional dominance values
if \code{.cdl} is \code{TRUE}.}
\item{Matrix describing the complete dominance designations
if \code{.cpt} is \code{TRUE}.}
\item{If following \code{\link{summary.domir}}, matrix describing the strongest
dominance designations between all elements if both \code{.cdl} and \code{.cpt}
are \code{TRUE}.}}

The \code{domir} print method alters dimension names for readability and they
do not display as stored in the \code{domir} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula_list.R
\name{fmllst2Fml}
\alias{fmllst2Fml}
\title{Translate \code{formula_list} into \code{Formula::Formula}}
\usage{
fmllst2Fml(fmllst, drop_lhs = NULL)
}
\arguments{
\item{fmllst}{A \code{formula_list} classed object.}

\item{drop_lhs}{An integer vector.

Used as a selection vector to remove response names prior to
generating the \code{Formula} object. This vector must be composed of
integers (e.g., 1L and not 1).

This is useful for some \code{Formulas} that do not have a separate
responses for each left hand side model part (e.g., \code{\link[pscl:zeroinfl]{pscl::zeroinfl}}) but
are required to have separate responses by \code{formula_list}.}
}
\value{
A \code{Formula::Formula} object.
}
\description{
Translates \code{\link{formula_list}} objects into a \code{\link[Formula:Formula]{Formula::Formula}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/domir-package.r
\name{domir-package}
\alias{domir-package}
\title{Dominance Analysis Methods}
\description{
Dominance analysis methods intended for predictive modeling functions.
}
\details{
This package implements several methods to compute dominance analysis
(Azen & Budescu, 2004; Budescu, 1993). Dominance analysis is a relative
importance analysis approach that derives conceptually from Shapley values
(e.g.,  Grömping, 200;  Lipovsky & Conklin, 2001) in that it ascribes
'values' from some function to inputs (known as 'names' in the package)
to that function.

When applied to predictive models, the method compares components of a
fit metric ascribed to each 'name' (i.e., independent variable,
predictor, feature, or parameter estimate) to each other 'name' in a
pairwise fashion to determine a hierarchy of dominance or
relative importance.
}
\references{
\itemize{
\item Azen, R., & Budescu, D. V. (2003). The dominance analysis approach
for comparing predictors in multiple regression. Psychological Methods,
8(2), 129-148. doi:10.1037/1082-989X.8.2.129
\item Budescu, D. V. (1993). Dominance analysis: A new approach to the
problem of relative importance of predictors in multiple regression.
Psychological Bulletin, 114(3), 542-551. doi:10.1037/0033-2909.114.3.542
\item Grömping, U. (2007). Estimators of relative importance in linear
regression based on variance decomposition. The American Statistician,
61(2), 139-147. doi:10.1198/000313007X188252
\item Lipovetsky, S, & and Conklin, M. (2001). Analysis of regression in
game theory approach. Applied Stochastic Models in Business and Industry,
17(4), 319-330. doi:10.1002/asmb.446
}
}
\author{
Joseph Luchman \email{jluchman@gmail.com}
}
\keyword{"_PACKAGE"}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula_list.R
\name{formula_list}
\alias{formula_list}
\title{A \code{\link{list}} composed of \code{formulas}}
\usage{
formula_list(...)
}
\arguments{
\item{...}{\code{formula}s, possibly named}
}
\value{
A \code{list} of class \code{formula_list}.
}
\description{
Defines a list object composed of \code{formula}s. The purpose of this
class of object is to impose structure of the list to ensure that it
can be used to obtain response-term pairs and will be able to be
parsed in \code{\link{domir}}.
}
\details{
The \code{formula_list} requires that each element of the list is a \code{formula}
and that each \code{formula} is unique with a different, non-\code{NULL}
dependent variable/response.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/domir.r
\name{summary.domir}
\alias{summary.domir}
\title{Summary method for \code{domir}}
\usage{
\method{summary}{domir}(object, ...)
}
\arguments{
\item{object}{an object of class "domir".}

\item{...}{further arguments passed to or from other methods.
Not used currently.}
}
\value{
The submitted "domir" object with an additional
\code{Strongest_Dominance} element added.
\describe{
\item{\code{Strongest_Dominance}}{Matrix comparing the element in the first
row to the element in the third row.  The second row denotes the strongest
designation between the two names.}
}
}
\description{
Reports dominance designation results from the \code{domir}
class object.
}
\details{
The summary method for class \code{domir} objects is used for obtaining
the strongest dominance designations (i.e., general, conditional, or
complete) among all pairs of dominance analyzed names.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/domin.r
\name{summary.domin}
\alias{summary.domin}
\title{Summary method for \code{domin}}
\usage{
\method{summary}{domin}(object, ...)
}
\arguments{
\item{object}{an object of class "domin".}

\item{...}{further arguments passed to or from other methods. Not used currently.}
}
\value{
The originally submitted "domin" object with an additional \code{Strongest_Dominance} element added.
\describe{
\item{\code{Strongest_Dominance}}{Matrix comparing the independent variable in the first row to the independent variable in the third row.  The second row denotes the strongest designation between the two independent variables.}
}
}
\description{
Reports dominance designation results from the \code{domin} class object.
}
\details{
The summary method for class \code{domin} is used for obtaining the strongest dominance designations (i.e., general, conditional, or complete) among the independent variables.
}
